@@ -1,6 +1,7 @@
 #include "converter.hpp"
 
 #include <stdexcept>
+#include <algorithm>
 
 #include <osgDB/WriteFile>
 
@@ -53,6 +54,36 @@ namespace
             return true;
         return false;
     }
+
+    void splitIndexedRefId(const std::string& indexedRefId, int& refIndex, std::string& refId)
+    {
+        std::stringstream stream;
+        stream << std::hex << indexedRefId.substr(indexedRefId.size()-8,8);
+        stream >> refIndex;
+
+        refId = indexedRefId.substr(0,indexedRefId.size()-8);
+    }
+
+    int convertActorId(const std::string& indexedRefId, ESSImport::Context& context)
+    {
+        if (isIndexedRefId(indexedRefId))
+        {
+            int refIndex;
+            std::string refId;
+            splitIndexedRefId(indexedRefId, refIndex, refId);
+
+            auto it = context.mActorIdMap.find(std::make_pair(refIndex, refId));
+            if (it == context.mActorIdMap.end())
+                return -1;
+            return it->second;
+        }
+        else if (indexedRefId == "PlayerSaveGame")
+        {
+            return context.mPlayer.mObject.mCreatureStats.mActorId;
+        }
+
+        return -1;
+    }
 }
 
 namespace ESSImport
@@ -137,7 +168,7 @@ namespace ESSImport
         osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("png");
         if (!readerwriter)
         {
-            std::cerr << "can't write global map image, no png readerwriter found" << std::endl;
+            std::cerr << "Error: can't write global map image, no png readerwriter found" << std::endl;
             return;
         }
 
@@ -146,7 +177,7 @@ namespace ESSImport
         osgDB::ReaderWriter::WriteResult result = readerwriter->writeImage(*image2, ostream);
         if (!result.success())
         {
-            std::cerr << "can't write global map image: " << result.message() << " code " << result.status() << std::endl;
+            std::cerr << "Error: can't write global map image: " << result.message() << " code " << result.status() << std::endl;
             return;
         }
 
@@ -322,12 +353,9 @@ namespace ESSImport
             }
             else
             {
-                std::stringstream stream;
-                stream << std::hex << cellref.mIndexedRefId.substr(cellref.mIndexedRefId.size()-8,8);
                 int refIndex;
-                stream >> refIndex;
+                splitIndexedRefId(cellref.mIndexedRefId, refIndex, out.mRefID);
 
-                out.mRefID = cellref.mIndexedRefId.substr(0,cellref.mIndexedRefId.size()-8);
                 std::string idLower = Misc::StringUtils::lowerCase(out.mRefID);
 
                 std::map<std::pair<int, std::string>, NPCC>::const_iterator npccIt = mContext->mNpcChanges.find(
@@ -347,6 +375,10 @@ namespace ESSImport
                     convertNpcData(cellref, objstate.mNpcStats);
                     convertNPCC(npccIt->second, objstate);
                     convertCellRef(cellref, objstate);
+
+                    objstate.mCreatureStats.mActorId = mContext->generateActorId();
+                    mContext->mActorIdMap.insert(std::make_pair(std::make_pair(refIndex, out.mRefID), objstate.mCreatureStats.mActorId));
+
                     esm.writeHNT ("OBJE", ESM::REC_NPC_);
                     objstate.save(esm);
                     continue;
@@ -383,6 +415,10 @@ namespace ESSImport
                         convertACSC(cellref.mACSC, objstate.mCreatureStats);
                     convertCREC(crecIt->second, objstate);
                     convertCellRef(cellref, objstate);
+
+                    objstate.mCreatureStats.mActorId = mContext->generateActorId();
+                    mContext->mActorIdMap.insert(std::make_pair(std::make_pair(refIndex, out.mRefID), objstate.mCreatureStats.mActorId));
+
                     esm.writeHNT ("OBJE", ESM::REC_CREA);
                     objstate.save(esm);
                     continue;
@@ -413,4 +449,73 @@ namespace ESSImport
         }
     }
 
+    void ConvertPROJ::read(ESM::ESMReader& esm)
+    {
+        mProj.load(esm);
+    }
+
+    void ConvertPROJ::write(ESM::ESMWriter& esm)
+    {
+        for (const PROJ::PNAM& pnam : mProj.mProjectiles)
+        {
+            if (!pnam.isMagic())
+            {
+                ESM::ProjectileState out;
+                convertBaseState(out, pnam);
+
+                out.mBowId = pnam.mBowId.toString();
+                out.mVelocity = pnam.mVelocity;
+                out.mAttackStrength = pnam.mAttackStrength;
+
+                esm.startRecord(ESM::REC_PROJ);
+                out.save(esm);
+                esm.endRecord(ESM::REC_PROJ);
+            }
+            else
+            {
+                ESM::MagicBoltState out;
+                convertBaseState(out, pnam);
+
+                auto it = std::find_if(mContext->mActiveSpells.begin(), mContext->mActiveSpells.end(),
+                                       [&pnam](const SPLM::ActiveSpell& spell) -> bool { return spell.mIndex == pnam.mSplmIndex; });
+
+                if (it == mContext->mActiveSpells.end())
+                {
+                    std::cerr << "Warning: Skipped conversion for magic projectile \"" << pnam.mArrowId.toString() << "\" (invalid spell link)" << std::endl;
+                    continue;
+                }
+
+                out.mSpellId = it->mSPDT.mId.toString();
+                out.mSpeed = pnam.mSpeed * 0.001f; // not sure where this factor comes from
+
+                esm.startRecord(ESM::REC_MPRJ);
+                out.save(esm);
+                esm.endRecord(ESM::REC_MPRJ);
+            }
+        }
+    }
+
+    void ConvertPROJ::convertBaseState(ESM::BaseProjectileState& base, const PROJ::PNAM& pnam)
+    {
+        base.mId = pnam.mArrowId.toString();
+        base.mPosition = pnam.mPosition;
+
+        osg::Quat orient;
+        orient.makeRotate(osg::Vec3f(0,1,0), pnam.mVelocity);
+        base.mOrientation = orient;
+
+        base.mActorId = convertActorId(pnam.mActorId.toString(), *mContext);
+    }
+
+    void ConvertSPLM::read(ESM::ESMReader& esm)
+    {
+        mSPLM.load(esm);
+        mContext->mActiveSpells = mSPLM.mActiveSpells;
+    }
+
+    void ConvertSPLM::write(ESM::ESMWriter& esm)
+    {
+        std::cerr << "Warning: Skipped active spell conversion (not implemented)" << std::endl;
+    }
+
 }
