@@ -1,6 +1,6 @@
 #include "messages.hpp"
 
-CSMDoc::Message::Message() {}
+CSMDoc::Message::Message() : mSeverity(Severity_Default){}
 
 CSMDoc::Message::Message (const CSMWorld::UniversalId& id, const std::string& message,
     const std::string& hint, Severity severity)
