@@ -152,6 +152,8 @@ void CSMPrefs::State::declare()
         setRange (0, 10000);
     declareInt ("error-height", "Initial height of the error panel", 100).
         setRange (100, 10000);
+    declareBool ("highlight-occurrences", "Highlight other occurrences of selected names", true);
+    declareColour ("colour-highlight", "Colour of highlighted occurrences", QColor("lightcyan"));
     declareSeparator();
     declareColour ("colour-int", "Highlight Colour: Integer Literals", QColor ("darkmagenta"));
     declareColour ("colour-float", "Highlight Colour: Float Literals", QColor ("magenta"));
@@ -238,6 +240,8 @@ void CSMPrefs::State::declare()
     declareShortcut ("document-world-cells", "Open Cell List", QKeySequence());
     declareShortcut ("document-world-referencables", "Open Object List", QKeySequence());
     declareShortcut ("document-world-references", "Open Instance List", QKeySequence());
+    declareShortcut ("document-world-lands", "Open Lands List", QKeySequence());
+    declareShortcut ("document-world-landtextures", "Open Land Textures List", QKeySequence());
     declareShortcut ("document-world-pathgrid", "Open Pathgrid List", QKeySequence());
     declareShortcut ("document-world-regionmap", "Open Region Map", QKeySequence());
     declareShortcut ("document-mechanics-globals", "Open Global List", QKeySequence());
@@ -257,6 +261,7 @@ void CSMPrefs::State::declare()
     declareShortcut ("document-character-topicinfos", "Open Topic Info List", QKeySequence());
     declareShortcut ("document-character-journalinfos", "Open Journal Info List", QKeySequence());
     declareShortcut ("document-character-bodyparts", "Open Body Part List", QKeySequence());
+    declareShortcut ("document-assets-reload", "Reload Assets", QKeySequence(Qt::Key_F5));
     declareShortcut ("document-assets-sounds", "Open Sound Asset List", QKeySequence());
     declareShortcut ("document-assets-soundgens", "Open Sound Generator List", QKeySequence());
     declareShortcut ("document-assets-meshes", "Open Mesh Asset List", QKeySequence());
@@ -273,6 +278,7 @@ void CSMPrefs::State::declare()
     declareShortcut ("table-edit", "Edit Record", QKeySequence());
     declareShortcut ("table-add", "Add Row/Record", QKeySequence(Qt::ShiftModifier | Qt::Key_A));
     declareShortcut ("table-clone", "Clone Record", QKeySequence(Qt::ShiftModifier | Qt::Key_D));
+    declareShortcut ("touch-record", "Touch Record", QKeySequence());
     declareShortcut ("table-revert", "Revert Record", QKeySequence());
     declareShortcut ("table-remove", "Remove Row/Record", QKeySequence(Qt::Key_Delete));
     declareShortcut ("table-moveup", "Move Record Up", QKeySequence());
@@ -326,6 +332,10 @@ void CSMPrefs::State::declare()
     declareShortcut ("orbit-roll-right", "Roll Right", QKeySequence(Qt::Key_E));
     declareShortcut ("orbit-speed-mode", "Toggle Speed Mode", QKeySequence(Qt::Key_F));
     declareShortcut ("orbit-center-selection", "Center On Selected", QKeySequence(Qt::Key_C));
+
+    declareSubcategory ("Script Editor");
+    declareShortcut ("script-editor-comment", "Comment Selection", QKeySequence());
+    declareShortcut ("script-editor-uncomment", "Uncomment Selection", QKeySequence());
 }
 
 void CSMPrefs::State::declareCategory (const std::string& key)
@@ -581,6 +591,41 @@ CSMPrefs::State& CSMPrefs::State::get()
     return *sThis;
 }
 
+void CSMPrefs::State::resetCategory(const std::string& category)
+{
+    for (Settings::CategorySettingValueMap::iterator i = mSettings.mUserSettings.begin();
+         i != mSettings.mUserSettings.end(); ++i)
+    {
+        // if the category matches
+        if (i->first.first == category)
+        {
+            // mark the setting as changed
+            mSettings.mChangedSettings.insert(std::make_pair(i->first.first, i->first.second));
+            // reset the value to the default
+            i->second = mSettings.mDefaultSettings[i->first];
+        }
+    }
+
+    Collection::iterator container = mCategories.find(category);
+    if (container != mCategories.end())
+    {
+        Category settings = container->second;
+        for (Category::Iterator i = settings.begin(); i != settings.end(); ++i)
+        {
+            (*i)->updateWidget();
+            update(**i);
+        }
+    }
+}
+
+void CSMPrefs::State::resetAll()
+{
+    for (Collection::iterator iter = mCategories.begin(); iter != mCategories.end(); ++iter)
+    {
+        resetCategory(iter->first);
+    }
+}
+
 
 CSMPrefs::State& CSMPrefs::get()
 {
