@@ -29,9 +29,16 @@ void CSMTools::RegionCheckStage::perform (int stage, CSMDoc::Messages& messages)
 
     // test for empty name
     if (region.mName.empty())
-        messages.push_back (std::make_pair (id, region.mId + " has an empty name"));
+        messages.add(id, region.mId + " has an empty name", "", CSMDoc::Message::Severity_Error);
 
     /// \todo test that the ID in mSleeplist exists
 
+    // test that chances add up to 100
+    int chances = region.mData.mClear + region.mData.mCloudy + region.mData.mFoggy + region.mData.mOvercast +
+        region.mData.mRain + region.mData.mThunder + region.mData.mAsh + region.mData.mBlight +
+        region.mData.mA + region.mData.mB;
+    if (chances != 100)
+        messages.add(id, "Weather chances do not add up to 100", "", CSMDoc::Message::Severity_Error);
+
     /// \todo check data members that can't be edited in the table view
 }
