@@ -2,12 +2,12 @@
 #define CSM_WOLRD_COLUMNIMP_H
 
 #include <cassert>
+#include <cstdint>
 #include <sstream>
 #include <stdexcept>
 
-#include <boost/lexical_cast.hpp>
-
 #include <QColor>
+#include <QVector>
 
 #include <components/esm/loadbody.hpp>
 #include <components/esm/loadskil.hpp>
@@ -17,6 +17,9 @@
 #include "columns.hpp"
 #include "info.hpp"
 
+#include "land.hpp"
+#include "landtexture.hpp"
+
 namespace CSMWorld
 {
     /// \note Shares ID with VarValueColumn. A table can not have both.
@@ -62,6 +65,20 @@ namespace CSMWorld
         }
     };
 
+    template<>
+    inline QVariant StringIdColumn<Land>::get(const Record<Land>& record) const
+    {
+        const Land& land = record.get();
+        return QString::fromUtf8(Land::createUniqueRecordId(land.mX, land.mY).c_str());
+    }
+
+    template<>
+    inline QVariant StringIdColumn<LandTexture>::get(const Record<LandTexture>& record) const
+    {
+        const LandTexture& ltex = record.get();
+        return QString::fromUtf8(LandTexture::createUniqueRecordId(ltex.mPluginIndex, ltex.mIndex).c_str());
+    }
+
     template<typename ESXRecordT>
     struct RecordStateColumn : public Column<ESXRecordT>
     {
@@ -675,32 +692,23 @@ namespace CSMWorld
         }
     };
 
-    /// \todo QColor is a GUI class and should not be in model. Need to think of an alternative
-    /// solution.
     template<typename ESXRecordT>
     struct MapColourColumn : public Column<ESXRecordT>
     {
-        /// \todo Replace Display_Integer with something that displays the colour value more directly.
         MapColourColumn()
         : Column<ESXRecordT> (Columns::ColumnId_MapColour, ColumnBase::Display_Colour)
         {}
 
         virtual QVariant get (const Record<ESXRecordT>& record) const
         {
-            int colour = record.get().mMapColor;
-
-            return QColor (colour & 0xff, (colour>>8) & 0xff, (colour>>16) & 0xff);
+            return record.get().mMapColor;
         }
 
         virtual void set (Record<ESXRecordT>& record, const QVariant& data)
         {
-            ESXRecordT record2 = record.get();
-
-            QColor colour = data.value<QColor>();
-
-            record2.mMapColor = (colour.blue() << 16) | (colour.green() << 8) | colour.red();
-
-            record.setModified (record2);
+            ESXRecordT copy = record.get();
+            copy.mMapColor = data.toInt();
+            record.setModified (copy);
         }
 
         virtual bool isEditable() const
@@ -1501,9 +1509,9 @@ namespace CSMWorld
     template<typename ESXRecordT>
     struct TopicColumn : public Column<ESXRecordT>
     {
-        TopicColumn (bool journal) 
+        TopicColumn (bool journal)
         : Column<ESXRecordT> (journal ? Columns::ColumnId_Journal : Columns::ColumnId_Topic,
-                              journal ? ColumnBase::Display_Journal : ColumnBase::Display_Topic) 
+                              journal ? ColumnBase::Display_Journal : ColumnBase::Display_Topic)
         {}
 
         virtual QVariant get (const Record<ESXRecordT>& record) const
@@ -1758,6 +1766,41 @@ namespace CSMWorld
         }
     };
 
+    template<typename ESXRecordT>
+    struct GenderNpcColumn : public Column<ESXRecordT>
+    {
+        GenderNpcColumn()
+            : Column<ESXRecordT>(Columns::ColumnId_GenderNpc, ColumnBase::Display_GenderNpc)
+        {}
+
+        virtual QVariant get(const Record<ESXRecordT>& record) const
+        {
+            // Implemented this way to allow additional gender types in the future.
+            if ((record.get().mData.mFlags & ESM::BodyPart::BPF_Female) == ESM::BodyPart::BPF_Female)
+                return 1;
+
+            return 0;
+        }
+
+        virtual void set(Record<ESXRecordT>& record, const QVariant& data)
+        {
+            ESXRecordT record2 = record.get();
+
+            // Implemented this way to allow additional gender types in the future.
+            if (data.toInt() == 1)
+                record2.mData.mFlags = (record2.mData.mFlags & ~ESM::BodyPart::BPF_Female) | ESM::BodyPart::BPF_Female;
+            else
+                record2.mData.mFlags = record2.mData.mFlags & ~ESM::BodyPart::BPF_Female;
+
+            record.setModified(record2);
+        }
+
+        virtual bool isEditable() const
+        {
+            return true;
+        }
+    };
+
     template<typename ESXRecordT>
     struct EnchantmentTypeColumn : public Column<ESXRecordT>
     {
@@ -2165,8 +2208,8 @@ namespace CSMWorld
     struct EffectTextureColumn : public Column<ESXRecordT>
     {
         EffectTextureColumn (Columns::ColumnId columnId)
-        : Column<ESXRecordT> (columnId, 
-                              columnId == Columns::ColumnId_Particle ? ColumnBase::Display_Texture 
+        : Column<ESXRecordT> (columnId,
+                              columnId == Columns::ColumnId_Particle ? ColumnBase::Display_Texture
                                                                      : ColumnBase::Display_Icon)
         {
             assert (this->mColumnId==Columns::ColumnId_Icon ||
@@ -2384,7 +2427,95 @@ namespace CSMWorld
             return true;
         }
     };
-    
+
+    struct LandTextureNicknameColumn : public Column<LandTexture>
+    {
+        LandTextureNicknameColumn();
+
+        QVariant get(const Record<LandTexture>& record) const override;
+        void set(Record<LandTexture>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
+    struct LandTextureIndexColumn : public Column<LandTexture>
+    {
+        LandTextureIndexColumn();
+
+        QVariant get(const Record<LandTexture>& record) const override;
+        bool isEditable() const override;
+    };
+
+    struct LandPluginIndexColumn : public Column<Land>
+    {
+        LandPluginIndexColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        bool isEditable() const override;
+    };
+
+    struct LandTexturePluginIndexColumn : public Column<LandTexture>
+    {
+        LandTexturePluginIndexColumn();
+
+        QVariant get(const Record<LandTexture>& record) const override;
+        bool isEditable() const override;
+    };
+
+    struct LandMapLodColumn : public Column<Land>
+    {
+        using DataType = QVector<signed char>;
+
+        LandMapLodColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        void set(Record<Land>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
+    struct LandNormalsColumn : public Column<Land>
+    {
+        using DataType = QVector<signed char>;
+
+        LandNormalsColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        void set(Record<Land>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
+    struct LandHeightsColumn : public Column<Land>
+    {
+        using DataType = QVector<float>;
+
+        LandHeightsColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        void set(Record<Land>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
+    struct LandColoursColumn : public Column<Land>
+    {
+        using DataType = QVector<unsigned char>;
+
+        LandColoursColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        void set(Record<Land>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
+    struct LandTexturesColumn : public Column<Land>
+    {
+        using DataType = QVector<uint16_t>;
+
+        LandTexturesColumn();
+
+        QVariant get(const Record<Land>& record) const override;
+        void set(Record<Land>& record, const QVariant& data) override;
+        bool isEditable() const override;
+    };
+
     struct BodyPartRaceColumn : public RaceColumn<ESM::BodyPart>
     {
         const MeshTypeColumn<ESM::BodyPart> *mMeshType;
@@ -2397,4 +2528,11 @@ namespace CSMWorld
     };
 }
 
+// This is required to access the type as a QVariant.
+Q_DECLARE_METATYPE(CSMWorld::LandMapLodColumn::DataType)
+//Q_DECLARE_METATYPE(CSMWorld::LandNormalsColumn::DataType) // Same as LandMapLodColumn::DataType
+Q_DECLARE_METATYPE(CSMWorld::LandHeightsColumn::DataType)
+Q_DECLARE_METATYPE(CSMWorld::LandColoursColumn::DataType)
+Q_DECLARE_METATYPE(CSMWorld::LandTexturesColumn::DataType)
+
 #endif
