@@ -92,7 +92,7 @@ namespace CSMWorld
             ColumnId_ArmorType = 77,
             ColumnId_Health = 78,
             ColumnId_ArmorValue = 79,
-            ColumnId_Scroll = 80,
+            ColumnId_BookType = 80,
             ColumnId_ClothingType = 81,
             ColumnId_WeightCapacity = 82,
             ColumnId_OrganicContainer = 83,
@@ -107,8 +107,8 @@ namespace CSMWorld
             ColumnId_Flies = 92,
             ColumnId_Walks = 93,
             ColumnId_Essential = 94,
-            ColumnId_SkeletonBlood = 95,
-            ColumnId_MetalBlood = 96,
+            ColumnId_BloodType = 95,
+            // unused
             ColumnId_OpenSound = 97,
             ColumnId_CloseSound = 98,
             ColumnId_Duration = 99,
@@ -118,10 +118,8 @@ namespace CSMWorld
             ColumnId_Dynamic = 103,
             ColumnId_Portable = 104,
             ColumnId_NegativeLight = 105,
-            ColumnId_Flickering = 106,
-            ColumnId_SlowFlickering = 107,
-            ColumnId_Pulsing = 108,
-            ColumnId_SlowPulsing = 109,
+            ColumnId_EmitterType = 106,
+            // unused (3x)
             ColumnId_Fire = 110,
             ColumnId_OffByDefault = 111,
             ColumnId_IsKey = 112,
@@ -278,7 +276,7 @@ namespace CSMWorld
             ColumnId_NpcMisc = 251,
             ColumnId_Level = 252,
             ColumnId_NpcFactionID = 253,
-            // unused
+            ColumnId_GenderNpc = 254,
             ColumnId_Mana = 255,
             ColumnId_Fatigue = 256,
             ColumnId_NpcDisposition = 257,
@@ -329,6 +327,17 @@ namespace CSMWorld
             ColumnId_WeatherName = 295,
             ColumnId_WeatherChance = 296,
 
+            ColumnId_Text = 297,
+
+            ColumnId_TextureNickname = 298,
+            ColumnId_PluginIndex = 299,
+            ColumnId_TextureIndex = 300,
+            ColumnId_LandMapLodIndex = 301,
+            ColumnId_LandNormalsIndex = 302,
+            ColumnId_LandHeightsIndex = 303,
+            ColumnId_LandColoursIndex = 304,
+            ColumnId_LandTexturesIndex = 305,
+
             // Allocated to a separate value range, so we don't get a collision should we ever need
             // to extend the number of use values.
             ColumnId_UseValue1 = 0x10000,
