@@ -31,6 +31,7 @@
 
 #include <components/resource/resourcesystem.hpp>
 
+#include <components/files/multidircollection.hpp>
 #include <components/to_utf8/to_utf8.hpp>
 
 #include "../doc/stage.hpp"
@@ -46,6 +47,7 @@
 #include "infocollection.hpp"
 #include "nestedinfocollection.hpp"
 #include "pathgrid.hpp"
+#include "resourcesmanager.hpp"
 #include "metadata.hpp"
 #ifndef Q_MOC_RUN
 #include "subcellcollection.hpp"
@@ -108,7 +110,6 @@ namespace CSMWorld
             RefCollection mRefs;
             IdCollection<ESM::Filter> mFilters;
             Collection<MetaData> mMetaData;
-            const ResourcesManager& mResourcesManager;
             const Fallback::Map* mFallbackMap;
             std::vector<QAbstractItemModel *> mModels;
             std::map<UniversalId::Type, QAbstractItemModel *> mModelIndex;
@@ -119,9 +120,14 @@ namespace CSMWorld
             std::map<std::string, std::map<ESM::RefNum, std::string> > mRefLoadCache;
             int mReaderIndex;
 
-            boost::shared_ptr<Resource::ResourceSystem> mResourceSystem;
+            bool mFsStrict;
+            Files::PathContainer mDataPaths;
+            std::vector<std::string> mArchives;
+            std::unique_ptr<VFS::Manager> mVFS;
+            ResourcesManager mResourcesManager;
+            std::shared_ptr<Resource::ResourceSystem> mResourceSystem;
 
-            std::vector<boost::shared_ptr<ESM::ESMReader> > mReaders;
+            std::vector<std::shared_ptr<ESM::ESMReader> > mReaders;
 
             std::map<std::string, int> mContentFileNames;
 
@@ -140,7 +146,9 @@ namespace CSMWorld
 
         public:
 
-            Data (ToUTF8::FromType encoding, const ResourcesManager& resourcesManager, const Fallback::Map* fallback, const boost::filesystem::path& resDir);
+            Data (ToUTF8::FromType encoding, bool fsStrict, const Files::PathContainer& dataPaths,
+                const std::vector<std::string>& archives, const Fallback::Map* fallback,
+                const boost::filesystem::path& resDir);
 
             virtual ~Data();
 
@@ -148,9 +156,9 @@ namespace CSMWorld
 
             const Fallback::Map* getFallbackMap() const;
 
-            boost::shared_ptr<Resource::ResourceSystem> getResourceSystem();
+            std::shared_ptr<Resource::ResourceSystem> getResourceSystem();
 
-            boost::shared_ptr<const Resource::ResourceSystem> getResourceSystem() const;
+            std::shared_ptr<const Resource::ResourceSystem> getResourceSystem() const;
 
             const IdCollection<ESM::Global>& getGlobals() const;
 
@@ -304,8 +312,12 @@ namespace CSMWorld
 
             void idListChanged();
 
+            void assetTablesChanged();
+
         private slots:
 
+            void assetsChanged();
+
             void dataChanged (const QModelIndex& topLeft, const QModelIndex& bottomRight);
 
             void rowsChanged (const QModelIndex& parent, int start, int end);
