@@ -301,9 +301,9 @@ void CSMWorld::ArmorRefIdAdapter::setData (const RefIdColumn *column, RefIdData&
 }
 
 CSMWorld::BookRefIdAdapter::BookRefIdAdapter (const EnchantableColumns& columns,
-    const RefIdColumn *scroll, const RefIdColumn *skill)
+    const RefIdColumn *bookType, const RefIdColumn *skill, const RefIdColumn *text)
 : EnchantableRefIdAdapter<ESM::Book> (UniversalId::Type_Book, columns),
-    mScroll (scroll), mSkill (skill)
+    mBookType (bookType), mSkill (skill), mText (text)
 {}
 
 QVariant CSMWorld::BookRefIdAdapter::getData (const RefIdColumn *column,
@@ -312,11 +312,14 @@ QVariant CSMWorld::BookRefIdAdapter::getData (const RefIdColumn *column,
     const Record<ESM::Book>& record = static_cast<const Record<ESM::Book>&> (
         data.getRecord (RefIdData::LocalIndex (index, UniversalId::Type_Book)));
 
-    if (column==mScroll)
-        return record.get().mData.mIsScroll!=0;
+    if (column==mBookType)
+        return record.get().mData.mIsScroll;
 
     if (column==mSkill)
-        return record.get().mData.mSkillID;
+        return record.get().mData.mSkillId;
+
+    if (column==mText)
+        return QString::fromUtf8 (record.get().mText.c_str());
 
     return EnchantableRefIdAdapter<ESM::Book>::getData (column, data, index);
 }
@@ -329,10 +332,12 @@ void CSMWorld::BookRefIdAdapter::setData (const RefIdColumn *column, RefIdData&
 
     ESM::Book book = record.get();
 
-    if (column==mScroll)
+    if (column==mBookType)
         book.mData.mIsScroll = value.toInt();
     else if (column==mSkill)
-        book.mData.mSkillID = value.toInt();
+        book.mData.mSkillId = value.toInt();
+    else if (column==mText)
+        book.mText = value.toString().toUtf8().data();
     else
     {
         EnchantableRefIdAdapter<ESM::Book>::setData (column, data, index, value);
@@ -453,7 +458,8 @@ CSMWorld::CreatureColumns::CreatureColumns (const ActorColumns& actorColumns)
   mOriginal(NULL),
   mAttributes(NULL),
   mAttacks(NULL),
-  mMisc(NULL)
+  mMisc(NULL),
+  mBloodType(NULL)
 {}
 
 CSMWorld::CreatureRefIdAdapter::CreatureRefIdAdapter (const CreatureColumns& columns)
@@ -484,6 +490,19 @@ QVariant CSMWorld::CreatureRefIdAdapter::getData (const RefIdColumn *column, con
     if (column==mColumns.mMisc)
         return QVariant::fromValue(ColumnBase::TableEdit_Full);
 
+    if (column == mColumns.mBloodType)
+    {
+        int mask = ESM::Creature::Skeleton | ESM::Creature::Metal;
+
+        if ((record.get().mFlags & mask) == ESM::Creature::Skeleton)
+            return 1;
+
+        if ((record.get().mFlags & mask) == ESM::Creature::Metal)
+            return 2;
+
+        return 0;
+    }
+
     std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
         mColumns.mFlags.find (column);
 
@@ -507,6 +526,17 @@ void CSMWorld::CreatureRefIdAdapter::setData (const RefIdColumn *column, RefIdDa
         creature.mScale = value.toFloat();
     else if (column==mColumns.mOriginal)
         creature.mOriginal = value.toString().toUtf8().constData();
+    else if (column == mColumns.mBloodType)
+    {
+        int mask = ~(ESM::Creature::Skeleton | ESM::Creature::Metal);
+
+        if (value.toInt() == 1)
+            creature.mFlags = (creature.mFlags & mask) | ESM::Creature::Skeleton;
+        else if (value.toInt() == 2)
+            creature.mFlags = (creature.mFlags & mask) | ESM::Creature::Metal;
+        else
+            creature.mFlags = creature.mFlags & mask;
+    }
     else
     {
         std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
@@ -598,6 +628,25 @@ QVariant CSMWorld::LightRefIdAdapter::getData (const RefIdColumn *column, const
     if (column==mColumns.mSound)
         return QString::fromUtf8 (record.get().mSound.c_str());
 
+    if (column == mColumns.mEmitterType)
+    {
+        int mask = ESM::Light::Flicker | ESM::Light::FlickerSlow | ESM::Light::Pulse | ESM::Light::PulseSlow;
+
+        if ((record.get().mData.mFlags & mask) == ESM::Light::Flicker)
+            return 1;
+
+        if ((record.get().mData.mFlags & mask) == ESM::Light::FlickerSlow)
+            return 2;
+
+        if ((record.get().mData.mFlags & mask) == ESM::Light::Pulse)
+            return 3;
+
+        if ((record.get().mData.mFlags & mask) == ESM::Light::PulseSlow)
+            return 4;
+
+        return 0;
+    }
+
     std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
         mColumns.mFlags.find (column);
 
@@ -623,6 +672,21 @@ void CSMWorld::LightRefIdAdapter::setData (const RefIdColumn *column, RefIdData&
         light.mData.mColor = value.toInt();
     else if (column==mColumns.mSound)
         light.mSound = value.toString().toUtf8().constData();
+    else if (column == mColumns.mEmitterType)
+    {
+        int mask = ~(ESM::Light::Flicker | ESM::Light::FlickerSlow | ESM::Light::Pulse | ESM::Light::PulseSlow);
+
+        if (value.toInt() == 0)
+            light.mData.mFlags = light.mData.mFlags & mask;
+        else if (value.toInt() == 1)
+            light.mData.mFlags = (light.mData.mFlags & mask) | ESM::Light::Flicker;
+        else if (value.toInt() == 2)
+            light.mData.mFlags = (light.mData.mFlags & mask) | ESM::Light::FlickerSlow;
+        else if (value.toInt() == 3)
+            light.mData.mFlags = (light.mData.mFlags & mask) | ESM::Light::Pulse;
+        else
+            light.mData.mFlags = (light.mData.mFlags & mask) | ESM::Light::PulseSlow;
+    }
     else
     {
         std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
@@ -691,7 +755,9 @@ CSMWorld::NpcColumns::NpcColumns (const ActorColumns& actorColumns)
   mHead(NULL),
   mAttributes(NULL),
   mSkills(NULL),
-  mMisc(NULL)
+  mMisc(NULL),
+  mBloodType(NULL),
+  mGender(NULL)
 {}
 
 CSMWorld::NpcRefIdAdapter::NpcRefIdAdapter (const NpcColumns& columns)
@@ -730,6 +796,28 @@ QVariant CSMWorld::NpcRefIdAdapter::getData (const RefIdColumn *column, const Re
     if (column==mColumns.mMisc)
         return QVariant::fromValue(ColumnBase::TableEdit_Full);
 
+    if (column == mColumns.mBloodType)
+    {
+        int mask = ESM::NPC::Skeleton | ESM::NPC::Metal;
+
+        if ((record.get().mFlags & mask) == ESM::NPC::Skeleton)
+            return 1;
+
+        if ((record.get().mFlags & mask) == ESM::NPC::Metal)
+            return 2;
+
+        return 0;
+    }
+
+    if (column == mColumns.mGender)
+    {
+        // Implemented this way to allow additional gender types in the future.
+        if ((record.get().mFlags & ESM::NPC::Female) == ESM::NPC::Female)
+            return 1;
+
+        return 0;
+    }
+
     std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
         mColumns.mFlags.find (column);
 
@@ -757,6 +845,25 @@ void CSMWorld::NpcRefIdAdapter::setData (const RefIdColumn *column, RefIdData& d
         npc.mHair = value.toString().toUtf8().constData();
     else if (column==mColumns.mHead)
         npc.mHead = value.toString().toUtf8().constData();
+    else if (column == mColumns.mBloodType)
+    {
+        int mask = ~(ESM::NPC::Skeleton | ESM::NPC::Metal);
+
+        if (value.toInt() == 1)
+            npc.mFlags = (npc.mFlags & mask) | ESM::NPC::Skeleton;
+        else if (value.toInt() == 2)
+            npc.mFlags = (npc.mFlags & mask) | ESM::NPC::Metal;
+        else
+            npc.mFlags = npc.mFlags & mask;
+    }
+    else if (column == mColumns.mGender)
+    {
+        // Implemented this way to allow additional gender types in the future.
+        if (value.toInt() == 1)
+            npc.mFlags = (npc.mFlags & ~ESM::NPC::Female) | ESM::NPC::Female;
+        else
+            npc.mFlags = npc.mFlags & ~ESM::NPC::Female;
+    }
     else
     {
         std::map<const RefIdColumn *, unsigned int>::const_iterator iter =
