@@ -12,18 +12,19 @@
 
 #include "page.hpp"
 #include "keybindingpage.hpp"
+#include "contextmenulist.hpp"
 
 void CSVPrefs::Dialogue::buildCategorySelector (QSplitter *main)
 {
-    mList = new QListWidget (main);
-    mList->setMinimumWidth (50);
-    mList->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
+    CSVPrefs::ContextMenuList* list = new CSVPrefs::ContextMenuList (main);
+    list->setMinimumWidth (50);
+    list->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
 
-    mList->setSelectionBehavior (QAbstractItemView::SelectItems);
+    list->setSelectionBehavior (QAbstractItemView::SelectItems);
 
-    main->addWidget (mList);
+    main->addWidget (list);
 
-    QFontMetrics metrics (QApplication::font());
+    QFontMetrics metrics (QApplication::font(list));
 
     int maxWidth = 1;
 
@@ -33,12 +34,12 @@ void CSVPrefs::Dialogue::buildCategorySelector (QSplitter *main)
         QString label = QString::fromUtf8 (iter->second.getKey().c_str());
         maxWidth = std::max (maxWidth, metrics.width (label));
 
-        mList->addItem (label);
+        list->addItem (label);
     }
 
-    mList->setMaximumWidth (maxWidth + 10);
+    list->setMaximumWidth (maxWidth + 10);
 
-    connect (mList, SIGNAL (currentItemChanged (QListWidgetItem *, QListWidgetItem *)),
+    connect (list, SIGNAL (currentItemChanged (QListWidgetItem *, QListWidgetItem *)),
         this, SLOT (selectionChanged (QListWidgetItem *, QListWidgetItem *)));
 }
 
