@@ -16,6 +16,7 @@
 #include "../../model/world/idtableproxymodel.hpp"
 #include "../../model/world/idtablebase.hpp"
 #include "../../model/world/idtable.hpp"
+#include "../../model/world/landtexturetableproxymodel.hpp"
 #include "../../model/world/record.hpp"
 #include "../../model/world/columns.hpp"
 #include "../../model/world/commanddispatcher.hpp"
@@ -60,6 +61,9 @@ void CSVWorld::Table::contextMenuEvent (QContextMenuEvent *event)
                 menu.addAction(mCloneAction);
         }
 
+        if (mTouchAction)
+            menu.addAction (mTouchAction);
+
         if (mCreateAction)
             menu.addAction (mCreateAction);
 
@@ -226,17 +230,22 @@ void CSVWorld::Table::mouseDoubleClickEvent (QMouseEvent *event)
 
 CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
     bool createAndDelete, bool sorting, CSMDoc::Document& document)
-: DragRecordTable(document), mCreateAction (0),
-  mCloneAction(0), mRecordStatusDisplay (0), mJumpToAddedRecord(false), mUnselectAfterJump(false)
+    : DragRecordTable(document), mCreateAction (nullptr), mCloneAction(nullptr), mTouchAction(nullptr),
+    mRecordStatusDisplay (0), mJumpToAddedRecord(false), mUnselectAfterJump(false)
 {
     mModel = &dynamic_cast<CSMWorld::IdTableBase&> (*mDocument.getData().getTableModel (id));
 
     bool isInfoTable = id.getType() == CSMWorld::UniversalId::Type_TopicInfos ||
                        id.getType() == CSMWorld::UniversalId::Type_JournalInfos;
+    bool isLtexTable = (id.getType() == CSMWorld::UniversalId::Type_LandTextures);
     if (isInfoTable)
     {
         mProxyModel = new CSMWorld::InfoTableProxyModel(id.getType(), this);
     }
+    else if (isLtexTable)
+    {
+        mProxyModel = new CSMWorld::LandTextureTableProxyModel (this);
+    }
     else
     {
         mProxyModel = new CSMWorld::IdTableProxyModel (this);
@@ -302,6 +311,15 @@ CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
         cloneShortcut->associateAction(mCloneAction);
     }
 
+    if (mModel->getFeatures() & CSMWorld::IdTableBase::Feature_AllowTouch)
+    {
+        mTouchAction = new QAction(tr("Touch Record"), this);
+        connect(mTouchAction, SIGNAL(triggered()), this, SLOT(touchRecord()));
+        addAction(mTouchAction);
+        CSMPrefs::Shortcut* touchShortcut = new CSMPrefs::Shortcut("table-touch", this);
+        touchShortcut->associateAction(mTouchAction);
+    }
+
     mRevertAction = new QAction (tr ("Revert Record"), this);
     connect (mRevertAction, SIGNAL (triggered()), mDispatcher, SLOT (executeRevert()));
     addAction (mRevertAction);
@@ -442,6 +460,22 @@ void CSVWorld::Table::cloneRecord()
     }
 }
 
+void CSVWorld::Table::touchRecord()
+{
+    if (!mEditLock && mModel->getFeatures() & CSMWorld::IdTableBase::Feature_AllowTouch)
+    {
+        std::vector<CSMWorld::UniversalId> touchIds;
+
+        QModelIndexList selectedRows = selectionModel()->selectedRows();
+        for (auto it = selectedRows.begin(); it != selectedRows.end(); ++it)
+        {
+            touchIds.push_back(getUniversalId(it->row()));
+        }
+
+        emit touchRequest(touchIds);
+    }
+}
+
 void CSVWorld::Table::moveUpRecord()
 {
     if (mEditLock || (mModel->getFeatures() & CSMWorld::IdTableBase::Feature_Constant))
@@ -692,7 +726,7 @@ void CSVWorld::Table::requestFocus (const std::string& id)
         scrollTo (index, QAbstractItemView::PositionAtTop);
 }
 
-void CSVWorld::Table::recordFilterChanged (boost::shared_ptr<CSMFilter::Node> filter)
+void CSVWorld::Table::recordFilterChanged (std::shared_ptr<CSMFilter::Node> filter)
 {
     mProxyModel->setFilter (filter);
     tableSizeUpdate();
