@@ -18,6 +18,11 @@ namespace Resource
     class ResourceSystem;
 }
 
+namespace SceneUtil
+{
+    class WorkQueue;
+}
+
 namespace VFS
 {
     class Manager;
@@ -66,8 +71,9 @@ namespace OMW
     class Engine
     {
             SDL_Window* mWindow;
-            std::auto_ptr<VFS::Manager> mVFS;
-            std::auto_ptr<Resource::ResourceSystem> mResourceSystem;
+            std::unique_ptr<VFS::Manager> mVFS;
+            std::unique_ptr<Resource::ResourceSystem> mResourceSystem;
+            osg::ref_ptr<SceneUtil::WorkQueue> mWorkQueue;
             MWBase::Environment mEnvironment;
             ToUTF8::FromType mEncoding;
             ToUTF8::Utf8Encoder* mEncoder;
@@ -78,7 +84,6 @@ namespace OMW
             osg::ref_ptr<osgViewer::ScreenCaptureHandler> mScreenCaptureHandler;
             std::string mCellName;
             std::vector<std::string> mContentFiles;
-            bool mVerboseScripts;
             bool mSkipMenu;
             bool mUseSound;
             bool mCompileAll;
@@ -143,7 +148,7 @@ namespace OMW
             /// Set resource dir
             void setResourceDir(const boost::filesystem::path& parResDir);
 
-            /// Set start cell name (only interiors for now)
+            /// Set start cell name
             void setCell(const std::string& cellName);
 
             /**
@@ -152,9 +157,6 @@ namespace OMW
              */
             void addContentFile(const std::string& file);
 
-            /// Enable or disable verbose script output
-            void setScriptsVerbosity(bool scriptsVerbosity);
-
             /// Disable or enable all sounds
             void setSoundUsage(bool soundUsage);
 
