@@ -7,6 +7,8 @@
 #include <map>
 #include <set>
 
+#include <MyGUI_KeyCode.h>
+
 #include "../mwgui/mode.hpp"
 
 namespace Loading
@@ -72,6 +74,8 @@ namespace MWGui
         ShowInDialogueMode_Only,
         ShowInDialogueMode_Never
     };
+
+    struct TextColours;
 }
 
 namespace SFO
@@ -98,23 +102,17 @@ namespace MWBase
 
             virtual ~WindowManager() {}
 
-            /**
-             * Should be called each frame to update windows/gui elements.
-             * This could mean updating sizes of gui elements or opening
-             * new dialogs.
-             */
-            virtual void update() = 0;
-
             /// @note This method will block until the video finishes playing
             /// (and will continually update the window while doing so)
             virtual void playVideo(const std::string& name, bool allowSkipping) = 0;
 
             virtual void setNewGame(bool newgame) = 0;
 
+            virtual void pushGuiMode (MWGui::GuiMode mode, const MWWorld::Ptr& arg) = 0;
             virtual void pushGuiMode (MWGui::GuiMode mode) = 0;
-            virtual void popGuiMode() = 0;
+            virtual void popGuiMode(bool noSound=false) = 0;
 
-            virtual void removeGuiMode (MWGui::GuiMode mode) = 0;
+            virtual void removeGuiMode (MWGui::GuiMode mode, bool noSound=false) = 0;
             ///< can be anywhere in the stack
 
             virtual void goToJail(int days) = 0;
@@ -142,7 +140,6 @@ namespace MWBase
             virtual bool isAllowed (MWGui::GuiWindow wnd) const = 0;
 
             /// \todo investigate, if we really need to expose every single lousy UI element to the outside world
-            virtual MWGui::DialogueWindow* getDialogueWindow() = 0;
             virtual MWGui::InventoryWindow* getInventoryWindow() = 0;
             virtual MWGui::CountDialog* getCountDialog() = 0;
             virtual MWGui::ConfirmationDialog* getConfirmationDialog() = 0;
@@ -183,6 +180,7 @@ namespace MWBase
             virtual void setFocusObjectScreenCoords(float min_x, float min_y, float max_x, float max_y) = 0;
 
             virtual void setCursorVisible(bool visible) = 0;
+            virtual void setCursorActive(bool active) = 0;
             virtual void getMousePosition(int &x, int &y) = 0;
             virtual void getMousePosition(float &x, float &y) = 0;
             virtual void setDragDrop(bool dragDrop) = 0;
@@ -210,18 +208,23 @@ namespace MWBase
             virtual void setSpellVisibility(bool visible) = 0;
             virtual void setSneakVisibility(bool visible) = 0;
 
-            virtual void activateQuickKey  (int index) = 0;
+            /// activate selected quick key
+            virtual void activateQuickKey (int index) = 0;
+            /// update activated quick key state (if action executing was delayed for some reason)
+            virtual void updateActivatedQuickKey () = 0;
 
             virtual std::string getSelectedSpell() = 0;
             virtual void setSelectedSpell(const std::string& spellId, int successChancePercent) = 0;
             virtual void setSelectedEnchantItem(const MWWorld::Ptr& item) = 0;
+            virtual const MWWorld::Ptr& getSelectedEnchantItem() const = 0;
             virtual void setSelectedWeapon(const MWWorld::Ptr& item) = 0;
+            virtual const MWWorld::Ptr& getSelectedWeapon() const = 0;
             virtual void unsetSelectedSpell() = 0;
             virtual void unsetSelectedWeapon() = 0;
 
             virtual void showCrosshair(bool show) = 0;
             virtual bool getSubtitlesEnabled() = 0;
-            virtual bool toggleGui() = 0;
+            virtual bool toggleHud() = 0;
 
             virtual void disallowMouse() = 0;
             virtual void allowMouse() = 0;
@@ -233,7 +236,7 @@ namespace MWBase
             virtual void removeDialog(MWGui::Layout* dialog) = 0;
 
             ///Gracefully attempts to exit the topmost GUI mode
-            /** No guarentee of actually closing the window **/
+            /** No guarantee of actually closing the window **/
             virtual void exitCurrentGuiMode() = 0;
 
             virtual void messageBox (const std::string& message, enum MWGui::ShowInDialogueMode showInDialogueMode = MWGui::ShowInDialogueMode_IfPossible) = 0;
@@ -275,21 +278,6 @@ namespace MWBase
             virtual bool getPlayerSleeping() = 0;
             virtual void wakeUpPlayer() = 0;
 
-            virtual void showCompanionWindow(MWWorld::Ptr actor) = 0;
-            virtual void startSpellMaking(MWWorld::Ptr actor) = 0;
-            virtual void startEnchanting(MWWorld::Ptr actor) = 0;
-            virtual void startRecharge(MWWorld::Ptr soulgem) = 0;
-            virtual void startSelfEnchanting(MWWorld::Ptr soulgem) = 0;
-            virtual void startTraining(MWWorld::Ptr actor) = 0;
-            virtual void startRepair(MWWorld::Ptr actor) = 0;
-            virtual void startRepairItem(MWWorld::Ptr item) = 0;
-            virtual void startTravel(const MWWorld::Ptr& actor) = 0;
-            virtual void startSpellBuying(const MWWorld::Ptr& actor) = 0;
-            virtual void startTrade(const MWWorld::Ptr& actor) = 0;
-            virtual void openContainer(const MWWorld::Ptr& container, bool loot) = 0;
-            virtual void showBook(const MWWorld::Ptr& item, bool showTakeButton) = 0;
-            virtual void showScroll(const MWWorld::Ptr& item, bool showTakeButton) = 0;
-
             virtual void showSoulgemDialog (MWWorld::Ptr item) = 0;
 
             virtual void changePointer (const std::string& name) = 0;
@@ -331,11 +319,11 @@ namespace MWBase
             virtual void pinWindow (MWGui::GuiWindow window) = 0;
 
             /// Fade the screen in, over \a time seconds
-            virtual void fadeScreenIn(const float time, bool clearQueue=true) = 0;
+            virtual void fadeScreenIn(const float time, bool clearQueue=true, float delay=0.f) = 0;
             /// Fade the screen out to black, over \a time seconds
-            virtual void fadeScreenOut(const float time, bool clearQueue=true) = 0;
+            virtual void fadeScreenOut(const float time, bool clearQueue=true, float delay=0.f) = 0;
             /// Fade the screen to a specified percentage of black, over \a time seconds
-            virtual void fadeScreenTo(const int percent, const float time, bool clearQueue=true) = 0;
+            virtual void fadeScreenTo(const int percent, const float time, bool clearQueue=true, float delay=0.f) = 0;
             /// Darken the screen to a specified percentage
             virtual void setBlindness(const int percent) = 0;
 
@@ -349,14 +337,20 @@ namespace MWBase
             /// Cycle to next or previous weapon
             virtual void cycleWeapon(bool next) = 0;
 
+            virtual void playSound(const std::string& soundId, float volume = 1.f, float pitch = 1.f) = 0;
+
             // In WindowManager for now since there isn't a VFS singleton
             virtual std::string correctIconPath(const std::string& path) = 0;
-            virtual std::string correctBookartPath(const std::string& path, int width, int height) = 0;
+            virtual std::string correctBookartPath(const std::string& path, int width, int height, bool* exists = nullptr) = 0;
             virtual std::string correctTexturePath(const std::string& path) = 0;
             virtual bool textureExists(const std::string& path) = 0;
 
             virtual void removeCell(MWWorld::CellStore* cell) = 0;
             virtual void writeFog(MWWorld::CellStore* cell) = 0;
+
+            virtual const MWGui::TextColours& getTextColours() = 0;
+
+            virtual bool injectKeyPress(MyGUI::KeyCode key, unsigned int text) = 0;
     };
 }
 
