@@ -12,7 +12,7 @@ namespace MWClass
 
     void ItemLevList::registerSelf()
     {
-        boost::shared_ptr<Class> instance (new ItemLevList);
+        std::shared_ptr<Class> instance (new ItemLevList);
 
         registerClass (typeid (ESM::ItemLevList).name(), instance);
     }
