@@ -5,6 +5,7 @@
 #include <algorithm>
 #include <iterator>
 #include <list>
+#include <iostream>
 
 #include <components/esm/loaddial.hpp>
 #include <components/esm/loadinfo.hpp>
@@ -33,8 +34,6 @@
 #include "../mwworld/containerstore.hpp"
 #include "../mwworld/esmstore.hpp"
 
-#include "../mwgui/dialogue.hpp"
-
 #include "../mwscript/compilercontext.hpp"
 #include "../mwscript/interpretercontext.hpp"
 #include "../mwscript/extensions.hpp"
@@ -48,7 +47,7 @@
 
 namespace MWDialogue
 {
-    DialogueManager::DialogueManager (const Compiler::Extensions& extensions, bool scriptVerbose, Translation::Storage& translationDataStorage) :
+    DialogueManager::DialogueManager (const Compiler::Extensions& extensions, Translation::Storage& translationDataStorage) :
       mTranslationDataStorage(translationDataStorage)
       , mCompilerContext (MWScript::CompilerContext::Type_Dialogue)
       , mErrorStream(std::cout.rdbuf())
@@ -59,16 +58,8 @@ namespace MWDialogue
     {
         mChoice = -1;
         mIsInChoice = false;
+        mGoodbye = false;
         mCompilerContext.setExtensions (&extensions);
-
-        const MWWorld::Store<ESM::Dialogue> &dialogs =
-            MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
-
-        MWWorld::Store<ESM::Dialogue>::iterator it = dialogs.begin();
-        for (; it != dialogs.end(); ++it)
-        {
-            mDialogueMap[Misc::StringUtils::lowerCase(it->mId)] = *it;
-        }
     }
 
     void DialogueManager::clear()
@@ -108,17 +99,15 @@ namespace MWDialogue
             if (mActorKnownTopics.count( topicId ))
                 mKnownTopics.insert( topicId );
         }
-
-        updateTopics();
     }
 
-    void DialogueManager::startDialogue (const MWWorld::Ptr& actor)
+    bool DialogueManager::startDialogue (const MWWorld::Ptr& actor, ResponseCallback* callback)
     {
         updateGlobals();
 
         // Dialogue with dead actor (e.g. through script) should not be allowed.
         if (actor.getClass().getCreatureStats(actor).isDead())
-            return;
+            return false;
 
         mLastTopic = "";
         mPermanentDispositionChange = 0;
@@ -126,6 +115,8 @@ namespace MWDialogue
 
         mChoice = -1;
         mIsInChoice = false;
+        mGoodbye = false;
+        mChoices.clear();
 
         mActor = actor;
 
@@ -134,13 +125,6 @@ namespace MWDialogue
 
         mActorKnownTopics.clear();
 
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
-
-        // If the dialogue window was already open, keep the existing history
-        bool resetHistory = (!MWBase::Environment::get().getWindowManager()->containsMode(MWGui::GM_Dialogue));
-
-        win->startDialogue(actor, actor.getClass().getName (actor), resetHistory);
-
         //greeting
         const MWWorld::Store<ESM::Dialogue> &dialogs =
             MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
@@ -154,9 +138,6 @@ namespace MWDialogue
                 // Search a response (we do not accept a fallback to "Info refusal" here)
                 if (const ESM::DialInfo *info = filter.search (*it, false))
                 {
-                    //initialise the GUI
-                    MWBase::Environment::get().getWindowManager()->pushGuiMode(MWGui::GM_Dialogue);
-
                     creatureStats.talkedToPlayer();
 
                     if (!info->mSound.empty())
@@ -165,32 +146,26 @@ namespace MWDialogue
                     }
 
                     // first topics update so that parseText knows the keywords to highlight
-                    updateTopics();
+                    updateActorKnownTopics();
 
                     parseText (info->mResponse);
 
                     MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
-                    win->addResponse (Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
-                    executeScript (info->mResultScript);
-                    mLastTopic = Misc::StringUtils::lowerCase(it->mId);
+                    callback->addResponse("", Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
+                    executeScript (info->mResultScript, mActor);
+                    mLastTopic = it->mId;
 
-                    // update topics again to accomodate changes resulting from executeScript
-                    updateTopics();
+                    // update topics again to accommodate changes resulting from executeScript
+                    updateActorKnownTopics();
 
-                    return;
+                    return true;
                 }
             }
         }
-
-        // No greetings found. The dialogue window should not be shown.
-        // If this is a companion, we must show the companion window directly (used by BM_bear_be_unique).
-        bool isCompanion = !mActor.getClass().getScript(mActor).empty()
-                && mActor.getRefData().getLocals().getIntVar(mActor.getClass().getScript(mActor), "companion");
-        if (isCompanion)
-            MWBase::Environment::get().getWindowManager()->showCompanionWindow(mActor);
+        return false;
     }
 
-    bool DialogueManager::compile (const std::string& cmd,std::vector<Interpreter::Type_Code>& code)
+    bool DialogueManager::compile (const std::string& cmd, std::vector<Interpreter::Type_Code>& code, const MWWorld::Ptr& actor)
     {
         bool success = true;
 
@@ -198,13 +173,15 @@ namespace MWDialogue
         {
             mErrorHandler.reset();
 
+            mErrorHandler.setContext("[dialogue script]");
+
             std::istringstream input (cmd + "\n");
 
             Compiler::Scanner scanner (mErrorHandler, input, mCompilerContext.getExtensions());
 
             Compiler::Locals locals;
 
-            std::string actorScript = mActor.getClass().getScript (mActor);
+            std::string actorScript = actor.getClass().getScript (actor);
 
             if (!actorScript.empty())
             {
@@ -236,7 +213,7 @@ namespace MWDialogue
         if (!success)
         {
             std::cerr
-                << "compiling failed (dialogue script)" << std::endl
+                << "Warning: compiling failed (dialogue script)" << std::endl
                 << cmd
                 << std::endl << std::endl;
         }
@@ -244,14 +221,14 @@ namespace MWDialogue
         return success;
     }
 
-    void DialogueManager::executeScript (const std::string& script)
+    void DialogueManager::executeScript (const std::string& script, const MWWorld::Ptr& actor)
     {
         std::vector<Interpreter::Type_Code> code;
-        if(compile(script,code))
+        if(compile(script, code, actor))
         {
             try
             {
-                MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
+                MWScript::InterpreterContext interpreterContext(&actor.getRefData().getLocals(), actor);
                 Interpreter::Interpreter interpreter;
                 MWScript::installOpcodes (interpreter);
                 interpreter.run (&code[0], code.size(), interpreterContext);
@@ -263,7 +240,7 @@ namespace MWDialogue
         }
     }
 
-    void DialogueManager::executeTopic (const std::string& topic)
+    void DialogueManager::executeTopic (const std::string& topic, ResponseCallback* callback)
     {
         Filter filter (mActor, mChoice, mTalkedTo);
 
@@ -272,8 +249,6 @@ namespace MWDialogue
 
         const ESM::Dialogue& dialogue = *dialogues.find (topic);
 
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
-
         const ESM::DialInfo* info = filter.search(dialogue, true);
         if (info)
         {
@@ -298,7 +273,7 @@ namespace MWDialogue
                 title = topic;
 
             MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
-            win->addResponse (Interpreter::fixDefinesDialog(info->mResponse, interpreterContext), title);
+            callback->addResponse(title, Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
 
             if (dialogue.mType == ESM::Dialogue::Topic)
             {
@@ -309,21 +284,21 @@ namespace MWDialogue
                 {
                     if (iter->mId == info->mId)
                     {
-                        MWBase::Environment::get().getJournal()->addTopic (topic, info->mId, mActor);
+                        MWBase::Environment::get().getJournal()->addTopic (Misc::StringUtils::lowerCase(topic), info->mId, mActor);
                         break;
                     }
                 }
             }
 
-            executeScript (info->mResultScript);
+            executeScript (info->mResultScript, mActor);
 
             mLastTopic = topic;
         }
-        else
-        {
-            // no response found, print a fallback text
-            win->addResponse ("…", topic);
-        }
+    }
+
+    const ESM::Dialogue *DialogueManager::searchDialogue(const std::string& id)
+    {
+        return MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>().search(id);
     }
 
     void DialogueManager::updateGlobals()
@@ -331,19 +306,16 @@ namespace MWDialogue
         MWBase::Environment::get().getWorld()->updateDialogueGlobals();
     }
 
-    void DialogueManager::updateTopics()
+    void DialogueManager::updateActorKnownTopics()
     {
         updateGlobals();
 
-        std::list<std::string> keywordList;
-        int choice = mChoice;
-        mChoice = -1;
         mActorKnownTopics.clear();
 
         const MWWorld::Store<ESM::Dialogue> &dialogs =
             MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
 
-        Filter filter (mActor, mChoice, mTalkedTo);
+        Filter filter (mActor, -1, mTalkedTo);
 
         for (MWWorld::Store<ESM::Dialogue>::iterator iter = dialogs.begin(); iter != dialogs.end(); ++iter)
         {
@@ -351,92 +323,41 @@ namespace MWDialogue
             {
                 if (filter.responseAvailable (*iter))
                 {
-                    std::string lower = Misc::StringUtils::lowerCase(iter->mId);
-                    mActorKnownTopics.insert (lower);
-
-                    //does the player know the topic?
-                    if (mKnownTopics.count(lower))
-                    {
-                        keywordList.push_back (iter->mId);
-                    }
+                    mActorKnownTopics.insert (iter->mId);
                 }
             }
         }
 
-        // check the available services of this actor
-        int services = 0;
-        if (mActor.getTypeName() == typeid(ESM::NPC).name())
+    }
+
+    std::list<std::string> DialogueManager::getAvailableTopics()
+    {
+        updateActorKnownTopics();
+
+        std::list<std::string> keywordList;
+
+        for (const std::string& topic : mActorKnownTopics)
         {
-            MWWorld::LiveCellRef<ESM::NPC>* ref = mActor.get<ESM::NPC>();
-            if (ref->mBase->mHasAI)
-                services = ref->mBase->mAiData.mServices;
+            //does the player know the topic?
+            if (mKnownTopics.count(topic))
+                keywordList.push_back(topic);
         }
-        else if (mActor.getTypeName() == typeid(ESM::Creature).name())
-        {
-            MWWorld::LiveCellRef<ESM::Creature>* ref = mActor.get<ESM::Creature>();
-            if (ref->mBase->mHasAI)
-                services = ref->mBase->mAiData.mServices;
-        }
-
-        int windowServices = 0;
-
-        if (services & ESM::NPC::Weapon
-            || services & ESM::NPC::Armor
-            || services & ESM::NPC::Clothing
-            || services & ESM::NPC::Books
-            || services & ESM::NPC::Ingredients
-            || services & ESM::NPC::Picks
-            || services & ESM::NPC::Probes
-            || services & ESM::NPC::Lights
-            || services & ESM::NPC::Apparatus
-            || services & ESM::NPC::RepairItem
-            || services & ESM::NPC::Misc)
-            windowServices |= MWGui::DialogueWindow::Service_Trade;
-
-        if((mActor.getTypeName() == typeid(ESM::NPC).name() && !mActor.get<ESM::NPC>()->mBase->getTransport().empty())
-                || (mActor.getTypeName() == typeid(ESM::Creature).name() && !mActor.get<ESM::Creature>()->mBase->getTransport().empty()))
-            windowServices |= MWGui::DialogueWindow::Service_Travel;
-
-        if (services & ESM::NPC::Spells)
-            windowServices |= MWGui::DialogueWindow::Service_BuySpells;
-
-        if (services & ESM::NPC::Spellmaking)
-            windowServices |= MWGui::DialogueWindow::Service_CreateSpells;
-
-        if (services & ESM::NPC::Training)
-            windowServices |= MWGui::DialogueWindow::Service_Training;
-
-        if (services & ESM::NPC::Enchanting)
-            windowServices |= MWGui::DialogueWindow::Service_Enchant;
-
-        if (services & ESM::NPC::Repair)
-            windowServices |= MWGui::DialogueWindow::Service_Repair;
-
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
-
-        win->setServices (windowServices);
 
         // sort again, because the previous sort was case-sensitive
         keywordList.sort(Misc::StringUtils::ciLess);
-        win->setKeywords(keywordList);
-
-        mChoice = choice;
+        return keywordList;
     }
 
-    void DialogueManager::keywordSelected (const std::string& keyword)
+    void DialogueManager::keywordSelected (const std::string& keyword, ResponseCallback* callback)
     {
         if(!mIsInChoice)
         {
-            if(mDialogueMap.find(keyword) != mDialogueMap.end())
+            const ESM::Dialogue* dialogue = searchDialogue(keyword);
+            if (dialogue && dialogue->mType == ESM::Dialogue::Topic)
             {
-                if (mDialogueMap[keyword].mType == ESM::Dialogue::Topic)
-                {
-                    executeTopic (keyword);
-                }
+                executeTopic (keyword, callback);
             }
         }
-
-        updateTopics();
     }
 
     bool DialogueManager::isInChoice() const
@@ -446,16 +367,14 @@ namespace MWDialogue
 
     void DialogueManager::goodbyeSelected()
     {
-        MWBase::Environment::get().getWindowManager()->removeGuiMode(MWGui::GM_Dialogue);
-
-        // Clamp permanent disposition change so that final disposition doesn't go below 0 (could happen with intimidate)       
-        float curDisp = static_cast<float>(MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mActor, false));
-        if (curDisp + mPermanentDispositionChange < 0)
-            mPermanentDispositionChange = -curDisp;
-
         // Apply disposition change to NPC's base disposition
         if (mActor.getClass().isNpc())
         {
+            // Clamp permanent disposition change so that final disposition doesn't go below 0 (could happen with intimidate)       
+            float curDisp = static_cast<float>(MWBase::Environment::get().getMechanicsManager()->getDerivedDisposition(mActor, false));
+            if (curDisp + mPermanentDispositionChange < 0)
+                mPermanentDispositionChange = -curDisp;
+
             MWMechanics::NpcStats& npcStats = mActor.getClass().getNpcStats(mActor);
             npcStats.setBaseDisposition(static_cast<int>(npcStats.getBaseDisposition() + mPermanentDispositionChange));
         }
@@ -463,73 +382,79 @@ namespace MWDialogue
         mTemporaryDispositionChange = 0;
     }
 
-    void DialogueManager::questionAnswered (int answer)
+    void DialogueManager::questionAnswered (int answer, ResponseCallback* callback)
     {
         mChoice = answer;
 
-        if (mDialogueMap.find(mLastTopic) != mDialogueMap.end())
+        const ESM::Dialogue* dialogue = searchDialogue(mLastTopic);
+        if (dialogue)
         {
             Filter filter (mActor, mChoice, mTalkedTo);
 
-            if (mDialogueMap[mLastTopic].mType == ESM::Dialogue::Topic
-                    || mDialogueMap[mLastTopic].mType == ESM::Dialogue::Greeting)
+            if (dialogue->mType == ESM::Dialogue::Topic || dialogue->mType  == ESM::Dialogue::Greeting)
             {
-                if (const ESM::DialInfo *info = filter.search (mDialogueMap[mLastTopic], true))
+                if (const ESM::DialInfo *info = filter.search (*dialogue, true))
                 {
                     std::string text = info->mResponse;
                     parseText (text);
 
                     mChoice = -1;
                     mIsInChoice = false;
-                    MWBase::Environment::get().getWindowManager()->getDialogueWindow()->clearChoices();
+                    mChoices.clear();
 
                     MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
-                    MWBase::Environment::get().getWindowManager()->getDialogueWindow()->addResponse (Interpreter::fixDefinesDialog(text, interpreterContext));
+                    callback->addResponse("", Interpreter::fixDefinesDialog(text, interpreterContext));
 
                     // Make sure the returned DialInfo is from the Dialogue we supplied. If could also be from the Info refusal group,
                     // in which case it should not be added to the journal.
-                    for (ESM::Dialogue::InfoContainer::const_iterator iter = mDialogueMap[mLastTopic].mInfo.begin();
-                        iter!=mDialogueMap[mLastTopic].mInfo.end(); ++iter)
+                    for (ESM::Dialogue::InfoContainer::const_iterator iter = dialogue->mInfo.begin();
+                        iter!=dialogue->mInfo.end(); ++iter)
                     {
                         if (iter->mId == info->mId)
                         {
-                            MWBase::Environment::get().getJournal()->addTopic (mLastTopic, info->mId, mActor);
+                            MWBase::Environment::get().getJournal()->addTopic (Misc::StringUtils::lowerCase(mLastTopic), info->mId, mActor);
                             break;
                         }
                     }
 
-                    executeScript (info->mResultScript);
+                    executeScript (info->mResultScript, mActor);
                 }
                 else
                 {
                     mChoice = -1;
                     mIsInChoice = false;
-                    MWBase::Environment::get().getWindowManager()->getDialogueWindow()->clearChoices();
+                    mChoices.clear();
                 }
             }
         }
 
-        updateTopics();
+        updateActorKnownTopics();
     }
 
-    void DialogueManager::askQuestion (const std::string& question, int choice)
+    void DialogueManager::addChoice (const std::string& text, int choice)
     {
         mIsInChoice = true;
 
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
-        win->addChoice(question, choice);
+        mChoices.push_back(std::make_pair(text, choice));
+    }
+
+    const std::vector<std::pair<std::string, int> >& DialogueManager::getChoices()
+    {
+        return mChoices;
+    }
+
+    bool DialogueManager::isGoodbye()
+    {
+        return mGoodbye;
     }
 
     void DialogueManager::goodbye()
     {
-        mIsInChoice = true;
-
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
-
-        win->goodbye();
+        mIsInChoice = false;
+        mGoodbye = true;
     }
 
-    void DialogueManager::persuade(int type)
+    void DialogueManager::persuade(int type, ResponseCallback* callback)
     {
         bool success;
         float temp, perm;
@@ -578,7 +503,7 @@ namespace MWDialogue
             text = "Bribe";
         }
 
-        executeTopic (text + (success ? " Success" : " Fail"));
+        executeTopic (text + (success ? " Success" : " Fail"), callback);
     }
 
     int DialogueManager::getTemporaryDispositionChange() const
@@ -591,7 +516,7 @@ namespace MWDialogue
         mTemporaryDispositionChange += delta;
     }
 
-    bool DialogueManager::checkServiceRefused()
+    bool DialogueManager::checkServiceRefused(ResponseCallback* callback)
     {
         Filter filter (mActor, mChoice, mTalkedTo);
 
@@ -599,7 +524,6 @@ namespace MWDialogue
             MWBase::Environment::get().getWorld()->getStore().get<ESM::Dialogue>();
 
         const ESM::Dialogue& dialogue = *dialogues.find ("Service Refusal");
-        MWGui::DialogueWindow* win = MWBase::Environment::get().getWindowManager()->getDialogueWindow();
 
         std::vector<const ESM::DialInfo *> infos = filter.list (dialogue, false, false, true);
         if (!infos.empty())
@@ -613,16 +537,15 @@ namespace MWDialogue
 
             MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
 
-            win->addResponse (Interpreter::fixDefinesDialog(info->mResponse, interpreterContext),
-                              gmsts.find ("sServiceRefusal")->getString());
+            callback->addResponse(gmsts.find ("sServiceRefusal")->getString(), Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
 
-            executeScript (info->mResultScript);
+            executeScript (info->mResultScript, mActor);
             return true;
         }
         return false;
     }
 
-    void DialogueManager::say(const MWWorld::Ptr &actor, const std::string &topic) const
+    void DialogueManager::say(const MWWorld::Ptr &actor, const std::string &topic)
     {
         MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
         if(!sndMgr->sayDone(actor))
@@ -631,6 +554,18 @@ namespace MWDialogue
             return;
         }
 
+        if (actor.getClass().isNpc() && MWBase::Environment::get().getWorld()->isSwimming(actor))
+        {
+            // NPCs don't talk while submerged
+            return;
+        }
+
+        if (actor.getClass().getCreatureStats(actor).getKnockedDown())
+        {
+            // Unconscious actors can not speak
+            return;
+        }
+
         const MWWorld::ESMStore &store = MWBase::Environment::get().getWorld()->getStore();
         const ESM::Dialogue *dial = store.get<ESM::Dialogue>().find(topic);
 
@@ -644,6 +579,8 @@ namespace MWDialogue
                 winMgr->messageBox(info->mResponse);
             if (!info->mSound.empty())
                 sndMgr->say(actor, info->mSound);
+            if (!info->mResultScript.empty())
+                executeScript(info->mResultScript, actor);
         }
     }
 
@@ -740,7 +677,7 @@ namespace MWDialogue
         if (actor == mActor && !mLastTopic.empty())
         {
             MWBase::Environment::get().getJournal()->removeLastAddedTopicResponse(
-                        mLastTopic, actor.getClass().getName(actor));
+                        Misc::StringUtils::lowerCase(mLastTopic), actor.getClass().getName(actor));
         }
     }
 }
