@@ -21,7 +21,9 @@ namespace MWGui
         std::string getText() const;
         void setButtons(ButtonList &buttons);
 
-        virtual void open();
+        virtual void onOpen();
+
+        bool exit() { return false; }
 
         // Events
         typedef MyGUI::delegates::CMultiDelegate1<int> EventHandle_Int;
@@ -67,6 +69,8 @@ namespace MWGui
         std::string getClassId() const;
         void setClassId(const std::string &classId);
 
+        bool exit() { return false; }
+
         // Events
         typedef MyGUI::delegates::CMultiDelegate0 EventHandle_Void;
 
@@ -100,7 +104,9 @@ namespace MWGui
         void setClassId(const std::string &classId);
 
         void setNextButtonShow(bool shown);
-        virtual void open();
+        virtual void onOpen();
+
+        bool exit() { return false; }
 
         // Events
         typedef MyGUI::delegates::CMultiDelegate0 EventHandle_Void;
@@ -142,7 +148,7 @@ namespace MWGui
         SelectSpecializationDialog();
         ~SelectSpecializationDialog();
 
-        virtual void exit();
+        virtual bool exit();
 
         ESM::Class::Specialization getSpecializationId() const { return mSpecializationId; }
 
@@ -175,7 +181,7 @@ namespace MWGui
         SelectAttributeDialog();
         ~SelectAttributeDialog();
 
-        virtual void exit();
+        virtual bool exit();
 
         ESM::Attribute::AttributeID getAttributeId() const { return mAttributeId; }
 
@@ -206,7 +212,7 @@ namespace MWGui
         SelectSkillDialog();
         ~SelectSkillDialog();
 
-        virtual void exit();
+        virtual bool exit();
 
         ESM::Skill::SkillEnum getSkillId() const { return mSkillId; }
 
@@ -262,6 +268,8 @@ namespace MWGui
         CreateClassDialog();
         virtual ~CreateClassDialog();
 
+        bool exit() { return false; }
+
         std::string getName() const;
         std::string getDescription() const;
         ESM::Class::Specialization getSpecializationId() const;
