@@ -8,16 +8,13 @@
 
 #include <MyGUI_TextBox.h>
 #include <MyGUI_Button.h>
-
-#include <boost/bind.hpp>
-#include <boost/function.hpp>
+#include <MyGUI_InputManager.h>
 
 #include <components/misc/stringops.hpp>
 #include <components/widgets/imagebutton.hpp>
 #include <components/widgets/list.hpp>
 
 #include "../mwbase/environment.hpp"
-#include "../mwbase/soundmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/journal.hpp"
 
@@ -48,7 +45,7 @@ namespace
     static char const LeftTopicIndex [] = "LeftTopicIndex";
     static char const RightTopicIndex [] = "RightTopicIndex";
 
-    struct JournalWindowImpl : MWGui::WindowBase, MWGui::JournalBooks, MWGui::JournalWindow
+    struct JournalWindowImpl : MWGui::JournalBooks, MWGui::JournalWindow
     {
         struct DisplayState
         {
@@ -62,6 +59,7 @@ namespace
         Book mTopicIndexBook;
         bool mQuestMode;
         bool mOptionsMode;
+        bool mTopicsMode;
         bool mAllQuests;
 
         template <typename T>
@@ -87,19 +85,24 @@ namespace
 
         void adviseButtonClick (char const * name, void (JournalWindowImpl::*Handler) (MyGUI::Widget* _sender))
         {
-            getWidget <Gui::ImageButton> (name) ->
+            getWidget <MyGUI::Widget> (name) ->
                 eventMouseButtonClick += newDelegate(this, Handler);
         }
 
+        void adviseKeyPress (char const * name, void (JournalWindowImpl::*Handler) (MyGUI::Widget* _sender, MyGUI::KeyCode key, MyGUI::Char character))
+        {
+            getWidget <MyGUI::Widget> (name) ->
+                eventKeyButtonPressed += newDelegate(this, Handler);
+        }
+
         MWGui::BookPage* getPage (char const * name)
         {
             return getWidget <MWGui::BookPage> (name);
         }
 
         JournalWindowImpl (MWGui::JournalViewModel::Ptr Model, bool questList)
-            : WindowBase("openmw_journal.layout"), JournalBooks (Model)
+            : JournalBooks (Model), JournalWindow()
         {
-            mMainWidget->setVisible(false);
             center();
 
             adviseButtonClick (OptionsBTN,    &JournalWindowImpl::notifyOptions   );
@@ -115,6 +118,12 @@ namespace
             adviseButtonClick (ShowAllBTN,    &JournalWindowImpl::notifyShowAll   );
             adviseButtonClick (ShowActiveBTN, &JournalWindowImpl::notifyShowActive);
 
+            adviseKeyPress (OptionsBTN, &JournalWindowImpl::notifyKeyPress);
+            adviseKeyPress (PrevPageBTN, &JournalWindowImpl::notifyKeyPress);
+            adviseKeyPress (NextPageBTN, &JournalWindowImpl::notifyKeyPress);
+            adviseKeyPress (CloseBTN, &JournalWindowImpl::notifyKeyPress);
+            adviseKeyPress (JournalBTN, &JournalWindowImpl::notifyKeyPress);
+
             Gui::MWList* list = getWidget<Gui::MWList>(QuestsList);
             list->eventItemSelected += MyGUI::newDelegate(this, &JournalWindowImpl::notifyQuestClicked);
 
@@ -124,7 +133,7 @@ namespace
             {
                 MWGui::BookPage::ClickCallback callback;
                 
-                callback = boost::bind (&JournalWindowImpl::notifyTopicClicked, this, _1);
+                callback = std::bind (&JournalWindowImpl::notifyTopicClicked, this, std::placeholders::_1);
 
                 getPage (LeftBookPage)->adviseLinkClicked (callback);
                 getPage (RightBookPage)->adviseLinkClicked (callback);
@@ -136,7 +145,7 @@ namespace
             {
                 MWGui::BookPage::ClickCallback callback;
                 
-                callback = boost::bind (&JournalWindowImpl::notifyIndexLinkClicked, this, _1);
+                callback = std::bind(&JournalWindowImpl::notifyIndexLinkClicked, this, std::placeholders::_1);
 
                 getPage (LeftTopicIndex)->adviseLinkClicked (callback);
                 getPage (RightTopicIndex)->adviseLinkClicked (callback);
@@ -190,16 +199,17 @@ namespace
             }
 
             adjustButton(TopicsBTN);
-            int width = getWidget<MyGUI::Widget>(TopicsBTN)->getSize().width + getWidget<MyGUI::Widget>(QuestsBTN)->getSize().width;
             int topicsWidth = getWidget<MyGUI::Widget>(TopicsBTN)->getSize().width;
-            int pageWidth = getWidget<MyGUI::Widget>(RightBookPage)->getSize().width;
+            int cancelLeft = getWidget<MyGUI::Widget>(CancelBTN)->getPosition().left;
+            int cancelRight = getWidget<MyGUI::Widget>(CancelBTN)->getPosition().left + getWidget<MyGUI::Widget>(CancelBTN)->getSize().width;
 
-            getWidget<MyGUI::Widget>(TopicsBTN)->setPosition((pageWidth - width)/2, getWidget<MyGUI::Widget>(TopicsBTN)->getPosition().top);
-            getWidget<MyGUI::Widget>(QuestsBTN)->setPosition((pageWidth - width)/2 + topicsWidth, getWidget<MyGUI::Widget>(QuestsBTN)->getPosition().top);
+            getWidget<MyGUI::Widget>(TopicsBTN)->setPosition(cancelLeft - topicsWidth, getWidget<MyGUI::Widget>(TopicsBTN)->getPosition().top);
+            getWidget<MyGUI::Widget>(QuestsBTN)->setPosition(cancelRight, getWidget<MyGUI::Widget>(QuestsBTN)->getPosition().top);
 
             mQuestMode = false;
             mAllQuests = false;
             mOptionsMode = false;
+            mTopicsMode = false;
         }
 
         void adjustButton (char const * name)
@@ -213,7 +223,7 @@ namespace
                 button->setPosition(button->getPosition() + MyGUI::IntPoint(diff.width,0));
         }
 
-        void open()
+        void onOpen()
         {
             if (!MWBase::Environment::get().getWindowManager ()->getJournalAllowed ())
             {
@@ -240,9 +250,11 @@ namespace
                     --page;
             }
             updateShowingPages();
+
+            MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(getWidget<MyGUI::Widget>(CloseBTN));
         }
 
-        void close()
+        void onClose()
         {
             mModel->unload ();
 
@@ -263,6 +275,7 @@ namespace
         void setBookMode ()
         {
             mOptionsMode = false;
+            mTopicsMode = false;
             setVisible (OptionsBTN, true);
             setVisible (OptionsOverlay, false);
 
@@ -273,6 +286,7 @@ namespace
         void setOptionsMode ()
         {
             mOptionsMode = true;
+            mTopicsMode = false;
 
             setVisible (OptionsBTN, false);
             setVisible (OptionsOverlay, true);
@@ -296,6 +310,8 @@ namespace
             // If in quest mode, ensure the quest list is updated
             if (mQuestMode)
                 notifyQuests(getWidget<MyGUI::Widget>(QuestsList));
+            else
+                notifyTopics(getWidget<MyGUI::Widget>(TopicsList));
         }
 
         void pushBook (Book book, unsigned int page)
@@ -347,8 +363,19 @@ namespace
                 relPages = 0;
             }
 
-            setVisible (PrevPageBTN, page > 0);
-            setVisible (NextPageBTN, relPages > 2);
+            MyGUI::Widget* nextPageBtn = getWidget<MyGUI::Widget>(NextPageBTN);
+            MyGUI::Widget* prevPageBtn = getWidget<MyGUI::Widget>(PrevPageBTN);
+
+            MyGUI::Widget* focus = MyGUI::InputManager::getInstance().getKeyFocusWidget();
+            bool nextPageVisible = relPages > 2;
+            nextPageBtn->setVisible(nextPageVisible);
+            bool prevPageVisible = page > 0;
+            prevPageBtn->setVisible(prevPageVisible);
+
+            if (focus == nextPageBtn && !nextPageVisible && prevPageVisible)
+                MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(prevPageBtn);
+            else if (focus == prevPageBtn && !prevPageVisible && nextPageVisible)
+                MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(nextPageBtn);
 
             setVisible (PageOneNum, relPages > 0);
             setVisible (PageTwoNum, relPages > 1);
@@ -360,6 +387,14 @@ namespace
             setText (PageTwoNum, page + 2);
         }
 
+        void notifyKeyPress(MyGUI::Widget* sender, MyGUI::KeyCode key, MyGUI::Char character)
+        {
+            if (key == MyGUI::KeyCode::ArrowUp)
+                notifyPrevPage(sender);
+            else if (key == MyGUI::KeyCode::ArrowDown)
+                notifyNextPage(sender);
+        }
+
         void notifyTopicClicked (intptr_t linkId)
         {
             Book topicBook = createTopicBook (linkId);
@@ -374,6 +409,9 @@ namespace
             setVisible (JournalBTN, true);
 
             mOptionsMode = false;
+            mTopicsMode = false;
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         void notifyTopicSelected (const std::string& topic, int id)
@@ -403,6 +441,8 @@ namespace
             setVisible (JournalBTN, true);
 
             mOptionsMode = false;
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         void notifyOptions(MyGUI::Widget* _sender)
@@ -420,6 +460,8 @@ namespace
         {
             assert (mStates.size () > 1);
             popBook ();
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         void notifyIndexLinkClicked (MWGui::TypesetBook::InteractiveId character)
@@ -428,6 +470,8 @@ namespace
             setVisible (RightTopicIndex, false);
             setVisible (TopicsList, true);
 
+            mTopicsMode = true;
+
             Gui::MWList* list = getWidget<Gui::MWList>(TopicsList);
             list->clear();
 
@@ -436,17 +480,22 @@ namespace
             mModel->visitTopicNamesStartingWith((char) character, add);
 
             list->adjustSize();
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         void notifyTopics(MyGUI::Widget* _sender)
         {
             mQuestMode = false;
+            mTopicsMode = false;
             setVisible (LeftTopicIndex, true);
             setVisible (RightTopicIndex, true);
             setVisible (TopicsList, false);
             setVisible (QuestsList, false);
             setVisible (ShowAllBTN, false);
             setVisible (ShowActiveBTN, false);
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         struct AddNamesToList
@@ -498,6 +547,8 @@ namespace
                 SetNamesInactive setInactive(list);
                 mModel->visitQuestNames(!mAllQuests, setInactive);
             }
+
+            MWBase::Environment::get().getWindowManager()->playSound("book page");
         }
 
         void notifyShowAll(MyGUI::Widget* _sender)
@@ -514,13 +565,23 @@ namespace
 
         void notifyCancel(MyGUI::Widget* _sender)
         {
-            setBookMode ();
+            if (mTopicsMode)
+            {
+                notifyTopics(_sender);
+            }
+            else
+            {
+                setBookMode();
+                MWBase::Environment::get().getWindowManager()->playSound("book page");
+            }
+
         }
 
         void notifyClose(MyGUI::Widget* _sender)
         {
-            MWBase::Environment::get().getSoundManager()->playSound ("book close", 1.0, 1.0);
-            MWBase::Environment::get().getWindowManager ()->popGuiMode ();
+            MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
+            winMgr->playSound("book close");
+            winMgr->popGuiMode();
         }
 
         void notifyMouseWheel(MyGUI::Widget* sender, int rel)
@@ -542,6 +603,8 @@ namespace
 
                 if (page+2 < book->pageCount())
                 {
+                    MWBase::Environment::get().getWindowManager()->playSound("book page");
+
                     page += 2;
                     updateShowingPages ();
                 }
@@ -558,6 +621,8 @@ namespace
 
                 if(page >= 2)
                 {
+                    MWBase::Environment::get().getWindowManager()->playSound("book page");
+
                     page -= 2;
                     updateShowingPages ();
                 }
@@ -571,3 +636,9 @@ MWGui::JournalWindow * MWGui::JournalWindow::create (JournalViewModel::Ptr Model
 {
     return new JournalWindowImpl (Model, questList);
 }
+
+MWGui::JournalWindow::JournalWindow()
+    :WindowBase("openmw_journal.layout")
+{
+
+}
