@@ -38,6 +38,7 @@ namespace MWGui
         , mLoadingOnTime(0.0)
         , mImportantLabel(false)
         , mProgress(0)
+        , mShowWallpaper(true)
     {
         mMainWidget->setSize(MyGUI::RenderManager::getInstance().getViewSize());
 
@@ -49,8 +50,8 @@ namespace MWGui
 
         mBackgroundImage = MyGUI::Gui::getInstance().createWidgetReal<BackgroundImage>("ImageBox", 0,0,1,1,
             MyGUI::Align::Stretch, "Menu");
-
-        setVisible(false);
+        mSceneImage = MyGUI::Gui::getInstance().createWidgetReal<BackgroundImage>("ImageBox", 0,0,1,1,
+            MyGUI::Align::Stretch, "Scene");
 
         findSplashScreens();
     }
@@ -65,15 +66,27 @@ namespace MWGui
         std::string pattern = "Splash/";
         mVFS->normalizeFilename(pattern);
 
-        std::map<std::string, VFS::File*>::const_iterator found = index.lower_bound(pattern);
+        /* priority given to the left */
+        std::list<std::string> supported_extensions = {".tga", ".dds", ".ktx", ".png", ".bmp", ".jpeg", ".jpg"};
+
+        auto found = index.lower_bound(pattern);
         while (found != index.end())
         {
             const std::string& name = found->first;
             if (name.size() >= pattern.size() && name.substr(0, pattern.size()) == pattern)
             {
                 size_t pos = name.find_last_of('.');
-                if (pos != std::string::npos && name.compare(pos, name.size()-pos, ".tga") == 0)
-                    mSplashScreens.push_back(found->first);
+                if (pos != std::string::npos)
+                {
+                    for(auto const extension: supported_extensions)
+                    {
+                        if (name.compare(pos, name.size() - pos, extension) == 0)
+                        {
+                            mSplashScreens.push_back(found->first);
+                            break;  /* based on priority */
+                        }
+                    }
+                }
             }
             else
                 break;
@@ -99,19 +112,31 @@ namespace MWGui
     {
         WindowBase::setVisible(visible);
         mBackgroundImage->setVisible(visible);
+        mSceneImage->setVisible(visible);
+    }
+
+    double LoadingScreen::getTargetFrameRate() const
+    {
+        double frameRateLimit = MWBase::Environment::get().getFrameRateLimit();
+        if (frameRateLimit > 0)
+            return std::min(frameRateLimit, mTargetFrameRate);
+        else
+            return mTargetFrameRate;
     }
 
     class CopyFramebufferToTextureCallback : public osg::Camera::DrawCallback
     {
     public:
-        CopyFramebufferToTextureCallback(osg::Texture2D* texture, int w, int h)
-            : mTexture(texture), mWidth(w), mHeight(h)
+        CopyFramebufferToTextureCallback(osg::Texture2D* texture)
+            : mTexture(texture)
         {
         }
 
         virtual void operator () (osg::RenderInfo& renderInfo) const
         {
-            mTexture->copyTexImage2D(*renderInfo.getState(), 0, 0, mWidth, mHeight);
+            int w = renderInfo.getCurrentCamera()->getViewport()->width();
+            int h = renderInfo.getCurrentCamera()->getViewport()->height();
+            mTexture->copyTexImage2D(*renderInfo.getState(), 0, 0, w, h);
 
             // Callback removes itself when done
             if (renderInfo.getCurrentCamera())
@@ -120,7 +145,12 @@ namespace MWGui
 
     private:
         osg::ref_ptr<osg::Texture2D> mTexture;
-        int mWidth, mHeight;
+    };
+
+    class DontComputeBoundCallback : public osg::Node::ComputeBoundingSphereCallback
+    {
+    public:
+        virtual osg::BoundingSphere computeBound(const osg::Node&) const { return osg::BoundingSphere(); }
     };
 
     void LoadingScreen::loadingOn()
@@ -133,48 +163,24 @@ namespace MWGui
         if (mViewer->getIncrementalCompileOperation())
         {
             mViewer->getIncrementalCompileOperation()->setMaximumNumOfObjectsToCompilePerFrame(100);
-            mViewer->getIncrementalCompileOperation()->setTargetFrameRate(mTargetFrameRate);
+            mViewer->getIncrementalCompileOperation()->setTargetFrameRate(getTargetFrameRate());
         }
 
-        bool showWallpaper = (MWBase::Environment::get().getStateManager()->getState()
+        // Assign dummy bounding sphere callback to avoid the bounding sphere of the entire scene being recomputed after each frame of loading
+        // We are already using node masks to avoid the scene from being updated/rendered, but node masks don't work for computeBound()
+        mViewer->getSceneData()->setComputeBoundingSphereCallback(new DontComputeBoundCallback);
+
+        mShowWallpaper = (MWBase::Environment::get().getStateManager()->getState()
                 == MWBase::StateManager::State_NoGame);
 
-        if (!showWallpaper)
-        {
-            // Copy the current framebuffer onto a texture and display that texture as the background image
-            // Note, we could also set the camera to disable clearing and have the background image transparent,
-            // but then we get shaking effects on buffer swaps.
-
-            if (!mTexture)
-            {
-                mTexture = new osg::Texture2D;
-                mTexture->setInternalFormat(GL_RGB);
-                mTexture->setResizeNonPowerOfTwoHint(false);
-            }
-
-            int width = mViewer->getCamera()->getViewport()->width();
-            int height = mViewer->getCamera()->getViewport()->height();
-            mViewer->getCamera()->setInitialDrawCallback(new CopyFramebufferToTextureCallback(mTexture, width, height));
-
-            if (!mGuiTexture.get())
-            {
-                mGuiTexture.reset(new osgMyGUI::OSGTexture(mTexture));
-            }
-
-            mBackgroundImage->setBackgroundImage("");
-
-            mBackgroundImage->setRenderItemTexture(mGuiTexture.get());
-            mBackgroundImage->getSubWidgetMain()->_setUVSet(MyGUI::FloatRect(0.f, 0.f, 1.f, 1.f));
-        }
-
         setVisible(true);
 
-        if (showWallpaper)
+        if (mShowWallpaper)
         {
             changeWallpaper();
         }
 
-        MWBase::Environment::get().getWindowManager()->pushGuiMode(showWallpaper ? GM_LoadingWallpaper : GM_Loading);
+        MWBase::Environment::get().getWindowManager()->pushGuiMode(mShowWallpaper ? GM_LoadingWallpaper : GM_Loading);
     }
 
     void LoadingScreen::loadingOff()
@@ -192,6 +198,9 @@ namespace MWGui
         else
             mImportantLabel = false; // label was already shown on loading screen
 
+        mViewer->getSceneData()->setComputeBoundingSphereCallback(NULL);
+        mViewer->getSceneData()->dirtyBound();
+
         //std::cout << "loading took " << mTimer.time_m() - mLoadingOnTime << std::endl;
         setVisible(false);
 
@@ -208,8 +217,11 @@ namespace MWGui
             // TODO: add option (filename pattern?) to use image aspect ratio instead of 4:3
             // we can't do this by default, because the Morrowind splash screens are 1024x1024, but should be displayed as 4:3
             bool stretch = Settings::Manager::getBool("stretch menu background", "GUI");
+            mBackgroundImage->setVisible(true);
             mBackgroundImage->setBackgroundImage(randomSplash, true, stretch);
         }
+        mSceneImage->setBackgroundImage("");
+        mSceneImage->setVisible(false);
     }
 
     void LoadingScreen::setProgressRange (size_t range)
@@ -223,7 +235,7 @@ namespace MWGui
     void LoadingScreen::setProgress (size_t value)
     {
         // skip expensive update if there isn't enough visible progress
-        if (value - mProgress < mProgressBar->getScrollRange()/200.f)
+        if (mProgressBar->getWidth() <= 0 || value - mProgress < mProgressBar->getScrollRange()/mProgressBar->getWidth())
             return;
         value = std::min(value, mProgressBar->getScrollRange()-1);
         mProgress = value;
@@ -244,7 +256,7 @@ namespace MWGui
 
     bool LoadingScreen::needToDrawLoadingScreen()
     {
-        if ( mTimer.time_m() <= mLastRenderTime + (1.0/mTargetFrameRate) * 1000.0)
+        if ( mTimer.time_m() <= mLastRenderTime + (1.0/getTargetFrameRate()) * 1000.0)
             return false;
 
         // the minimal delay before a loading screen shows
@@ -260,31 +272,60 @@ namespace MWGui
             diff -= mProgress / static_cast<float>(mProgressBar->getScrollRange()) * 100.f;
         }
 
-        bool showWallpaper = (MWBase::Environment::get().getStateManager()->getState()
-                == MWBase::StateManager::State_NoGame);
-        if (!showWallpaper && diff < initialDelay*1000)
+        if (!mShowWallpaper && diff < initialDelay*1000)
             return false;
         return true;
     }
 
+    void LoadingScreen::setupCopyFramebufferToTextureCallback()
+    {
+        // Copy the current framebuffer onto a texture and display that texture as the background image
+        // Note, we could also set the camera to disable clearing and have the background image transparent,
+        // but then we get shaking effects on buffer swaps.
+
+        if (!mTexture)
+        {
+            mTexture = new osg::Texture2D;
+            mTexture->setInternalFormat(GL_RGB);
+            mTexture->setResizeNonPowerOfTwoHint(false);
+        }
+
+        if (!mGuiTexture.get())
+        {
+            mGuiTexture.reset(new osgMyGUI::OSGTexture(mTexture));
+        }
+
+        mViewer->getCamera()->setInitialDrawCallback(new CopyFramebufferToTextureCallback(mTexture));
+
+        mBackgroundImage->setBackgroundImage("");
+        mBackgroundImage->setVisible(false);
+
+        mSceneImage->setRenderItemTexture(mGuiTexture.get());
+        mSceneImage->getSubWidgetMain()->_setUVSet(MyGUI::FloatRect(0.f, 0.f, 1.f, 1.f));
+        mSceneImage->setVisible(true);
+    }
+
     void LoadingScreen::draw()
     {
         if (!needToDrawLoadingScreen())
             return;
 
-        bool showWallpaper = (MWBase::Environment::get().getStateManager()->getState()
-                == MWBase::StateManager::State_NoGame);
-        if (showWallpaper && mTimer.time_m() > mLastWallpaperChangeTime + 5000*1)
+        if (mShowWallpaper && mTimer.time_m() > mLastWallpaperChangeTime + 5000*1)
         {
             mLastWallpaperChangeTime = mTimer.time_m();
             changeWallpaper();
         }
 
+        if (!mShowWallpaper && mLastRenderTime < mLoadingOnTime)
+        {
+            setupCopyFramebufferToTextureCallback();
+        }
+
         // Turn off rendering except the GUI
         int oldUpdateMask = mViewer->getUpdateVisitor()->getTraversalMask();
         int oldCullMask = mViewer->getCamera()->getCullMask();
-        mViewer->getUpdateVisitor()->setTraversalMask(MWRender::Mask_GUI);
-        mViewer->getCamera()->setCullMask(MWRender::Mask_GUI);
+        mViewer->getUpdateVisitor()->setTraversalMask(MWRender::Mask_GUI|MWRender::Mask_PreCompile);
+        mViewer->getCamera()->setCullMask(MWRender::Mask_GUI|MWRender::Mask_PreCompile);
 
         MWBase::Environment::get().getInputManager()->update(0, true, true);
 
