@@ -2,6 +2,7 @@
 #define MWGUI_RACE_H
 
 #include "windowbase.hpp"
+#include <MyGUI_RenderManager.h>
 
 
 namespace MWGui
@@ -50,8 +51,10 @@ namespace MWGui
         void setGender(Gender gender) { mGenderIndex = gender == GM_Male ? 0 : 1; }
 
         void setNextButtonShow(bool shown);
-        virtual void open();
-        virtual void close();
+        virtual void onOpen();
+        virtual void onClose();
+
+        bool exit() { return false; }
 
         // Events
         typedef MyGUI::delegates::CMultiDelegate0 EventHandle_Void;
@@ -115,8 +118,8 @@ namespace MWGui
 
         float mCurrentAngle;
 
-        std::auto_ptr<MWRender::RaceSelectionPreview> mPreview;
-        std::auto_ptr<MyGUI::ITexture> mPreviewTexture;
+        std::unique_ptr<MWRender::RaceSelectionPreview> mPreview;
+        std::unique_ptr<MyGUI::ITexture> mPreviewTexture;
 
         bool mPreviewDirty;
     };
