@@ -40,6 +40,7 @@ namespace MWGui
         , mEnabled(true)
         , mFullHelp(false)
         , mShowOwned(0)
+        , mFrameDuration(0.f)
     {
         getWidget(mDynamicToolTipBox, "DynamicToolTipBox");
 
@@ -67,6 +68,11 @@ namespace MWGui
     }
 
     void ToolTips::onFrame(float frameDuration)
+    {
+        mFrameDuration = frameDuration;
+    }
+
+    void ToolTips::update(float frameDuration)
     {
 
         while (mDynamicToolTipBox->getChildCount())
@@ -91,6 +97,8 @@ namespace MWGui
 
         if (guiMode)
         {
+            if (!MWBase::Environment::get().getWindowManager()->getCursorVisible())
+                return;
             const MyGUI::IntPoint& mousePos = MyGUI::InputManager::getInstance().getMousePosition();
 
             if (MWBase::Environment::get().getWindowManager()->getWorldMouseOver() && ((MWBase::Environment::get().getWindowManager()->getMode() == GM_Console)
@@ -112,7 +120,7 @@ namespace MWGui
                     if (info.caption.empty())
                         info.caption=mFocusObject.getCellRef().getRefId();
                     info.icon="";
-                    tooltipSize = createToolTip(info, true);
+                    tooltipSize = createToolTip(info, checkOwned());
                 }
                 else
                     tooltipSize = getToolTipViaPtr(mFocusObject.getRefData().getCount(), true);
@@ -178,22 +186,23 @@ namespace MWGui
                     ToolTipInfo info;
                     info.text = data.caption;
                     info.notes = data.notes;
-                    tooltipSize = createToolTip(info, false);
+                    tooltipSize = createToolTip(info);
                 }
                 else if (type == "ItemPtr")
                 {
                     mFocusObject = *focus->getUserData<MWWorld::Ptr>();
-                    tooltipSize = getToolTipViaPtr(mFocusObject.getRefData().getCount(), false);
+                    tooltipSize = getToolTipViaPtr(mFocusObject.getRefData().getCount(), false, checkOwned());
                 }
                 else if (type == "ItemModelIndex")
                 {
                     std::pair<ItemModel::ModelIndex, ItemModel*> pair = *focus->getUserData<std::pair<ItemModel::ModelIndex, ItemModel*> >();
                     mFocusObject = pair.second->getItem(pair.first).mBase;
-                    tooltipSize = getToolTipViaPtr(pair.second->getItem(pair.first).mCount, false);
+                    bool isAllowedToUse = pair.second->allowedToUseItems();
+                    tooltipSize = getToolTipViaPtr(pair.second->getItem(pair.first).mCount, false, !isAllowedToUse);
                 }
                 else if (type == "ToolTipInfo")
                 {
-                    tooltipSize = createToolTip(*focus->getUserData<MWGui::ToolTipInfo>(), false);
+                    tooltipSize = createToolTip(*focus->getUserData<MWGui::ToolTipInfo>());
                 }
                 else if (type == "AvatarItemSelection")
                 {
@@ -236,7 +245,7 @@ namespace MWGui
                         info.text = "#{sSchool}: " + sSchoolNames[school];
                     }
                     info.effects = effects;
-                    tooltipSize = createToolTip(info, false);
+                    tooltipSize = createToolTip(info);
                 }
                 else if (type == "Layout")
                 {
@@ -290,7 +299,7 @@ namespace MWGui
         {
             if (!mFocusObject.isEmpty())
             {
-                MyGUI::IntSize tooltipSize = getToolTipViaPtr(mFocusObject.getRefData().getCount());
+                MyGUI::IntSize tooltipSize = getToolTipViaPtr(mFocusObject.getRefData().getCount(), true, checkOwned());
 
                 setCoord(viewSize.width/2 - tooltipSize.width/2,
                         std::max(0, int(mFocusToolTipY*viewSize.height - tooltipSize.height)),
@@ -317,12 +326,14 @@ namespace MWGui
         }
     }
 
-    void ToolTips::setFocusObject(const MWWorld::ConstPtr& focus)
+    void ToolTips::setFocusObject(const MWWorld::Ptr& focus)
     {
         mFocusObject = focus;
+
+        update(mFrameDuration);
     }
 
-    MyGUI::IntSize ToolTips::getToolTipViaPtr (int count, bool image)
+    MyGUI::IntSize ToolTips::getToolTipViaPtr (int count, bool image, bool isOwned)
     {
         // this the maximum width of the tooltip before it starts word-wrapping
         setCoord(0, 0, 300, 300);
@@ -341,7 +352,7 @@ namespace MWGui
             ToolTipInfo info = object.getToolTipInfo(mFocusObject, count);
             if (!image)
                 info.icon = "";
-            tooltipSize = createToolTip(info, true);
+            tooltipSize = createToolTip(info, isOwned);
         }
 
         return tooltipSize;
@@ -349,38 +360,24 @@ namespace MWGui
     
     bool ToolTips::checkOwned()
     {
-        if(!mFocusObject.isEmpty())
-        {
-            const MWWorld::CellRef& cellref = mFocusObject.getCellRef();
-            MWWorld::Ptr ptr = MWMechanics::getPlayer();
-            MWWorld::Ptr victim;
-            
-            MWBase::MechanicsManager* mm = MWBase::Environment::get().getMechanicsManager();
-            bool allowed = mm->isAllowedToUse(ptr, cellref, victim); 
-
-            return !allowed;
-        }
-        else
-        {
+        if(mFocusObject.isEmpty())
             return false;
-        }
+
+        MWWorld::Ptr ptr = MWMechanics::getPlayer();
+        MWWorld::Ptr victim;
+
+        MWBase::MechanicsManager* mm = MWBase::Environment::get().getMechanicsManager();
+        return !mm->isAllowedToUse(ptr, mFocusObject, victim);
     }
 
-    MyGUI::IntSize ToolTips::createToolTip(const MWGui::ToolTipInfo& info, bool isFocusObject)
+    MyGUI::IntSize ToolTips::createToolTip(const MWGui::ToolTipInfo& info, bool isOwned)
     {
         mDynamicToolTipBox->setVisible(true);
         
-        if(mShowOwned == 1 || mShowOwned == 3)
-        {
-            if(isFocusObject && checkOwned())
-            {
-                mDynamicToolTipBox->changeWidgetSkin("HUD_Box_NoTransp_Owned");
-            }
-            else
-            {
-                mDynamicToolTipBox->changeWidgetSkin("HUD_Box_NoTransp");
-            }
-        }
+        if((mShowOwned == 1 || mShowOwned == 3) && isOwned)
+            mDynamicToolTipBox->changeWidgetSkin(MWBase::Environment::get().getWindowManager()->isGuiMode() ? "HUD_Box_NoTransp_Owned" : "HUD_Box_Owned");
+        else
+            mDynamicToolTipBox->changeWidgetSkin(MWBase::Environment::get().getWindowManager()->isGuiMode() ? "HUD_Box_NoTransp" : "HUD_Box");
 
         std::string caption = info.caption;
         std::string image = info.icon;
@@ -414,26 +411,28 @@ namespace MWGui
 
         const MyGUI::IntPoint padding(8, 8);
 
-        const int maximumWidth = 500;
-
         const int imageCaptionHPadding = (caption != "" ? 8 : 0);
         const int imageCaptionVPadding = (caption != "" ? 4 : 0);
 
+        const int maximumWidth = MyGUI::RenderManager::getInstance().getViewSize().width - imageCaptionHPadding * 2;
+
         std::string realImage = MWBase::Environment::get().getWindowManager()->correctIconPath(image);
 
         MyGUI::EditBox* captionWidget = mDynamicToolTipBox->createWidget<MyGUI::EditBox>("NormalText", MyGUI::IntCoord(0, 0, 300, 300), MyGUI::Align::Left | MyGUI::Align::Top, "ToolTipCaption");
-        captionWidget->setProperty("Static", "true");
+        captionWidget->setEditStatic(true);
+        captionWidget->setNeedKeyFocus(false);
         captionWidget->setCaptionWithReplacing(caption);
         MyGUI::IntSize captionSize = captionWidget->getTextSize();
 
         int captionHeight = std::max(caption != "" ? captionSize.height : 0, imageSize);
 
         MyGUI::EditBox* textWidget = mDynamicToolTipBox->createWidget<MyGUI::EditBox>("SandText", MyGUI::IntCoord(0, captionHeight+imageCaptionVPadding, 300, 300-captionHeight-imageCaptionVPadding), MyGUI::Align::Stretch, "ToolTipText");
-        textWidget->setProperty("Static", "true");
-        textWidget->setProperty("MultiLine", "true");
-        textWidget->setProperty("WordWrap", info.wordWrap ? "true" : "false");
+        textWidget->setEditStatic(true);
+        textWidget->setEditMultiLine(true);
+        textWidget->setEditWordWrap(info.wordWrap);
         textWidget->setCaptionWithReplacing(text);
         textWidget->setTextAlign(MyGUI::Align::HCenter | MyGUI::Align::Top);
+        textWidget->setNeedKeyFocus(false);
         MyGUI::IntSize textSize = textWidget->getTextSize();
 
         captionSize += MyGUI::IntSize(imageSize, 0); // adjust for image
@@ -594,6 +593,14 @@ namespace MWGui
             return "\n" + prefix + ": " + toString(weight);
     }
 
+    std::string ToolTips::getPercentString(const float value, const std::string& prefix)
+    {
+        if (value == 0)
+            return "";
+        else
+            return "\n" + prefix + ": " + toString(value*100) +"%";
+    }
+
     std::string ToolTips::getValueString(const int value, const std::string& prefix)
     {
         if (value == 0)
@@ -701,20 +708,28 @@ namespace MWGui
 
     void ToolTips::createSpecializationToolTip(MyGUI::Widget* widget, const std::string& name, int specId)
     {
-        widget->setUserString("Caption_CenteredCaption", name);
+        widget->setUserString("Caption_Caption", name);
         std::string specText;
         // get all skills of this specialisation
         const MWWorld::Store<ESM::Skill> &skills =
             MWBase::Environment::get().getWorld()->getStore().get<ESM::Skill>();
 
+        bool isFirst = true;
         MWWorld::Store<ESM::Skill>::iterator it = skills.begin();
         for (; it != skills.end(); ++it)
         {
             if (it->second.mData.mSpecialization == specId)
-                specText += std::string("\n#{") + ESM::Skill::sSkillNameIds[it->first] + "}";
+            {
+                if (isFirst)
+                    isFirst = false;
+                else
+                    specText += "\n";
+
+                specText += std::string("#{") + ESM::Skill::sSkillNameIds[it->first] + "}";
+            }
         }
-        widget->setUserString("Caption_CenteredCaptionText", specText);
-        widget->setUserString("ToolTipLayout", "TextWithCenteredCaptionToolTip");
+        widget->setUserString("Caption_ColumnText", specText);
+        widget->setUserString("ToolTipLayout", "SpecializationToolTip");
         widget->setUserString("ToolTipType", "Layout");
     }
 
@@ -769,7 +784,7 @@ namespace MWGui
             {
                 if (it == categories[category].spells.begin())
                 {
-                    text += std::string("\n#{fontcolourhtml=header}") + std::string("#{") + categories[category].label + "}";
+                    text += std::string("\n\n#{fontcolourhtml=header}") + std::string("#{") + categories[category].label + "}";
                 }
 
                 const std::string &spellId = *it;
@@ -787,7 +802,7 @@ namespace MWGui
         widget->setUserString("Caption_CenteredCaption", playerRace->mName);
         widget->setUserString("Caption_CenteredCaptionText", playerRace->mDescription);
         widget->setUserString("ToolTipType", "Layout");
-        widget->setUserString("ToolTipLayout", "TextWithCenteredCaptionToolTip");
+        widget->setUserString("ToolTipLayout", "RaceToolTip");
     }
 
     void ToolTips::createClassToolTip(MyGUI::Widget* widget, const ESM::Class& playerClass)
