@@ -14,14 +14,18 @@ namespace MWGui
     public:
         TrainingWindow();
 
-        virtual void open();
+        virtual void onOpen();
 
-        virtual void exit();
+        bool exit();
 
-        void startTraining(MWWorld::Ptr actor);
+        void setPtr(const MWWorld::Ptr& actor);
 
         void onFrame(float dt);
 
+        WindowBase* getProgressBar() { return &mProgressBar; }
+
+        void clear() { resetReference(); }
+
     protected:
         virtual void onReferenceUnavailable ();
 
@@ -35,8 +39,6 @@ namespace MWGui
         MyGUI::Button* mCancelButton;
         MyGUI::TextBox* mPlayerGold;
 
-        float mFadeTimeRemaining;
-
         WaitDialogProgressBar mProgressBar;
         TimeAdvancer mTimeAdvancer;
     };
