@@ -24,9 +24,7 @@ namespace MWGui
         public:
             TravelWindow();
 
-            virtual void exit();
-
-            void startTravel(const MWWorld::Ptr& actor);
+            void setPtr (const MWWorld::Ptr& actor);
 
         protected:
             MyGUI::Button* mCancelButton;
