@@ -1,6 +1,8 @@
 #include "waitdialog.hpp"
 
 #include <MyGUI_ProgressBar.h>
+#include <MyGUI_InputManager.h>
+#include <MyGUI_ScrollBar.h>
 
 #include <components/misc/rng.hpp>
 
@@ -23,8 +25,6 @@
 
 #include "../mwstate/charactermanager.hpp"
 
-#include "widgets.hpp"
-
 namespace MWGui
 {
 
@@ -35,7 +35,7 @@ namespace MWGui
         getWidget(mProgressText, "ProgressText");
     }
 
-    void WaitDialogProgressBar::open()
+    void WaitDialogProgressBar::onOpen()
     {
         center();
     }
@@ -72,21 +72,49 @@ namespace MWGui
         mWaitButton->eventMouseButtonClick += MyGUI::newDelegate(this, &WaitDialog::onWaitButtonClicked);
         mHourSlider->eventScrollChangePosition += MyGUI::newDelegate(this, &WaitDialog::onHourSliderChangedPosition);
 
+        mCancelButton->eventKeyButtonPressed += MyGUI::newDelegate(this, &WaitDialog::onKeyButtonPressed);
+        mWaitButton->eventKeyButtonPressed += MyGUI::newDelegate(this, &WaitDialog::onKeyButtonPressed);
+        mUntilHealedButton->eventKeyButtonPressed += MyGUI::newDelegate(this, &WaitDialog::onKeyButtonPressed);
+
         mTimeAdvancer.eventProgressChanged += MyGUI::newDelegate(this, &WaitDialog::onWaitingProgressChanged);
         mTimeAdvancer.eventInterrupted += MyGUI::newDelegate(this, &WaitDialog::onWaitingInterrupted);
         mTimeAdvancer.eventFinished += MyGUI::newDelegate(this, &WaitDialog::onWaitingFinished);
-
-        mProgressBar.setVisible (false);
     }
 
-    void WaitDialog::exit()
+    void WaitDialog::setPtr(const MWWorld::Ptr &ptr)
     {
-        if(!mProgressBar.isVisible()) //Only exit if not currently waiting
-            MWBase::Environment::get().getWindowManager()->popGuiMode();
+        setCanRest(!ptr.isEmpty() || MWBase::Environment::get().getWorld ()->canRest () == 0);
+
+        if (mUntilHealedButton->getVisible())
+            MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mUntilHealedButton);
+        else
+            MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mWaitButton);
     }
 
-    void WaitDialog::open()
+    bool WaitDialog::exit()
     {
+        return (!mTimeAdvancer.isRunning()); //Only exit if not currently waiting
+    }
+
+    void WaitDialog::clear()
+    {
+        mSleeping = false;
+        mTimeAdvancer.stop();
+    }
+
+    void WaitDialog::onOpen()
+    {
+        if (mTimeAdvancer.isRunning())
+        {
+            mProgressBar.setVisible(true);
+            setVisible(false);
+            return;
+        }
+        else
+        {
+            mProgressBar.setVisible(false);
+        }
+
         if (!MWBase::Environment::get().getWindowManager ()->getRestEnabled ())
         {
             MWBase::Environment::get().getWindowManager()->popGuiMode ();
@@ -101,8 +129,6 @@ namespace MWGui
             MWBase::Environment::get().getWindowManager()->popGuiMode ();
         }
 
-        setCanRest(canRest == 0);
-
         onHourSliderChangedPosition(mHourSlider, 0);
         mHourSlider->setScrollPosition (0);
 
@@ -134,7 +160,7 @@ namespace MWGui
 
     void WaitDialog::startWaiting(int hoursToWait)
     {
-        if(Settings::Manager::getBool("autosave","Saves") && mSleeping) //autosaves when enabled and sleeping
+        if(Settings::Manager::getBool("autosave","Saves")) //autosaves when enabled
             MWBase::Environment::get().getStateManager()->quickSave("Autosave");
 
         MWBase::World* world = MWBase::Environment::get().getWorld();
@@ -177,13 +203,25 @@ namespace MWGui
 
     void WaitDialog::onCancelButtonClicked(MyGUI::Widget* sender)
     {
-        exit();
+        MWBase::Environment::get().getWindowManager()->removeGuiMode(GM_Rest);
     }
 
     void WaitDialog::onHourSliderChangedPosition(MyGUI::ScrollBar* sender, size_t position)
     {
         mHourText->setCaptionWithReplacing (MyGUI::utility::toString(position+1) + " #{sRestMenu2}");
         mManualHours = position+1;
+        MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mWaitButton);
+    }
+
+    void WaitDialog::onKeyButtonPressed(MyGUI::Widget *sender, MyGUI::KeyCode key, MyGUI::Char character)
+    {
+        if (key == MyGUI::KeyCode::ArrowDown)
+            mHourSlider->setScrollPosition(std::min(mHourSlider->getScrollPosition()+1, mHourSlider->getScrollRange()-1));
+        else if (key == MyGUI::KeyCode::ArrowUp)
+            mHourSlider->setScrollPosition(std::max(static_cast<int>(mHourSlider->getScrollPosition())-1, 0));
+        else
+            return;
+        onHourSliderChangedPosition(mHourSlider, mHourSlider->getScrollPosition());
     }
 
     void WaitDialog::onWaitingProgressChanged(int cur, int total)
@@ -265,7 +303,6 @@ namespace MWGui
         MWBase::Environment::get().getWindowManager()->fadeScreenIn(0.2f);
         mProgressBar.setVisible (false);
         MWBase::Environment::get().getWindowManager()->removeGuiMode (GM_Rest);
-        MWBase::Environment::get().getWindowManager()->removeGuiMode (GM_RestBed);
         mTimeAdvancer.stop();
     }
 
