@@ -105,7 +105,7 @@ namespace MWGui
             assignWidget(button, "StatValueButton");
             if (button)
             {
-                mSkillNameWidget = button;
+                mSkillValueWidget = button;
                 button->eventMouseButtonClick += MyGUI::newDelegate(this, &MWSkill::onClicked);
             }
         }
@@ -502,11 +502,10 @@ namespace MWGui
 
             if (mBarWidget)
             {
-                mBarWidget->setProgressRange(mMax);
-                mBarWidget->setProgressPosition(mValue);
+                mBarWidget->setProgressRange(std::max(0, mMax));
+                mBarWidget->setProgressPosition(std::max(0, mValue));
             }
 
-
             if (mBarTextWidget)
             {
                 std::stringstream out;
