@@ -1,9 +1,5 @@
 #include "inputmanagerimp.hpp"
 
-#include <cmath>
-
-#include <boost/lexical_cast.hpp>
-
 #include <osgViewer/ViewerEventHandlers>
 
 #include <MyGUI_InputManager.h>
@@ -22,9 +18,9 @@
 
 #include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwbase/soundmanager.hpp"
 #include "../mwbase/statemanager.hpp"
 #include "../mwbase/environment.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
 
 #include "../mwworld/player.hpp"
 #include "../mwworld/class.hpp"
@@ -166,28 +162,52 @@ namespace MWInput
 
     void InputManager::setPlayerControlsEnabled(bool enabled)
     {
-        int nPlayerChannels = 17;
-        int playerChannels[] = {A_Activate, A_AutoMove, A_AlwaysRun, A_ToggleWeapon,
+        int playerChannels[] = {A_AutoMove, A_AlwaysRun, A_ToggleWeapon,
                                 A_ToggleSpell, A_Rest, A_QuickKey1, A_QuickKey2,
                                 A_QuickKey3, A_QuickKey4, A_QuickKey5, A_QuickKey6,
                                 A_QuickKey7, A_QuickKey8, A_QuickKey9, A_QuickKey10,
-                               A_Use};
+                                A_Use, A_Journal};
 
-        for(int i = 0; i < nPlayerChannels; i++) {
+        for(size_t i = 0; i < sizeof(playerChannels)/sizeof(playerChannels[0]); i++) {
             int pc = playerChannels[i];
             mInputBinder->getChannel(pc)->setEnabled(enabled);
         }
     }
 
+    void InputManager::handleGuiArrowKey(int action)
+    {
+        if (SDL_IsTextInputActive())
+            return;
+        MyGUI::KeyCode key;
+        switch (action)
+        {
+        case A_MoveLeft:
+            key = MyGUI::KeyCode::ArrowLeft;
+            break;
+        case A_MoveRight:
+            key = MyGUI::KeyCode::ArrowRight;
+            break;
+        case A_MoveForward:
+            key = MyGUI::KeyCode::ArrowUp;
+            break;
+        case A_MoveBackward:
+        default:
+            key = MyGUI::KeyCode::ArrowDown;
+            break;
+        }
+
+        MWBase::Environment::get().getWindowManager()->injectKeyPress(key, 0);
+    }
+
     void InputManager::channelChanged(ICS::Channel* channel, float currentValue, float previousValue)
     {
-        if (mDragDrop)
-            return;
-
         resetIdleTime ();
 
         int action = channel->getNumber();
 
+        if (mDragDrop && action != A_GameMenu && action != A_Inventory)
+            return;
+
         if((previousValue == 1 || previousValue == 0) && (currentValue==1 || currentValue==0))
         {
             //Is a normal button press, so don't change it at all
@@ -223,9 +243,7 @@ namespace MWInput
             switch (action)
             {
             case A_GameMenu:
-                if(!(MWBase::Environment::get().getStateManager()->getState() != MWBase::StateManager::State_Running
-                    && MWBase::Environment::get().getWindowManager()->getMode() == MWGui::GM_MainMenu))
-                        toggleMainMenu ();
+                toggleMainMenu ();
                 break;
             case A_Screenshot:
                 screenshot();
@@ -238,8 +256,13 @@ namespace MWInput
                 break;
             case A_Activate:
                 resetIdleTime();
-                if (!MWBase::Environment::get().getWindowManager()->isGuiMode())
-                    activate();
+                activate();
+                break;
+            case A_MoveLeft:
+            case A_MoveRight:
+            case A_MoveForward:
+            case A_MoveBackward:
+                handleGuiArrowKey(action);
                 break;
             case A_Journal:
                 toggleJournal ();
@@ -293,7 +316,7 @@ namespace MWInput
                 showQuickKeysMenu();
                 break;
             case A_ToggleHUD:
-                MWBase::Environment::get().getWindowManager()->toggleGui();
+                MWBase::Environment::get().getWindowManager()->toggleHud();
                 break;
             case A_ToggleDebug:
                 MWBase::Environment::get().getWindowManager()->toggleDebugWindow();
@@ -372,8 +395,6 @@ namespace MWInput
         mInputManager->setMouseVisible(MWBase::Environment::get().getWindowManager()->getCursorVisible());
 
         mInputManager->capture(disableEvents);
-        // inject some fake mouse movement to force updating MyGUI's widget states
-        MyGUI::InputManager::getInstance().injectMouseMove( int(mGuiCursorX), int(mGuiCursorY), mMouseWheel);
 
         if (mControlsDisabled)
         {
@@ -386,34 +407,40 @@ namespace MWInput
 
         updateCursorMode();
 
-        if(mJoystickLastUsed)
+        if (mGuiCursorEnabled)
         {
-            if (mGuiCursorEnabled)
+            float xAxis = mInputBinder->getChannel(A_MoveLeftRight)->getValue()*2.0f-1.0f;
+            float yAxis = mInputBinder->getChannel(A_MoveForwardBackward)->getValue()*2.0f-1.0f;
+            float zAxis = mInputBinder->getChannel(A_LookUpDown)->getValue()*2.0f-1.0f;
+            const MyGUI::IntSize& viewSize = MyGUI::RenderManager::getInstance().getViewSize();
+
+            xAxis *= (1.5f - mInputBinder->getChannel(A_Use)->getValue());
+            yAxis *= (1.5f - mInputBinder->getChannel(A_Use)->getValue());
+
+            // We keep track of our own mouse position, so that moving the mouse while in
+            // game mode does not move the position of the GUI cursor
+            float xmove = xAxis * dt * 1500.0f * mInvUiScalingFactor;
+            float ymove = yAxis * dt * 1500.0f * mInvUiScalingFactor;
+            if (xmove != 0|| ymove != 0)
             {
-                float xAxis = mInputBinder->getChannel(A_MoveLeftRight)->getValue()*2.0f-1.0f;
-                float yAxis = mInputBinder->getChannel(A_MoveForwardBackward)->getValue()*2.0f-1.0f;
-                float zAxis = mInputBinder->getChannel(A_LookUpDown)->getValue()*2.0f-1.0f;
-                const MyGUI::IntSize& viewSize = MyGUI::RenderManager::getInstance().getViewSize();
-
-                xAxis *= (1.5f - mInputBinder->getChannel(A_Use)->getValue());
-                yAxis *= (1.5f - mInputBinder->getChannel(A_Use)->getValue());
-
-                // We keep track of our own mouse position, so that moving the mouse while in
-                // game mode does not move the position of the GUI cursor
-                mGuiCursorX += xAxis * dt * 1500.0f * mInvUiScalingFactor;
-                mGuiCursorY += yAxis * dt * 1500.0f * mInvUiScalingFactor;
+                mGuiCursorX += xmove;
+                mGuiCursorY += ymove;
                 mMouseWheel -= static_cast<int>(zAxis * dt * 1500.0f);
 
-                mGuiCursorX = std::max(0.f, std::min(mGuiCursorX, float(viewSize.width)));
-                mGuiCursorY = std::max(0.f, std::min(mGuiCursorY, float(viewSize.height)));
+                mGuiCursorX = std::max(0.f, std::min(mGuiCursorX, float(viewSize.width-1)));
+                mGuiCursorY = std::max(0.f, std::min(mGuiCursorY, float(viewSize.height-1)));
 
                 MyGUI::InputManager::getInstance().injectMouseMove(static_cast<int>(mGuiCursorX), static_cast<int>(mGuiCursorY), mMouseWheel);
                 mInputManager->warpMouse(static_cast<int>(mGuiCursorX/mInvUiScalingFactor), static_cast<int>(mGuiCursorY/mInvUiScalingFactor));
+                MWBase::Environment::get().getWindowManager()->setCursorActive(true);
             }
-            if (mMouseLookEnabled)
+        }
+        if (mMouseLookEnabled)
+        {
+            float xAxis = mInputBinder->getChannel(A_LookLeftRight)->getValue()*2.0f-1.0f;
+            float yAxis = mInputBinder->getChannel(A_LookUpDown)->getValue()*2.0f-1.0f;
+            if (xAxis != 0 || yAxis != 0)
             {
-                float xAxis = mInputBinder->getChannel(A_LookLeftRight)->getValue()*2.0f-1.0f;
-                float yAxis = mInputBinder->getChannel(A_LookUpDown)->getValue()*2.0f-1.0f;
                 resetIdleTime();
 
                 float rot[3];
@@ -440,73 +467,73 @@ namespace MWInput
             {
                 bool triedToMove = false;
                 bool isRunning = false;
-                if(mJoystickLastUsed)
+
+                // joystick movement
+                float xAxis = mInputBinder->getChannel(A_MoveLeftRight)->getValue();
+                float yAxis = mInputBinder->getChannel(A_MoveForwardBackward)->getValue();
+                if (xAxis < .5)
                 {
-                    float xAxis = mInputBinder->getChannel(A_MoveLeftRight)->getValue();
-                    float yAxis = mInputBinder->getChannel(A_MoveForwardBackward)->getValue();
-                    if (xAxis < .5)
-                    {
-                        triedToMove = true;
-                        mPlayer->setLeftRight (-1);
-                    }
-                    else if (xAxis > .5)
-                    {
-                        triedToMove = true;
-                        mPlayer->setLeftRight (1);
-                    }
-
-                    if (yAxis < .5)
-                    {
-                        triedToMove = true;
-                        mPlayer->setAutoMove (false);
-                        mPlayer->setForwardBackward (1);
-                    }
-                    else if (yAxis > .5)
-                    {
-                        triedToMove = true;
-                        mPlayer->setAutoMove (false);
-                        mPlayer->setForwardBackward (-1);
-                    }
-
-                    else if(mPlayer->getAutoMove())
-                    {
-                        triedToMove = true;
-                        mPlayer->setForwardBackward (1);
-                    }
-                    isRunning = xAxis > .75 || xAxis < .25 || yAxis > .75 || yAxis < .25;
-                    if(triedToMove) resetIdleTime();
+                    triedToMove = true;
+                    mPlayer->setLeftRight (-1);
                 }
-                else
+                else if (xAxis > .5)
                 {
-                    if (actionIsActive(A_MoveLeft))
-                    {
-                        triedToMove = true;
-                        mPlayer->setLeftRight (-1);
-                    }
-                    else if (actionIsActive(A_MoveRight))
-                    {
-                        triedToMove = true;
-                        mPlayer->setLeftRight (1);
-                    }
+                    triedToMove = true;
+                    mPlayer->setLeftRight (1);
+                }
 
-                    if (actionIsActive(A_MoveForward))
-                    {
-                        triedToMove = true;
-                        mPlayer->setAutoMove (false);
-                        mPlayer->setForwardBackward (1);
-                    }
-                    else if (actionIsActive(A_MoveBackward))
-                    {
-                        triedToMove = true;
-                        mPlayer->setAutoMove (false);
-                        mPlayer->setForwardBackward (-1);
-                    }
+                if (yAxis < .5)
+                {
+                    triedToMove = true;
+                    mPlayer->setAutoMove (false);
+                    mPlayer->setForwardBackward (1);
+                }
+                else if (yAxis > .5)
+                {
+                    triedToMove = true;
+                    mPlayer->setAutoMove (false);
+                    mPlayer->setForwardBackward (-1);
+                }
+                else if(mPlayer->getAutoMove())
+                {
+                    triedToMove = true;
+                    mPlayer->setForwardBackward (1);
+                }
+                if (triedToMove)
+                    mJoystickLastUsed = true;
 
-                    else if(mPlayer->getAutoMove())
-                    {
-                        triedToMove = true;
-                        mPlayer->setForwardBackward (1);
-                    }
+                // keyboard movement
+                isRunning = xAxis > .75 || xAxis < .25 || yAxis > .75 || yAxis < .25;
+                if(triedToMove) resetIdleTime();
+
+                if (actionIsActive(A_MoveLeft))
+                {
+                    triedToMove = true;
+                    mPlayer->setLeftRight (-1);
+                }
+                else if (actionIsActive(A_MoveRight))
+                {
+                    triedToMove = true;
+                    mPlayer->setLeftRight (1);
+                }
+
+                if (actionIsActive(A_MoveForward))
+                {
+                    triedToMove = true;
+                    mPlayer->setAutoMove (false);
+                    mPlayer->setForwardBackward (1);
+                }
+                else if (actionIsActive(A_MoveBackward))
+                {
+                    triedToMove = true;
+                    mPlayer->setAutoMove (false);
+                    mPlayer->setForwardBackward (-1);
+                }
+
+                else if(mPlayer->getAutoMove())
+                {
+                    triedToMove = true;
+                    mPlayer->setForwardBackward (1);
                 }
 
                 if (!mSneakToggles)
@@ -671,13 +698,17 @@ namespace MWInput
             SDL_StopTextInput();
 
         bool consumed = false;
-        if (kc != OIS::KC_UNASSIGNED)
+        if (kc != OIS::KC_UNASSIGNED && !mInputBinder->detectingBindingState())
         {
-            consumed = SDL_IsTextInputActive() &&
-                    ( !(SDLK_SCANCODE_MASK & arg.keysym.sym) && std::isprint(arg.keysym.sym)); // Little trick to check if key is printable
-            bool guiFocus = MyGUI::InputManager::getInstance().injectKeyPress(MyGUI::KeyCode::Enum(kc), 0);
-            setPlayerControlsEnabled(!guiFocus);
+            consumed = MWBase::Environment::get().getWindowManager()->injectKeyPress(MyGUI::KeyCode::Enum(kc), 0);
+            if (SDL_IsTextInputActive() &&  // Little trick to check if key is printable
+                                    ( !(SDLK_SCANCODE_MASK & arg.keysym.sym) && std::isprint(arg.keysym.sym)))
+                consumed = true;
+            setPlayerControlsEnabled(!consumed);
         }
+        if (arg.repeat)
+            return;
+
         if (!mControlsDisabled && !consumed)
             mInputBinder->keyPressed (arg);
         mJoystickLastUsed = false;
@@ -696,7 +727,8 @@ namespace MWInput
         mJoystickLastUsed = false;
         OIS::KeyCode kc = mInputManager->sdl2OISKeyCode(arg.keysym.sym);
 
-        setPlayerControlsEnabled(!MyGUI::InputManager::getInstance().injectKeyRelease(MyGUI::KeyCode::Enum(kc)));
+        if (!mInputBinder->detectingBindingState())
+            setPlayerControlsEnabled(!MyGUI::InputManager::getInstance().injectKeyRelease(MyGUI::KeyCode::Enum(kc)));
         mInputBinder->keyReleased (arg);
     }
 
@@ -712,11 +744,12 @@ namespace MWInput
             if (MyGUI::InputManager::getInstance ().getMouseFocusWidget () != 0)
             {
                 MyGUI::Button* b = MyGUI::InputManager::getInstance ().getMouseFocusWidget ()->castType<MyGUI::Button>(false);
-                if (b && b->getEnabled())
+                if (b && b->getEnabled() && id == SDL_BUTTON_LEFT)
                 {
-                    MWBase::Environment::get().getSoundManager ()->playSound ("Menu Click", 1.f, 1.f);
+                    MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
                 }
             }
+            MWBase::Environment::get().getWindowManager()->setCursorActive(true);
         }
 
         setPlayerControlsEnabled(!guiMode);
@@ -761,6 +794,10 @@ namespace MWInput
             mMouseWheel = int(arg.z);
 
             MyGUI::InputManager::getInstance().injectMouseMove( int(mGuiCursorX), int(mGuiCursorY), mMouseWheel);
+            // FIXME: inject twice to force updating focused widget states (tooltips) resulting from changing the viewport by scroll wheel
+            MyGUI::InputManager::getInstance().injectMouseMove( int(mGuiCursorX), int(mGuiCursorY), mMouseWheel);
+
+            MWBase::Environment::get().getWindowManager()->setCursorActive(true);
         }
 
         if (mMouseLookEnabled && !mControlsDisabled)
@@ -809,7 +846,7 @@ namespace MWInput
                     MyGUI::Button* b = MyGUI::InputManager::getInstance ().getMouseFocusWidget ()->castType<MyGUI::Button>(false);
                     if (b && b->getEnabled())
                     {
-                        MWBase::Environment::get().getSoundManager ()->playSound ("Menu Click", 1.f, 1.f);
+                        MWBase::Environment::get().getWindowManager()->playSound("Menu Click");
                     }
                 }
             }
@@ -851,7 +888,6 @@ namespace MWInput
 
     void InputManager::axisMoved(int deviceID, const SDL_ControllerAxisEvent &arg )
     {
-        mJoystickLastUsed = true;
         if (!mControlsDisabled)
             mInputBinder->axisMoved(deviceID, arg);
     }
@@ -930,6 +966,9 @@ namespace MWInput
             inventory.getSelectedEnchantItem() == inventory.end())
             return;
 
+        if (MWBase::Environment::get().getMechanicsManager()->isAttackingOrSpell(mPlayer->getPlayer()))
+            return;
+
         MWMechanics::DrawState_ state = mPlayer->getDrawState();
         if (state == MWMechanics::DrawState_Weapon || state == MWMechanics::DrawState_Nothing)
             mPlayer->setDrawState(MWMechanics::DrawState_Spell);
@@ -945,6 +984,13 @@ namespace MWInput
         if (!mControlSwitch["playerfighting"] || !mControlSwitch["playercontrols"])
             return;
 
+        // We want to interrupt animation only if attack is prepairing, but still is not triggered
+        // Otherwise we will get a "speedshooting" exploit, when player can skip reload animation by hitting "Toggle Weapon" key twice
+        if (MWBase::Environment::get().getMechanicsManager()->isAttackPrepairing(mPlayer->getPlayer()))
+            mPlayer->setAttackingOrSpell(false);
+        else if (MWBase::Environment::get().getMechanicsManager()->isAttackingOrSpell(mPlayer->getPlayer()))
+            return;
+
         MWMechanics::DrawState_ state = mPlayer->getDrawState();
         if (state == MWMechanics::DrawState_Spell || state == MWMechanics::DrawState_Nothing)
             mPlayer->setDrawState(MWMechanics::DrawState_Weapon);
@@ -1023,9 +1069,9 @@ namespace MWInput
             return;
 
         if(MWBase::Environment::get().getWindowManager()->getMode() != MWGui::GM_Journal
+                && MWBase::Environment::get().getWindowManager()->getMode() != MWGui::GM_MainMenu
                 && MWBase::Environment::get().getWindowManager ()->getJournalAllowed())
         {
-            MWBase::Environment::get().getSoundManager()->playSound ("book open", 1.0, 1.0);
             MWBase::Environment::get().getWindowManager()->pushGuiMode(MWGui::GM_Journal);
         }
         else if(MWBase::Environment::get().getWindowManager()->containsMode(MWGui::GM_Journal))
@@ -1066,7 +1112,12 @@ namespace MWInput
 
     void InputManager::activate()
     {
-        if (mControlSwitch["playercontrols"])
+        if (MWBase::Environment::get().getWindowManager()->isGuiMode())
+        {
+            if (!SDL_IsTextInputActive())
+                MWBase::Environment::get().getWindowManager()->injectKeyPress(MyGUI::KeyCode::Return, 0);
+        }
+        else if (mControlSwitch["playercontrols"])
             mPlayer->activate();
     }
 
@@ -1173,7 +1224,7 @@ namespace MWInput
             bool controlExists = mInputBinder->getChannel(i)->getControlsCount () != 0;
             if (!controlExists)
             {
-                control = new ICS::Control(boost::lexical_cast<std::string>(i), false, true, 0, ICS::ICS_MAX, ICS::ICS_MAX);
+                control = new ICS::Control(std::to_string(i), false, true, 0, ICS::ICS_MAX, ICS::ICS_MAX);
                 mInputBinder->addControl(control);
                 control->attachChannel(mInputBinder->getChannel(i), ICS::Channel::DIRECT);
             }
@@ -1201,6 +1252,17 @@ namespace MWInput
                     control->setInitialValue(0.0f);
                     mInputBinder->addMouseButtonBinding (control, defaultMouseButtonBindings[i], ICS::Control::INCREASE);
                 }
+
+                if (i == A_LookLeftRight && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_4) && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_6))
+                {
+                    mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_6, ICS::Control::INCREASE);
+                    mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_4, ICS::Control::DECREASE);
+                }
+                if (i == A_LookUpDown && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_8) && !mInputBinder->isKeyBound(SDL_SCANCODE_KP_2))
+                {
+                    mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_2, ICS::Control::INCREASE);
+                    mInputBinder->addKeyBinding(control, SDL_SCANCODE_KP_8, ICS::Control::DECREASE);
+                }
             }
         }
     }
@@ -1242,10 +1304,10 @@ namespace MWInput
             if (!controlExists)
             {
                 float initial;
-                if (defaultButtonBindings.find(i) != defaultButtonBindings.end())
+                if (defaultAxisBindings.find(i) == defaultAxisBindings.end())
                     initial = 0.0f;
                 else initial = 0.5f;
-                control = new ICS::Control(boost::lexical_cast<std::string>(i), false, true, initial, ICS::ICS_MAX, ICS::ICS_MAX);
+                control = new ICS::Control(std::to_string(i), false, true, initial, ICS::ICS_MAX, ICS::ICS_MAX);
                 mInputBinder->addControl(control);
                 control->attachChannel(mInputBinder->getChannel(i), ICS::Channel::DIRECT);
             }
@@ -1258,12 +1320,13 @@ namespace MWInput
             {
                 clearAllControllerBindings(control);
 
-                if (defaultButtonBindings.find(i) != defaultButtonBindings.end())
+                if (defaultButtonBindings.find(i) != defaultButtonBindings.end()
+                        && !mInputBinder->isJoystickButtonBound(mFakeDeviceID, defaultButtonBindings[i]))
                 {
                     control->setInitialValue(0.0f);
                     mInputBinder->addJoystickButtonBinding(control, mFakeDeviceID, defaultButtonBindings[i], ICS::Control::INCREASE);
                 }
-                else if (defaultAxisBindings.find(i) != defaultAxisBindings.end())
+                else if (defaultAxisBindings.find(i) != defaultAxisBindings.end() && !mInputBinder->isJoystickAxisBound(mFakeDeviceID, defaultAxisBindings[i]))
                 {
                     control->setValue(0.5f);
                     control->setInitialValue(0.5f);
@@ -1334,7 +1397,7 @@ namespace MWInput
         if (key != SDL_SCANCODE_UNKNOWN)
             return MyGUI::TextIterator::toTagsString(mInputBinder->scancodeToString (key));
         else if (mouse != ICS_MAX_DEVICE_BUTTONS)
-            return "#{sMouse} " + boost::lexical_cast<std::string>(mouse);
+            return "#{sMouse} " + std::to_string(mouse);
         else
             return "#{sNone}";
     }
@@ -1389,7 +1452,7 @@ namespace MWInput
             case SDL_CONTROLLER_BUTTON_Y:
                 return "Y Button";
             default:
-                return "Button " + boost::lexical_cast<std::string>(button);
+                return "Button " + std::to_string(button);
         }
     }
     std::string InputManager::sdlControllerAxisToString(int axis)
@@ -1409,7 +1472,7 @@ namespace MWInput
             case SDL_CONTROLLER_AXIS_TRIGGERRIGHT:
                 return "Right Trigger";
             default:
-                return "Axis " + boost::lexical_cast<std::string>(axis);
+                return "Axis " + std::to_string(axis);
         }
      }
 
@@ -1483,6 +1546,10 @@ namespace MWInput
         ret.push_back(A_QuickKey8);
         ret.push_back(A_QuickKey9);
         ret.push_back(A_QuickKey10);
+        ret.push_back(A_CycleSpellLeft);
+        ret.push_back(A_CycleSpellRight);
+        ret.push_back(A_CycleWeaponLeft);
+        ret.push_back(A_CycleWeaponRight);
 
         return ret;
     }
