@@ -222,10 +222,23 @@ namespace MWMechanics
         }
     }
 
-    void ActiveSpells::purgeAll(float chance)
+    void ActiveSpells::purgeAll(float chance, bool spellOnly)
     {
         for (TContainer::iterator it = mSpells.begin(); it != mSpells.end(); )
         {
+            const std::string spellId = it->first;
+
+            // if spellOnly is true, dispell only spells. Leave potions, enchanted items etc.
+            if (spellOnly)
+            {
+                const ESM::Spell* spell = MWBase::Environment::get().getWorld()->getStore().get<ESM::Spell>().search(spellId);
+                if (!spell || spell->mData.mType != ESM::Spell::ST_Spell)
+                {
+                    ++it;
+                    continue;
+                }
+            }
+
             if (Misc::Rng::roll0to99() < chance)
                 mSpells.erase(it++);
             else
