@@ -2,11 +2,15 @@
 #define GAME_MWMECHANICS_AIAVOIDDOOR_H
 
 #include "aipackage.hpp"
+
 #include <string>
-#include "pathfinding.hpp"
+
 #include <components/esm/defs.hpp>
+
 #include "../mwworld/class.hpp"
 
+#include "pathfinding.hpp"
+
 namespace MWMechanics
 {
     /// \brief AiPackage to have an actor avoid an opening door
@@ -26,6 +30,9 @@ namespace MWMechanics
 
             virtual unsigned int getPriority() const;
 
+            virtual bool canCancel() const { return false; }
+            virtual bool shouldCancelPreviousAi() const { return false; }
+
         private:
             float mDuration;
             MWWorld::ConstPtr mDoorPtr;
