@@ -45,8 +45,8 @@ namespace MWMechanics
         , mCellY(std::numeric_limits<int>::max())
     {
         // mDuration isn't saved in the save file, so just giving it "1" for now if the package has a duration.
-        // The exact value of mDuration only matters for repeating packages 
-        if (mRemainingDuration != 0)
+        // The exact value of mDuration only matters for repeating packages.
+        if (mRemainingDuration > 0) // Previously mRemainingDuration could be negative even when mDuration was 0. Checking for > 0 should fix old saves.
             mDuration = 1;
         else
             mDuration = 0;
@@ -62,7 +62,7 @@ namespace MWMechanics
     {
         // If AiEscort has ran for as long or longer then the duration specified
         // and the duration is not infinite, the package is complete.
-        if(mDuration > 0)
+        if (mDuration > 0)
         {
             mRemainingDuration -= ((duration*MWBase::Environment::get().getWorld()->getTimeScaleFactor()) / 3600);
             if (mRemainingDuration <= 0)
@@ -90,13 +90,13 @@ namespace MWMechanics
             (differenceBetween[0] * differenceBetween[0]) + (differenceBetween[1] * differenceBetween[1]) + (differenceBetween[2] *
                 differenceBetween[2]);
 
-        if(distanceBetweenResult <= mMaxDist * mMaxDist)
+        if (distanceBetweenResult <= mMaxDist * mMaxDist)
         {
             ESM::Pathgrid::Point point(static_cast<int>(mX), static_cast<int>(mY), static_cast<int>(mZ));
             point.mAutogenerated = 0;
             point.mConnectionNum = 0;
             point.mUnknown = 0;
-            if(pathTo(actor,point,duration)) //Returns true on path complete
+            if (pathTo(actor,point,duration)) //Returns true on path complete
             {
                 mRemainingDuration = mDuration;
                 return true;
@@ -126,7 +126,7 @@ namespace MWMechanics
 
     void AiEscort::writeState(ESM::AiSequence::AiSequence &sequence) const
     {
-        std::auto_ptr<ESM::AiSequence::AiEscort> escort(new ESM::AiSequence::AiEscort());
+        std::unique_ptr<ESM::AiSequence::AiEscort> escort(new ESM::AiSequence::AiEscort());
         escort->mData.mX = mX;
         escort->mData.mY = mY;
         escort->mData.mZ = mZ;
@@ -140,10 +140,11 @@ namespace MWMechanics
         sequence.mPackages.push_back(package);
     }
 
-void AiEscort::fastForward(const MWWorld::Ptr& actor, AiState &state)
-{
-    // Update duration counter
-    mRemainingDuration--;
-}
+    void AiEscort::fastForward(const MWWorld::Ptr& actor, AiState &state)
+    {
+        // Update duration counter if this package has a duration
+        if (mDuration > 0)
+            mRemainingDuration--;
+    }
 }
 
