@@ -32,7 +32,7 @@ namespace MWMechanics
     class AiPackage
     {
         public:
-            ///Enumerates the various AITypes availible.
+            ///Enumerates the various AITypes available
             enum TypeId {
                 TypeIdNone = -1,
                 TypeIdWander = 0,
@@ -41,12 +41,13 @@ namespace MWMechanics
                 TypeIdFollow = 3,
                 TypeIdActivate = 4,
 
-                // These 4 are not really handled as Ai Packages in the MW engine
+                // These 5 are not really handled as Ai Packages in the MW engine
                 // For compatibility do *not* return these in the getCurrentAiPackage script function..
                 TypeIdCombat = 5,
                 TypeIdPursue = 6,
                 TypeIdAvoidDoor = 7,
-                TypeIdFace = 8
+                TypeIdFace = 8,
+                TypeIdBreathe = 9
             };
 
             ///Default constructor
