@@ -38,6 +38,9 @@ namespace MWMechanics
 
             virtual void writeState (ESM::AiSequence::AiSequence& sequence) const;
 
+            virtual bool canCancel() const { return false; }
+            virtual bool shouldCancelPreviousAi() const { return false; }
+
         private:
 
             int mTargetActorId; // The actor to pursue
