@@ -64,7 +64,7 @@ namespace MWMechanics
             std::list<AiPackage*>::const_iterator begin() const;
             std::list<AiPackage*>::const_iterator end() const;
 
-            std::list<AiPackage*>::const_iterator erase (std::list<AiPackage*>::const_iterator package);
+            void erase (std::list<AiPackage*>::const_iterator package);
 
             /// Returns currently executing AiPackage type
             /** \see enum AiPackage::TypeId **/
@@ -79,9 +79,15 @@ namespace MWMechanics
             /// Return true and assign target if combat package is currently active, return false otherwise
             bool getCombatTarget (MWWorld::Ptr &targetActor) const;
 
+            /// Return true and assign targets for all combat packages, or return false if there are no combat packages
+            bool getCombatTargets(std::vector<MWWorld::Ptr> &targetActors) const;
+
             /// Is there any combat package?
             bool isInCombat () const;
 
+            /// Does this AI sequence have the given package type?
+            bool hasPackage(int typeId) const;
+
             /// Are we in combat with this particular actor?
             bool isInCombat (const MWWorld::Ptr& actor) const;
 
