@@ -113,7 +113,7 @@ namespace MWMechanics
     
     AiWander::AiWander(int distance, int duration, int timeOfDay, const std::vector<unsigned char>& idle, bool repeat):
         mDistance(distance), mDuration(duration), mRemainingDuration(duration), mTimeOfDay(timeOfDay), mIdle(idle),
-        mRepeat(repeat), mStoredInitialActorPosition(false)
+        mRepeat(repeat), mStoredInitialActorPosition(false), mInitialActorPosition(osg::Vec3f(0, 0, 0)), mHasDestination(false), mDestination(osg::Vec3f(0, 0, 0))
     {
         mIdle.resize(8, 0);
         init();
@@ -123,9 +123,6 @@ namespace MWMechanics
     {
         // NOTE: mDistance and mDuration must be set already
 
-        mHasReturnPosition = false;
-        mReturnPosition = osg::Vec3f(0,0,0);
-
         if(mDistance < 0)
             mDistance = 0;
         if(mDuration < 0)
@@ -203,6 +200,7 @@ namespace MWMechanics
             stopWalking(actor, storage);
             currentCell = actor.getCell();
             storage.mPopulateAvailableNodes = true;
+            mStoredInitialActorPosition = false;
         }
 
         mRemainingDuration -= ((duration*MWBase::Environment::get().getWorld()->getTimeScaleFactor()) / 3600);
@@ -211,6 +209,19 @@ namespace MWMechanics
         cStats.setMovementFlag(CreatureStats::Flag_Run, false);
 
         ESM::Position pos = actor.getRefData().getPosition();
+
+        // If there is already a destination due to the package having been interrupted by a combat or pursue package,
+        // rebuild a path to it
+        if (!mPathFinder.isPathConstructed() && mHasDestination)
+        {
+            ESM::Pathgrid::Point dest(PathFinder::MakePathgridPoint(mDestination));
+            ESM::Pathgrid::Point start(PathFinder::MakePathgridPoint(pos));
+
+            mPathFinder.buildSyncedPath(start, dest, actor.getCell());
+
+            if (mPathFinder.isPathConstructed())
+                storage.setState(Wander_Walking);
+        }
         
         doPerFrameActionsForState(actor, duration, storage, pos);
 
@@ -253,9 +264,8 @@ namespace MWMechanics
             getAllowedNodes(actor, currentCell->getCell(), storage);
         }
 
-        // Actor becomes stationary - see above URL's for previous research
-        // If a creature or an NPC with a wander distance and no pathgrid is available,
-        // randomly idle or wander around near spawn point
+        // If the package has a wander distance but no pathgrid is available,
+        // randomly idle or wander near spawn point
         if(storage.mAllowedNodes.empty() && mDistance > 0 && !storage.mIsWanderingManually) {
             // Typically want to idle for a short time before the next wander
             if (Misc::Rng::rollDice(100) >= 96) {
@@ -277,10 +287,8 @@ namespace MWMechanics
             mDistance = 0;
 
         // For stationary NPCs, move back to the starting location if another AiPackage moved us elsewhere
-        if (cellChange)
-            mHasReturnPosition = false;
-        if (mDistance == 0 && mHasReturnPosition 
-            && (pos.asVec3() - mReturnPosition).length2() > (DESTINATION_TOLERANCE * DESTINATION_TOLERANCE))
+        if (mDistance == 0 && !cellChange
+            && (pos.asVec3() - mInitialActorPosition).length2() > (DESTINATION_TOLERANCE * DESTINATION_TOLERANCE))
         {
             returnToStartLocation(actor, storage, pos);
         }
@@ -334,9 +342,10 @@ namespace MWMechanics
     {
         if (!mPathFinder.isPathConstructed())
         {
-            ESM::Pathgrid::Point dest(PathFinder::MakePathgridPoint(mReturnPosition));
+            mDestination = mInitialActorPosition;
+            ESM::Pathgrid::Point dest(PathFinder::MakePathgridPoint(mDestination));
 
-            // actor position is already in world co-ordinates
+            // actor position is already in world coordinates
             ESM::Pathgrid::Point start(PathFinder::MakePathgridPoint(pos));
 
             // don't take shortcuts for wandering
@@ -345,6 +354,7 @@ namespace MWMechanics
             if (mPathFinder.isPathConstructed())
             {
                 storage.setState(Wander_Walking);
+                mHasDestination = true;
             }
         }
     }
@@ -357,7 +367,6 @@ namespace MWMechanics
         const osg::Vec3f currentPositionVec3f = osg::Vec3f(currentPosition.mX, currentPosition.mY, currentPosition.mZ);
 
         std::size_t attempts = 10; // If a unit can't wander out of water, don't want to hang here
-        osg::Vec3f destination;
         ESM::Pathgrid::Point destinationPosition;
         bool isWaterCreature = actor.getClass().isPureWaterCreature(actor);
         do {
@@ -369,17 +378,18 @@ namespace MWMechanics
             const float destinationY = mInitialActorPosition.y() + wanderRadius * std::sin(randomDirection);
             const float destinationZ = mInitialActorPosition.z();
             destinationPosition = ESM::Pathgrid::Point(destinationX, destinationY, destinationZ);
-            destination = osg::Vec3f(destinationX, destinationY, destinationZ);
+            mDestination = osg::Vec3f(destinationX, destinationY, destinationZ);
 
             // Check if land creature will walk onto water or if water creature will swim onto land
-            if ((!isWaterCreature && !destinationIsAtWater(actor, destination)) ||
-                (isWaterCreature && !destinationThroughGround(currentPositionVec3f, destination))) {
+            if ((!isWaterCreature && !destinationIsAtWater(actor, mDestination)) ||
+                (isWaterCreature && !destinationThroughGround(currentPositionVec3f, mDestination))) {
                 mPathFinder.buildSyncedPath(currentPosition, destinationPosition, actor.getCell());
                 mPathFinder.addPointToPath(destinationPosition);
 
                 if (mPathFinder.isPathConstructed())
                 {
                     storage.setState(Wander_Walking, true);
+                    mHasDestination = true;
                 }
                 return;
             }
@@ -440,11 +450,13 @@ namespace MWMechanics
     {
         // Check if an idle actor is  too close to a door - if so start walking
         storage.mDoorCheckDuration += duration;
+
         if (storage.mDoorCheckDuration >= DOOR_CHECK_INTERVAL)
         {
             storage.mDoorCheckDuration = 0;    // restart timer
+            static float distance = MWBase::Environment::get().getWorld()->getMaxActivationDistance();
             if (mDistance &&            // actor is not intended to be stationary
-                proximityToDoor(actor, MIN_DIST_TO_DOOR_SQUARED*1.6f*1.6f)) // NOTE: checks interior cells only
+                proximityToDoor(actor, distance*1.6f))
             {
                 storage.setState(Wander_MoveNow);
                 storage.mTrimCurrentNode = false; // just in case
@@ -466,19 +478,21 @@ namespace MWMechanics
         GreetingState& greetingState = storage.mSaidGreeting;
         if (!checkIdle(actor, storage.mIdleAnimation) && (greetingState == Greet_Done || greetingState == Greet_None))
         {
-            storage.setState(Wander_ChooseAction);
+            if (mPathFinder.isPathConstructed())
+                storage.setState(Wander_Walking);
+            else
+                storage.setState(Wander_ChooseAction);
         }
     }
 
     void AiWander::onWalkingStatePerFrameActions(const MWWorld::Ptr& actor, 
         float duration, AiWanderStorage& storage, ESM::Position& pos)
     {
-        // Are we there yet?
-        if (pathTo(actor, mPathFinder.getPath().back(), duration, DESTINATION_TOLERANCE))
+        // Is there no destination or are we there yet?
+        if ((!mPathFinder.isPathConstructed()) || pathTo(actor, ESM::Pathgrid::Point(mPathFinder.getPath().back()), duration, DESTINATION_TOLERANCE))
         {
             stopWalking(actor, storage);
             storage.setState(Wander_ChooseAction);
-            mHasReturnPosition = false;
         }
         else
         {
@@ -490,8 +504,8 @@ namespace MWMechanics
     void AiWander::onChooseActionStatePerFrameActions(const MWWorld::Ptr& actor, AiWanderStorage& storage)
     {
 
-        short unsigned& idleAnimation = storage.mIdleAnimation;
-        idleAnimation = getRandomIdle();
+        unsigned short idleAnimation = getRandomIdle();
+        storage.mIdleAnimation = idleAnimation;
 
         if (!idleAnimation && mDistance)
         {
@@ -519,13 +533,14 @@ namespace MWMechanics
         if (mObstacleCheck.isEvading())
         {
             // first check if we're walking into a door
-            if (proximityToDoor(actor)) // NOTE: checks interior cells only
+            static float distance = MWBase::Environment::get().getWorld()->getMaxActivationDistance();
+            if (proximityToDoor(actor, distance))
             {
                 // remove allowed points then select another random destination
                 storage.mTrimCurrentNode = true;
                 trimAllowedNodes(storage.mAllowedNodes, mPathFinder);
                 mObstacleCheck.clear();
-                mPathFinder.clearPath();
+                stopWalking(actor, storage);
                 storage.setState(Wander_MoveNow);
             }
 
@@ -535,9 +550,7 @@ namespace MWMechanics
         // if stuck for sufficiently long, act like current location was the destination
         if (storage.mStuckCount >= COUNT_BEFORE_RESET) // something has gone wrong, reset
         {
-            //std::cout << "Reset \""<< cls.getName(actor) << "\"" << std::endl;
             mObstacleCheck.clear();
-
             stopWalking(actor, storage);
             storage.setState(Wander_ChooseAction);
             storage.mStuckCount = 0;
@@ -612,7 +625,7 @@ namespace MWMechanics
 
             if (storage.mState == Wander_Walking)
             {
-                stopWalking(actor, storage);
+                stopWalking(actor, storage, false);
                 mObstacleCheck.clear();
                 storage.setState(Wander_IdleNow);
             }
@@ -649,7 +662,7 @@ namespace MWMechanics
         ESM::Pathgrid::Point dest(storage.mAllowedNodes[randNode]);
         ToWorldCoordinates(dest, storage.mCell->getCell());
 
-        // actor position is already in world co-ordinates
+        // actor position is already in world coordinates
         ESM::Pathgrid::Point start(PathFinder::MakePathgridPoint(actorPos));
 
         // don't take shortcuts for wandering
@@ -657,6 +670,8 @@ namespace MWMechanics
 
         if (mPathFinder.isPathConstructed())
         {
+            mDestination = osg::Vec3f(dest.mX, dest.mY, dest.mZ);
+            mHasDestination = true;
             // Remove this node as an option and add back the previously used node (stops NPC from picking the same node):
             ESM::Pathgrid::Point temp = storage.mAllowedNodes[randNode];
             storage.mAllowedNodes.erase(storage.mAllowedNodes.begin() + randNode);
@@ -693,8 +708,8 @@ namespace MWMechanics
             ESM::Pathgrid::Point pt = paths.back();
             for(unsigned int j = 0; j < nodes.size(); j++)
             {
-                // FIXME: doesn't hadle a door with the same X/Y
-                //        co-ordinates but with a different Z
+                // FIXME: doesn't handle a door with the same X/Y
+                //        coordinates but with a different Z
                 if(nodes[j].mX == pt.mX && nodes[j].mY == pt.mY)
                 {
                     nodes.erase(nodes.begin() + j);
@@ -710,9 +725,13 @@ namespace MWMechanics
         return TypeIdWander;
     }
 
-    void AiWander::stopWalking(const MWWorld::Ptr& actor, AiWanderStorage& storage)
+    void AiWander::stopWalking(const MWWorld::Ptr& actor, AiWanderStorage& storage, bool clearPath)
     {
-        mPathFinder.clearPath();
+        if (clearPath)
+        {
+            mPathFinder.clearPath();
+            mHasDestination = false;
+        }
         actor.getClass().getMovementSettings(actor).mPosition[1] = 0;
     }
 
@@ -725,7 +744,7 @@ namespace MWMechanics
         }
         else
         {
-            std::cerr<< "Attempted to play out of range idle animation \""<<idleSelect<<"\" for " << actor.getCellRef().getRefId() << std::endl;
+            std::cerr<< "Error: Attempted to play out of range idle animation \""<<idleSelect<<"\" for " << actor.getCellRef().getRefId() << std::endl;
             return false;
         }
     }
@@ -743,15 +762,6 @@ namespace MWMechanics
         }
     }
 
-    void AiWander::setReturnPosition(const osg::Vec3f& position)
-    {
-        if (!mHasReturnPosition)
-        {
-            mHasReturnPosition = true;
-            mReturnPosition = position;
-        }
-    }
-
     short unsigned AiWander::getRandomIdle()
     {
         unsigned short idleRoll = 0;
@@ -828,7 +838,7 @@ namespace MWMechanics
         // ... pathgrids don't usually include water, so swimmers ignore them
         if (mDistance && storage.mCanWanderAlongPathGrid && !actor.getClass().isPureWaterCreature(actor))
         {
-            // get NPC's position in local (i.e. cell) co-ordinates
+            // get NPC's position in local (i.e. cell) coordinates
             osg::Vec3f npcPos(mInitialActorPosition);
             CoordinateConverter(cell).toLocal(npcPos);
             
@@ -837,7 +847,7 @@ namespace MWMechanics
 
             // mAllowedNodes for this actor with pathgrid point indexes based on mDistance
             // and if the point is connected to the closest current point
-            // NOTE: mPoints and mAllowedNodes are in local co-ordinates
+            // NOTE: mPoints and mAllowedNodes are in local coordinates
             int pointIndex = 0;
             for(unsigned int counter = 0; counter < pathgrid->mPoints.size(); counter++)
             {
@@ -893,7 +903,7 @@ namespace MWMechanics
         storage.mAllowedNodes.push_back(PathFinder::MakePathgridPoint(vectorStart + delta));
     }
 
-    void AiWander::SetCurrentNodeToClosestAllowedNode(osg::Vec3f npcPos, AiWanderStorage& storage)
+    void AiWander::SetCurrentNodeToClosestAllowedNode(const osg::Vec3f& npcPos, AiWanderStorage& storage)
     {
         float distanceToClosestNode = std::numeric_limits<float>::max();
         unsigned int index = 0;
@@ -919,7 +929,7 @@ namespace MWMechanics
         else
             remainingDuration = mDuration;
 
-        std::auto_ptr<ESM::AiSequence::AiWander> wander(new ESM::AiSequence::AiWander());
+        std::unique_ptr<ESM::AiSequence::AiWander> wander(new ESM::AiSequence::AiWander());
         wander->mData.mDistance = mDistance;
         wander->mData.mDuration = mDuration;
         wander->mData.mTimeOfDay = mTimeOfDay;
@@ -945,6 +955,8 @@ namespace MWMechanics
         , mTimeOfDay(wander->mData.mTimeOfDay)
         , mRepeat(wander->mData.mShouldRepeat != 0)
         , mStoredInitialActorPosition(wander->mStoredInitialActorPosition)
+        , mHasDestination(false)
+        , mDestination(osg::Vec3f(0, 0, 0))
     {
         if (mStoredInitialActorPosition)
             mInitialActorPosition = wander->mInitialActorPosition;
@@ -953,7 +965,6 @@ namespace MWMechanics
         if (mRemainingDuration <= 0 || mRemainingDuration >= 24)
             mRemainingDuration = mDuration;
 
-
         init();
     }
 }
