@@ -12,6 +12,7 @@
 class btCollisionWorld;
 class btCollisionShape;
 class btCollisionObject;
+class btConvexShape;
 
 namespace Resource
 {
@@ -61,6 +62,8 @@ namespace MWPhysics
             return mInternalCollisionMode;
         }
 
+        btConvexShape* getConvexShape() const { return mConvexShape; }
+
         /**
          * Enables or disables the *external* collision body. If disabled, other actors will not collide with this actor.
          */
@@ -69,6 +72,11 @@ namespace MWPhysics
         void updateScale();
         void updateRotation();
 
+        /**
+         * Return true if the collision shape looks the same no matter how its Z rotated.
+         */
+        bool isRotationallyInvariant() const;
+
         /**
          * Set mPosition and mPreviousPosition to the position in the Ptr's RefData. This should be used
          * when an object is "instantly" moved/teleported as opposed to being moved by the physics simulation.
@@ -124,6 +132,13 @@ namespace MWPhysics
             return mInternalCollisionMode && mOnGround;
         }
 
+        void setOnSlope(bool slope);
+
+        bool getOnSlope() const
+        {
+            return mInternalCollisionMode && mOnSlope;
+        }
+
         btCollisionObject* getCollisionObject() const
         {
             return mCollisionObject.get();
@@ -139,13 +154,18 @@ namespace MWPhysics
     private:
         /// Removes then re-adds the collision object to the dynamics world
         void updateCollisionMask();
+        void addCollisionMask(int collisionMask);
+        int getCollisionMask();
 
         bool mCanWaterWalk;
         bool mWalkingOnWater;
 
-        std::auto_ptr<btCollisionShape> mShape;
+        bool mRotationallyInvariant;
 
-        std::auto_ptr<btCollisionObject> mCollisionObject;
+        std::unique_ptr<btCollisionShape> mShape;
+        btConvexShape* mConvexShape;
+
+        std::unique_ptr<btCollisionObject> mCollisionObject;
 
         osg::Vec3f mMeshTranslation;
         osg::Vec3f mHalfExtents;
@@ -158,6 +178,7 @@ namespace MWPhysics
 
         osg::Vec3f mForce;
         bool mOnGround;
+        bool mOnSlope;
         bool mInternalCollisionMode;
         bool mExternalCollisionMode;
 
