@@ -109,7 +109,7 @@ void LocalMap::saveFogOfWar(MWWorld::CellStore* cell)
 
         if (segment.mFogOfWarImage && segment.mHasFogState)
         {
-            std::auto_ptr<ESM::FogState> fog (new ESM::FogState());
+            std::unique_ptr<ESM::FogState> fog (new ESM::FogState());
             fog->mFogTextures.push_back(ESM::FogTexture());
 
             segment.saveFogOfWar(fog->mFogTextures.back());
@@ -126,7 +126,7 @@ void LocalMap::saveFogOfWar(MWWorld::CellStore* cell)
         const int segsX = static_cast<int>(std::ceil(length.x() / mMapWorldSize));
         const int segsY = static_cast<int>(std::ceil(length.y() / mMapWorldSize));
 
-        std::auto_ptr<ESM::FogState> fog (new ESM::FogState());
+        std::unique_ptr<ESM::FogState> fog (new ESM::FogState());
 
         fog->mBounds.mMinX = mBounds.xMin();
         fog->mBounds.mMaxX = mBounds.xMax();
@@ -174,9 +174,7 @@ osg::ref_ptr<osg::Camera> LocalMap::createOrthographicCamera(float x, float y, f
     camera->setNodeMask(Mask_RenderToTexture);
 
     osg::ref_ptr<osg::StateSet> stateset = new osg::StateSet;
-    stateset->setMode(GL_LIGHTING, osg::StateAttribute::ON);
-    stateset->setMode(GL_NORMALIZE, osg::StateAttribute::ON);
-    stateset->setMode(GL_CULL_FACE, osg::StateAttribute::ON);
+
     // assign large value to effectively turn off fog
     // shaders don't respect glDisable(GL_FOG)
     osg::ref_ptr<osg::Fog> fog (new osg::Fog);
@@ -231,13 +229,51 @@ void LocalMap::setupRenderToTexture(osg::ref_ptr<osg::Camera> camera, int x, int
     segment.mMapTexture = texture;
 }
 
+bool needUpdate(std::set<std::pair<int, int> >& renderedGrid, std::set<std::pair<int, int> >& currentGrid, int cellX, int cellY)
+{
+    // if all the cells of the current grid are contained in the rendered grid then we can keep the old render
+    for (int dx=-1;dx<2;dx+=1)
+    {
+        for (int dy=-1;dy<2;dy+=1)
+        {
+            bool haveInRenderedGrid = renderedGrid.find(std::make_pair(cellX+dx,cellY+dy)) != renderedGrid.end();
+            bool haveInCurrentGrid = currentGrid.find(std::make_pair(cellX+dx,cellY+dy)) != currentGrid.end();
+            if (haveInCurrentGrid && !haveInRenderedGrid)
+                return true;
+        }
+    }
+    return false;
+}
+
 void LocalMap::requestMap(std::set<const MWWorld::CellStore*> cells)
 {
+    std::set<std::pair<int, int> > grid;
     for (std::set<const MWWorld::CellStore*>::iterator it = cells.begin(); it != cells.end(); ++it)
     {
         const MWWorld::CellStore* cell = *it;
         if (cell->isExterior())
-            requestExteriorMap(cell);
+            grid.insert(std::make_pair(cell->getCell()->getGridX(), cell->getCell()->getGridY()));
+    }
+
+    for (std::set<const MWWorld::CellStore*>::iterator it = cells.begin(); it != cells.end(); ++it)
+    {
+        const MWWorld::CellStore* cell = *it;
+        if (cell->isExterior())
+        {
+            int cellX = cell->getCell()->getGridX();
+            int cellY = cell->getCell()->getGridY();
+
+            MapSegment& segment = mSegments[std::make_pair(cellX, cellY)];
+            if (!needUpdate(segment.mGrid, grid, cellX, cellY))
+            {
+                continue;
+            }
+            else
+            {
+                segment.mGrid = grid;
+                requestExteriorMap(cell);
+            }
+        }
         else
             requestInteriorMap(cell);
     }
@@ -282,7 +318,7 @@ void LocalMap::markForRemoval(osg::Camera *cam)
     CameraVector::iterator found = std::find(mActiveCameras.begin(), mActiveCameras.end(), cam);
     if (found == mActiveCameras.end())
     {
-        std::cerr << "trying to remove an inactive camera" << std::endl;
+        std::cerr << "Error: trying to remove an inactive camera" << std::endl;
         return;
     }
     mActiveCameras.erase(found);
@@ -313,6 +349,11 @@ void LocalMap::requestExteriorMap(const MWWorld::CellStore* cell)
 
     osg::ref_ptr<osg::Camera> camera = createOrthographicCamera(x*mMapWorldSize + mMapWorldSize/2.f, y*mMapWorldSize + mMapWorldSize/2.f, mMapWorldSize, mMapWorldSize,
                                                                 osg::Vec3d(0,1,0), zmin, zmax);
+    camera->getOrCreateUserDataContainer()->addDescription("NoTerrainLod");
+    std::ostringstream stream;
+    stream << x << " " << y;
+    camera->getOrCreateUserDataContainer()->addDescription(stream.str());
+
     setupRenderToTexture(camera, cell->getCell()->getGridX(), cell->getCell()->getGridY());
 
     MapSegment& segment = mSegments[std::make_pair(cell->getCell()->getGridX(), cell->getCell()->getGridY())];
@@ -449,7 +490,7 @@ void LocalMap::requestInteriorMap(const MWWorld::CellStore* cell)
                     // We are using the same bounds and angle as we were using when the textures were originally made. Segments should come out the same.
                     if (i >= int(fog->mFogTextures.size()))
                     {
-                        std::cout << "Warning: fog texture count mismatch" << std::endl;
+                        std::cout << "Error: fog texture count mismatch" << std::endl;
                         break;
                     }
 
@@ -641,7 +682,7 @@ void LocalMap::MapSegment::loadFogOfWar(const ESM::FogTexture &esm)
     osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("tga");
     if (!readerwriter)
     {
-        std::cerr << "Unable to load fog, can't find a tga ReaderWriter" << std::endl;
+        std::cerr << "Error: Unable to load fog, can't find a tga ReaderWriter" << std::endl;
         return;
     }
 
@@ -650,7 +691,7 @@ void LocalMap::MapSegment::loadFogOfWar(const ESM::FogTexture &esm)
     osgDB::ReaderWriter::ReadResult result = readerwriter->readImage(in);
     if (!result.success())
     {
-        std::cerr << "Failed to read fog: " << result.message() << " code " << result.status() << std::endl;
+        std::cerr << "Error: Failed to read fog: " << result.message() << " code " << result.status() << std::endl;
         return;
     }
 
@@ -673,7 +714,7 @@ void LocalMap::MapSegment::saveFogOfWar(ESM::FogTexture &fog) const
     osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("tga");
     if (!readerwriter)
     {
-        std::cerr << "Unable to write fog, can't find a tga ReaderWriter" << std::endl;
+        std::cerr << "Error: Unable to write fog, can't find a tga ReaderWriter" << std::endl;
         return;
     }
 
@@ -682,7 +723,7 @@ void LocalMap::MapSegment::saveFogOfWar(ESM::FogTexture &fog) const
     osgDB::ReaderWriter::WriteResult result = readerwriter->writeImage(*mFogOfWarImage, ostream);
     if (!result.success())
     {
-        std::cerr << "Unable to write fog: " << result.message() << " code " << result.status() << std::endl;
+        std::cerr << "Error: Unable to write fog: " << result.message() << " code " << result.status() << std::endl;
         return;
     }
     mFogOfWarImage->flipVertical();
