@@ -48,9 +48,6 @@ namespace MWRender
         void enableCellPathgrid(const MWWorld::CellStore *store);
         void disableCellPathgrid(const MWWorld::CellStore *store);
 
-        // path grid meshes
-        osg::ref_ptr<osg::Geometry> createPathgridLines(const ESM::Pathgrid *pathgrid);
-        osg::ref_ptr<osg::Geometry> createPathgridPoints(const ESM::Pathgrid *pathgrid);
     public:
         Pathgrid(osg::ref_ptr<osg::Group> root);
         ~Pathgrid();
