@@ -25,6 +25,9 @@
 #include <osgParticle/ConstantRateCounter>
 #include <osgParticle/RadialShooter>
 
+#include <osgParticle/Operator>
+#include <osgParticle/ModularProgram>
+
 #include <components/misc/rng.hpp>
 
 #include <components/misc/resourcehelpers.hpp>
@@ -48,7 +51,6 @@
 
 namespace
 {
-
     osg::ref_ptr<osg::Material> createAlphaTrackingUnlitMaterial()
     {
         osg::ref_ptr<osg::Material> mat = new osg::Material;
@@ -428,10 +430,12 @@ private:
 class CelestialBody
 {
 public:
-    CelestialBody(osg::Group* parentNode, float scaleFactor, int numUvSets)
+    CelestialBody(osg::Group* parentNode, float scaleFactor, int numUvSets, unsigned int visibleMask=~0)
+        : mVisibleMask(visibleMask)
     {
         mGeom = createTexturedQuad(numUvSets);
         mTransform = new osg::PositionAttitudeTransform;
+        mTransform->setNodeMask(mVisibleMask);
         mTransform->setScale(osg::Vec3f(450,450,450) * scaleFactor);
         mTransform->addChild(mGeom);
 
@@ -444,10 +448,11 @@ public:
 
     void setVisible(bool visible)
     {
-        mTransform->setNodeMask(visible ? ~0 : 0);
+        mTransform->setNodeMask(visible ? mVisibleMask : 0);
     }
 
 protected:
+    unsigned int mVisibleMask;
     static const float mDistance;
     osg::ref_ptr<osg::PositionAttitudeTransform> mTransform;
     osg::ref_ptr<osg::Geometry> mGeom;
@@ -459,11 +464,10 @@ class Sun : public CelestialBody
 {
 public:
     Sun(osg::Group* parentNode, Resource::ImageManager& imageManager)
-        : CelestialBody(parentNode, 1.0f, 1)
+        : CelestialBody(parentNode, 1.0f, 1, Mask_Sun)
         , mUpdater(new Updater)
     {
         mTransform->addUpdateCallback(mUpdater);
-        mTransform->setNodeMask(Mask_Sun);
 
         osg::ref_ptr<osg::Texture2D> sunTex (new osg::Texture2D(imageManager.getImage("textures/tx_sun_05.dds")));
         sunTex->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
@@ -943,8 +947,8 @@ public:
 
     void setState(const MoonState& state)
     {
-        float radsX = ((state.mRotationFromHorizon) * M_PI) / 180.0f;
-        float radsZ = ((state.mRotationFromNorth) * M_PI) / 180.0f;
+        float radsX = ((state.mRotationFromHorizon) * static_cast<float>(osg::PI)) / 180.0f;
+        float radsZ = ((state.mRotationFromNorth) * static_cast<float>(osg::PI)) / 180.0f;
 
         osg::Quat rotX(radsX, osg::Vec3f(1.0f, 0.0f, 0.0f));
         osg::Quat rotZ(radsZ, osg::Vec3f(0.0f, 0.0f, 1.0f));
@@ -954,7 +958,7 @@ public:
 
         // The moon quad is initially oriented facing down, so we need to offset its X-axis
         // rotation to rotate it to face the camera when sitting at the horizon.
-        osg::Quat attX((-M_PI / 2.0f) + radsX, osg::Vec3f(1.0f, 0.0f, 0.0f));
+        osg::Quat attX((-static_cast<float>(osg::PI) / 2.0f) + radsX, osg::Vec3f(1.0f, 0.0f, 0.0f));
         mTransform->setAttitude(attX * rotZ);
 
         setPhase(state.mPhase);
@@ -1091,6 +1095,8 @@ private:
 
 SkyManager::SkyManager(osg::Group* parentNode, Resource::SceneManager* sceneManager)
     : mSceneManager(sceneManager)
+    , mCamera(NULL)
+    , mRainIntensityUniform(NULL)
     , mAtmosphereNightRoll(0.f)
     , mCreated(false)
     , mIsStorm(false)
@@ -1113,6 +1119,7 @@ SkyManager::SkyManager(osg::Group* parentNode, Resource::SceneManager* sceneMana
     , mSunEnabled(true)
 {
     osg::ref_ptr<CameraRelativeTransform> skyroot (new CameraRelativeTransform);
+    skyroot->setName("Sky Root");
     // Assign empty program to specify we don't want shaders
     // The shaders generated by the SceneManager can't handle everything we need
     skyroot->getOrCreateStateSet()->setAttributeAndModes(new osg::Program(), osg::StateAttribute::OVERRIDE);
@@ -1132,6 +1139,11 @@ SkyManager::SkyManager(osg::Group* parentNode, Resource::SceneManager* sceneMana
     mUnderwaterSwitch = new UnderwaterSwitchCallback(skyroot);
 }
 
+void SkyManager::setRainIntensityUniform(osg::Uniform *uniform)
+{
+    mRainIntensityUniform = uniform;
+}
+
 void SkyManager::create()
 {
     assert(!mCreated);
@@ -1233,9 +1245,11 @@ private:
 class AlphaFader : public SceneUtil::StateSetUpdater
 {
 public:
-    AlphaFader()
+    /// @param alphaUpdate variable which to update with alpha value
+    AlphaFader(float *alphaUpdate)
         : mAlpha(1.f)
     {
+        mAlphaUpdate = alphaUpdate;
     }
 
     void setAlpha(float alpha)
@@ -1254,15 +1268,19 @@ public:
     {
         osg::Material* mat = static_cast<osg::Material*>(stateset->getAttribute(osg::StateAttribute::MATERIAL));
         mat->setDiffuse(osg::Material::FRONT_AND_BACK, osg::Vec4f(0,0,0,mAlpha));
+
+        if (mAlphaUpdate)
+            *mAlphaUpdate = mAlpha;
     }
 
     // Helper for adding AlphaFaders to a subgraph
     class SetupVisitor : public osg::NodeVisitor
     {
     public:
-        SetupVisitor()
+        SetupVisitor(float *alphaUpdate)
             : osg::NodeVisitor(TRAVERSE_ALL_CHILDREN)
         {
+            mAlphaUpdate = alphaUpdate;
         }
 
         virtual void apply(osg::Node &node)
@@ -1273,14 +1291,16 @@ public:
                 {
                     SceneUtil::CompositeStateSetUpdater* composite = NULL;
                     osg::Callback* callback = node.getUpdateCallback();
+
                     while (callback)
                     {
                         if ((composite = dynamic_cast<SceneUtil::CompositeStateSetUpdater*>(callback)))
                             break;
+
                         callback = callback->getNestedCallback();
                     }
 
-                    osg::ref_ptr<AlphaFader> alphaFader (new AlphaFader);
+                    osg::ref_ptr<AlphaFader> alphaFader (new AlphaFader(mAlphaUpdate));
 
                     if (composite)
                         composite->addController(alphaFader);
@@ -1290,6 +1310,7 @@ public:
                     mAlphaFaders.push_back(alphaFader);
                 }
             }
+
             traverse(node);
         }
 
@@ -1300,15 +1321,21 @@ public:
 
     private:
         std::vector<osg::ref_ptr<AlphaFader> > mAlphaFaders;
+        float *mAlphaUpdate;
     };
 
-private:
+protected:
     float mAlpha;
+    float *mAlphaUpdate;
 };
 
 class RainFader : public AlphaFader
 {
 public:
+    RainFader(float *alphaUpdate): AlphaFader(alphaUpdate)
+    {
+    }
+
     virtual void setDefaults(osg::StateSet* stateset)
     {
         osg::ref_ptr<osg::Material> mat (new osg::Material);
@@ -1317,6 +1344,93 @@ public:
         mat->setColorMode(osg::Material::OFF);
         stateset->setAttributeAndModes(mat, osg::StateAttribute::ON);
     }
+
+    virtual void apply(osg::StateSet *stateset, osg::NodeVisitor *nv)
+    {
+        AlphaFader::apply(stateset,nv);
+        *mAlphaUpdate = mAlpha * 2.0;  // mAlpha is limited to 0.6 so multiply by 2 to reach full intensity
+    }
+};
+
+void SkyManager::setCamera(osg::Camera *camera)
+{
+    mCamera = camera;
+}
+
+class WrapAroundOperator : public osgParticle::Operator
+{
+public:
+    WrapAroundOperator(osg::Camera *camera, const osg::Vec3 &wrapRange): osgParticle::Operator()
+    {
+        mCamera = camera;
+        mWrapRange = wrapRange;
+        mHalfWrapRange = mWrapRange / 2.0;
+        mPreviousCameraPosition = getCameraPosition();
+    }
+
+    virtual osg::Object *cloneType() const override
+    {
+        return NULL;
+    }
+
+    virtual osg::Object *clone(const osg::CopyOp &op) const override
+    {
+        return NULL;
+    }
+
+    virtual void operate(osgParticle::Particle *P, double dt) override
+    {
+    }
+
+    virtual void operateParticles(osgParticle::ParticleSystem *ps, double dt) override
+    {
+        osg::Vec3 position = getCameraPosition();
+        osg::Vec3 positionDifference = position - mPreviousCameraPosition;
+
+        osg::Matrix toWorld, toLocal;
+
+        std::vector<osg::Matrix> worldMatrices = ps->getWorldMatrices();
+ 
+        if (!worldMatrices.empty())
+        {
+            toWorld = worldMatrices[0];
+            toLocal.invert(toWorld);
+        }
+
+        for (int i = 0; i < ps->numParticles(); ++i)
+        {
+            osgParticle::Particle *p = ps->getParticle(i);
+            p->setPosition(toWorld.preMult(p->getPosition()));
+            p->setPosition(p->getPosition() - positionDifference);
+
+            for (int j = 0; j < 3; ++j)  // wrap-around in all 3 dimensions
+            {
+                osg::Vec3 pos = p->getPosition();
+
+                if (pos[j] < -mHalfWrapRange[j])
+                    pos[j] = mHalfWrapRange[j] + fmod(pos[j] - mHalfWrapRange[j],mWrapRange[j]);
+                else if (pos[j] > mHalfWrapRange[j])
+                    pos[j] = fmod(pos[j] + mHalfWrapRange[j],mWrapRange[j]) - mHalfWrapRange[j];
+
+                p->setPosition(pos);
+            }
+
+            p->setPosition(toLocal.preMult(p->getPosition()));
+        }
+
+        mPreviousCameraPosition = position;
+    }
+
+protected:
+    osg::Camera *mCamera;
+    osg::Vec3 mPreviousCameraPosition;
+    osg::Vec3 mWrapRange;
+    osg::Vec3 mHalfWrapRange;
+
+    osg::Vec3 getCameraPosition()
+    {
+        return mCamera->getInverseViewMatrix().getTrans();
+    }
 };
 
 void SkyManager::createRain()
@@ -1327,6 +1441,8 @@ void SkyManager::createRain()
     mRainNode = new osg::Group;
 
     mRainParticleSystem = new osgParticle::ParticleSystem;
+    osg::Vec3 rainRange = osg::Vec3(600,600,600);
+
     mRainParticleSystem->setParticleAlignment(osgParticle::ParticleSystem::FIXED);
     mRainParticleSystem->setAlignVectorX(osg::Vec3f(0.1,0,0));
     mRainParticleSystem->setAlignVectorY(osg::Vec3f(0,0,1));
@@ -1352,8 +1468,8 @@ void SkyManager::createRain()
     emitter->setParticleSystem(mRainParticleSystem);
 
     osg::ref_ptr<osgParticle::BoxPlacer> placer (new osgParticle::BoxPlacer);
-    placer->setXRange(-300, 300); // Rain_Diameter
-    placer->setYRange(-300, 300);
+    placer->setXRange(-rainRange.x() / 2, rainRange.x() / 2); // Rain_Diameter
+    placer->setYRange(-rainRange.y() / 2, rainRange.y() / 2);
     placer->setZRange(300, 300);
     emitter->setPlacer(placer);
 
@@ -1368,11 +1484,16 @@ void SkyManager::createRain()
     osg::ref_ptr<osgParticle::ParticleSystemUpdater> updater (new osgParticle::ParticleSystemUpdater);
     updater->addParticleSystem(mRainParticleSystem);
 
+    osg::ref_ptr<osgParticle::ModularProgram> program (new osgParticle::ModularProgram);
+    program->addOperator(new WrapAroundOperator(mCamera,rainRange));
+    program->setParticleSystem(mRainParticleSystem);
+    mRainNode->addChild(program);
+
     mRainNode->addChild(emitter);
     mRainNode->addChild(mRainParticleSystem);
     mRainNode->addChild(updater);
 
-    mRainFader = new RainFader;
+    mRainFader = new RainFader(&mWeatherAlpha);
     mRainNode->addUpdateCallback(mRainFader);
     mRainNode->addCullCallback(mUnderwaterSwitch);
     mRainNode->setNodeMask(Mask_WeatherParticles);
@@ -1413,9 +1534,33 @@ int SkyManager::getSecundaPhase() const
     return mSecunda->getPhaseInt();
 }
 
+bool SkyManager::isEnabled()
+{
+    return mEnabled;
+}
+
+bool SkyManager::hasRain()
+{
+    return mRainNode != NULL;
+}
+
 void SkyManager::update(float duration)
 {
-    if (!mEnabled) return;
+    if (!mEnabled)
+    {
+        if (mRainIntensityUniform)
+            mRainIntensityUniform->set((float) 0.0);
+
+        return;
+    }
+
+    if (mRainIntensityUniform)
+    {
+        if (mIsStorm || (!hasRain() && !mParticleNode))
+            mRainIntensityUniform->set((float) 0.0);
+        else
+            mRainIntensityUniform->set((float) mWeatherAlpha);
+    }
 
     if (mIsStorm)
     {
@@ -1424,6 +1569,7 @@ void SkyManager::update(float duration)
 
         if (mParticleNode)
             mParticleNode->setAttitude(quat);
+
         mCloudNode->setAttitude(quat);
     }
     else
@@ -1520,17 +1666,35 @@ void SkyManager::setWeather(const WeatherResult& weather)
                 mParticleNode->setNodeMask(Mask_WeatherParticles);
                 mRootNode->addChild(mParticleNode);
             }
+
             mParticleEffect = mSceneManager->getInstance(mCurrentParticleEffect, mParticleNode);
 
-            SceneUtil::AssignControllerSourcesVisitor assignVisitor(boost::shared_ptr<SceneUtil::ControllerSource>(new SceneUtil::FrameTimeSource));
+            SceneUtil::AssignControllerSourcesVisitor assignVisitor(std::shared_ptr<SceneUtil::ControllerSource>(new SceneUtil::FrameTimeSource));
             mParticleEffect->accept(assignVisitor);
 
-            AlphaFader::SetupVisitor alphaFaderSetupVisitor;
+            AlphaFader::SetupVisitor alphaFaderSetupVisitor(&mWeatherAlpha);
+
             mParticleEffect->accept(alphaFaderSetupVisitor);
             mParticleFaders = alphaFaderSetupVisitor.getAlphaFaders();
 
             SceneUtil::DisableFreezeOnCullVisitor disableFreezeOnCullVisitor;
             mParticleEffect->accept(disableFreezeOnCullVisitor);
+
+            if (!weather.mIsStorm)
+            {
+                SceneUtil::FindByClassVisitor findPSVisitor(std::string("ParticleSystem"));
+                mParticleEffect->accept(findPSVisitor);
+
+                for (unsigned int i = 0; i < findPSVisitor.mFoundNodes.size(); ++i)
+                {
+                    osgParticle::ParticleSystem *ps = static_cast<osgParticle::ParticleSystem *>(findPSVisitor.mFoundNodes[i]);
+                    
+                    osg::ref_ptr<osgParticle::ModularProgram> program (new osgParticle::ModularProgram);
+                    program->addOperator(new WrapAroundOperator(mCamera,osg::Vec3(1024,1024,800)));
+                    program->setParticleSystem(ps);
+                    mParticleNode->addChild(program);
+                }
+            }
         }
     }
 
@@ -1606,7 +1770,8 @@ void SkyManager::setWeather(const WeatherResult& weather)
     mSun->adjustTransparency(weather.mGlareView * weather.mSunDiscColor.a());
 
     float nextStarsOpacity = weather.mNightFade * weather.mGlareView;
-    if(weather.mNight && mStarsOpacity != nextStarsOpacity)
+
+    if (weather.mNight && mStarsOpacity != nextStarsOpacity)
     {
         mStarsOpacity = nextStarsOpacity;
 
@@ -1617,6 +1782,7 @@ void SkyManager::setWeather(const WeatherResult& weather)
 
     if (mRainFader)
         mRainFader->setAlpha(weather.mEffectFade * 0.6); // * Rain_Threshold?
+
     for (std::vector<osg::ref_ptr<AlphaFader> >::const_iterator it = mParticleFaders.begin(); it != mParticleFaders.end(); ++it)
         (*it)->setAlpha(weather.mEffectFade);
 }
