@@ -37,7 +37,7 @@ namespace MWSound
     {
     public:
         MovieAudioDecoder(Video::VideoState *videoState)
-            : Video::MovieAudioDecoder(videoState)
+            : Video::MovieAudioDecoder(videoState), mAudioTrack(nullptr)
         {
             mDecoderBridge.reset(new MWSoundDecoderBridge(this));
         }
@@ -85,14 +85,14 @@ namespace MWSound
     public:
         ~MovieAudioDecoder()
         {
-            if(mAudioTrack.get())
+            if(mAudioTrack)
                 MWBase::Environment::get().getSoundManager()->stopTrack(mAudioTrack);
-            mAudioTrack.reset();
+            mAudioTrack = nullptr;
             mDecoderBridge.reset();
         }
 
-        MWBase::SoundStreamPtr mAudioTrack;
-        boost::shared_ptr<MWSoundDecoderBridge> mDecoderBridge;
+        MWBase::SoundStream *mAudioTrack;
+        std::shared_ptr<MWSoundDecoderBridge> mDecoderBridge;
     };
 
 
@@ -156,14 +156,14 @@ namespace MWSound
 
 
 
-    boost::shared_ptr<Video::MovieAudioDecoder> MovieAudioFactory::createDecoder(Video::VideoState* videoState)
+    std::shared_ptr<Video::MovieAudioDecoder> MovieAudioFactory::createDecoder(Video::VideoState* videoState)
     {
-        boost::shared_ptr<MWSound::MovieAudioDecoder> decoder(new MWSound::MovieAudioDecoder(videoState));
+        std::shared_ptr<MWSound::MovieAudioDecoder> decoder(new MWSound::MovieAudioDecoder(videoState));
         decoder->setupFormat();
 
         MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
-        MWBase::SoundStreamPtr sound = sndMgr->playTrack(decoder->mDecoderBridge, MWBase::SoundManager::Play_TypeMovie);
-        if (!sound.get())
+        MWBase::SoundStream *sound = sndMgr->playTrack(decoder->mDecoderBridge, MWSound::Type::Movie);
+        if (!sound)
         {
             decoder.reset();
             return decoder;
