@@ -6,11 +6,12 @@
 #include <utility>
 #include <deque>
 #include <map>
-
-#include <boost/shared_ptr.hpp>
+#include <unordered_map>
 
 #include <components/settings/settings.hpp>
 
+#include <components/fallback/fallback.hpp>
+
 #include "../mwbase/soundmanager.hpp"
 
 namespace VFS
@@ -28,6 +29,7 @@ namespace MWSound
     class Sound_Output;
     struct Sound_Decoder;
     class Sound;
+    class Stream;
     class Sound_Buffer;
 
     enum Environment {
@@ -44,10 +46,12 @@ namespace MWSound
     {
         const VFS::Manager* mVFS;
 
-        std::auto_ptr<Sound_Output> mOutput;
+        Fallback::Map mFallback;
+        std::unique_ptr<Sound_Output> mOutput;
 
         // Caches available music tracks by <playlist name, (sound files) >
-        std::map<std::string, std::vector<std::string> > mMusicFiles;
+        std::unordered_map<std::string, std::vector<std::string>> mMusicFiles;
+        std::unordered_map<std::string, std::vector<int>> mMusicToPlay; // A list with music files not yet played
         std::string mLastPlayedMusic; // The music file that was last played
 
         float mMasterVolume;
@@ -56,7 +60,13 @@ namespace MWSound
         float mVoiceVolume;
         float mFootstepsVolume;
 
-        typedef std::auto_ptr<std::deque<Sound_Buffer> > SoundBufferList;
+        int mNearWaterRadius;
+        int mNearWaterPoints;
+        float mNearWaterIndoorTolerance;
+        float mNearWaterOutdoorTolerance;
+        std::string mNearWaterIndoorID;
+        std::string mNearWaterOutdoorID;
+        typedef std::unique_ptr<std::deque<Sound_Buffer> > SoundBufferList;
         // List of sound buffers, grown as needed. New enties are added to the
         // back, allowing existing Sound_Buffer references/pointers to remain
         // valid.
@@ -65,25 +75,31 @@ namespace MWSound
         size_t mBufferCacheMax;
         size_t mBufferCacheSize;
 
-        typedef std::map<std::string,Sound_Buffer*> NameBufferMap;
+        typedef std::unordered_map<std::string,Sound_Buffer*> NameBufferMap;
         NameBufferMap mBufferNameMap;
 
         // NOTE: unused buffers are stored in front-newest order.
         typedef std::deque<Sound_Buffer*> SoundList;
         SoundList mUnusedBuffers;
 
-        typedef std::pair<MWBase::SoundPtr,Sound_Buffer*> SoundBufferRefPair;
+        std::unique_ptr<std::deque<Sound>> mSounds;
+        std::vector<Sound*> mUnusedSounds;
+
+        std::unique_ptr<std::deque<Stream>> mStreams;
+        std::vector<Stream*> mUnusedStreams;
+
+        typedef std::pair<MWBase::Sound*,Sound_Buffer*> SoundBufferRefPair;
         typedef std::vector<SoundBufferRefPair> SoundBufferRefPairList;
         typedef std::map<MWWorld::ConstPtr,SoundBufferRefPairList> SoundMap;
         SoundMap mActiveSounds;
 
-        typedef std::map<MWWorld::ConstPtr,MWBase::SoundStreamPtr> SaySoundMap;
+        typedef std::map<MWWorld::ConstPtr,Stream*> SaySoundMap;
         SaySoundMap mActiveSaySounds;
 
-        typedef std::vector<MWBase::SoundStreamPtr> TrackList;
+        typedef std::vector<Stream*> TrackList;
         TrackList mActiveTracks;
 
-        MWBase::SoundStreamPtr mMusic;
+        Stream *mMusic;
         std::string mCurrentPlaylist;
 
         bool mListenerUnderwater;
@@ -93,7 +109,8 @@ namespace MWSound
 
         int mPausedSoundTypes;
 
-        MWBase::SoundPtr mUnderwaterSound;
+        Sound *mUnderwaterSound;
+        Sound *mNearWaterSound;
 
         Sound_Buffer *insertSound(const std::string &soundId, const ESM::Sound *sound);
 
@@ -103,13 +120,23 @@ namespace MWSound
         // returns a decoder to start streaming
         DecoderPtr loadVoice(const std::string &voicefile);
 
-        MWBase::SoundStreamPtr playVoice(DecoderPtr decoder, const osg::Vec3f &pos, bool playlocal);
+        Sound *getSoundRef();
+        Stream *getStreamRef();
+
+        Stream *playVoice(DecoderPtr decoder, const osg::Vec3f &pos, bool playlocal);
 
         void streamMusicFull(const std::string& filename);
+        void advanceMusic(const std::string& filename);
+        void startRandomTitle();
+
         void updateSounds(float duration);
         void updateRegionSound(float duration);
+        void updateWaterSound(float duration);
+        void updateMusic(float duration);
 
-        float volumeFromType(PlayType type) const;
+        std::string mNextMusic;
+
+        float volumeFromType(Type type) const;
 
         SoundManager(const SoundManager &rhs);
         SoundManager& operator=(const SoundManager &rhs);
@@ -119,7 +146,7 @@ namespace MWSound
         friend class OpenAL_Output;
 
     public:
-        SoundManager(const VFS::Manager* vfs, bool useSound);
+        SoundManager(const VFS::Manager* vfs, const std::map<std::string, std::string>& fallbackMap, bool useSound);
         virtual ~SoundManager();
 
         virtual void processChangedSettings(const Settings::CategorySettingVector& settings);
@@ -131,9 +158,6 @@ namespace MWSound
         ///< Play a soundifle
         /// \param filename name of a sound file in "Music/" in the data directory.
 
-        virtual void startRandomTitle();
-        ///< Starts a random track from the current playlist
-
         virtual bool isMusicPlaying();
         ///< Returns true if music is playing
 
@@ -160,33 +184,33 @@ namespace MWSound
         /// and get an average loudness value (scale [0,1]) at the current time position.
         /// If the actor is not saying anything, returns 0.
 
-        virtual MWBase::SoundStreamPtr playTrack(const DecoderPtr& decoder, PlayType type);
+        virtual Stream *playTrack(const DecoderPtr& decoder, Type type);
         ///< Play a 2D audio track, using a custom decoder
 
-        virtual void stopTrack(MWBase::SoundStreamPtr stream);
+        virtual void stopTrack(Stream *stream);
         ///< Stop the given audio track from playing
 
-        virtual double getTrackTimeDelay(MWBase::SoundStreamPtr stream);
+        virtual double getTrackTimeDelay(Stream *stream);
         ///< Retives the time delay, in seconds, of the audio track (must be a sound
         /// returned by \ref playTrack). Only intended to be called by the track
         /// decoder's read method.
 
-        virtual MWBase::SoundPtr playSound(const std::string& soundId, float volume, float pitch, PlayType type=Play_TypeSfx, PlayMode mode=Play_Normal, float offset=0);
+        virtual Sound *playSound(const std::string& soundId, float volume, float pitch, Type type=Type::Sfx, PlayMode mode=PlayMode::Normal, float offset=0);
         ///< Play a sound, independently of 3D-position
         ///< @param offset Number of seconds into the sound to start playback.
 
-        virtual MWBase::SoundPtr playSound3D(const MWWorld::ConstPtr &reference, const std::string& soundId,
-                                             float volume, float pitch, PlayType type=Play_TypeSfx,
-                                             PlayMode mode=Play_Normal, float offset=0);
+        virtual Sound *playSound3D(const MWWorld::ConstPtr &reference, const std::string& soundId,
+                                   float volume, float pitch, Type type=Type::Sfx,
+                                   PlayMode mode=PlayMode::Normal, float offset=0);
         ///< Play a 3D sound attached to an MWWorld::Ptr. Will be updated automatically with the Ptr's position, unless Play_NoTrack is specified.
         ///< @param offset Number of seconds into the sound to start playback.
 
-        virtual MWBase::SoundPtr playSound3D(const osg::Vec3f& initialPos, const std::string& soundId,
-                                             float volume, float pitch, PlayType type, PlayMode mode, float offset=0);
+        virtual Sound *playSound3D(const osg::Vec3f& initialPos, const std::string& soundId,
+                                   float volume, float pitch, Type type, PlayMode mode, float offset=0);
         ///< Play a 3D sound at \a initialPos. If the sound should be moving, it must be updated using Sound::setPosition.
         ///< @param offset Number of seconds into the sound to start playback.
 
-        virtual void stopSound(MWBase::SoundPtr sound);
+        virtual void stopSound(Sound *sound);
         ///< Stop the given sound from playing
         /// @note no-op if \a sound is null
 
@@ -211,10 +235,10 @@ namespace MWSound
         virtual bool getSoundPlaying(const MWWorld::ConstPtr &reference, const std::string& soundId) const;
         ///< Is the given sound currently playing on the given object?
 
-        virtual void pauseSounds(int types=Play_TypeMask);
+        virtual void pauseSounds(int types);
         ///< Pauses all currently playing sounds, including music.
 
-        virtual void resumeSounds(int types=Play_TypeMask);
+        virtual void resumeSounds(int types);
         ///< Resumes all previously paused sounds.
 
         virtual void update(float duration);
