@@ -7,14 +7,17 @@
 #include <components/resource/bulletshapemanager.hpp>
 #include <components/resource/keyframemanager.hpp>
 #include <components/misc/resourcehelpers.hpp>
+#include <components/misc/stringops.hpp>
 #include <components/nifosg/nifloader.hpp>
 #include <components/terrain/world.hpp>
+#include <components/esmterrain/storage.hpp>
+#include <components/sceneutil/unrefqueue.hpp>
+#include <components/esm/loadcell.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
 
-#include "../mwworld/inventorystore.hpp"
-#include "../mwworld/esmstore.hpp"
+#include "../mwrender/landmanager.hpp"
 
 #include "cellstore.hpp"
 #include "manualref.hpp"
@@ -45,7 +48,7 @@ namespace MWWorld
     {
     public:
         /// Constructor to be called from the main thread.
-        PreloadItem(MWWorld::CellStore* cell, Resource::SceneManager* sceneManager, Resource::BulletShapeManager* bulletShapeManager, Resource::KeyframeManager* keyframeManager, Terrain::World* terrain, bool preloadInstances)
+        PreloadItem(MWWorld::CellStore* cell, Resource::SceneManager* sceneManager, Resource::BulletShapeManager* bulletShapeManager, Resource::KeyframeManager* keyframeManager, Terrain::World* terrain, MWRender::LandManager* landManager, bool preloadInstances)
             : mIsExterior(cell->getCell()->isExterior())
             , mX(cell->getCell()->getGridX())
             , mY(cell->getCell()->getGridY())
@@ -53,8 +56,12 @@ namespace MWWorld
             , mBulletShapeManager(bulletShapeManager)
             , mKeyframeManager(keyframeManager)
             , mTerrain(terrain)
+            , mLandManager(landManager)
             , mPreloadInstances(preloadInstances)
+            , mAbort(false)
         {
+            mTerrainView = mTerrain->createView();
+
             ListModelsVisitor visitor (mMeshes);
             if (cell->getState() == MWWorld::CellStore::State_Loaded)
             {
@@ -75,11 +82,31 @@ namespace MWWorld
             }
         }
 
+        virtual void abort()
+        {
+            mAbort = true;
+        }
+
         /// Preload work to be called from the worker thread.
         virtual void doWork()
         {
+            if (mIsExterior)
+            {
+                try
+                {
+                    mTerrain->cacheCell(mTerrainView.get(), mX, mY);
+                    mPreloadedObjects.push_back(mLandManager->getLand(mX, mY));
+                }
+                catch(std::exception& e)
+                {
+                }
+            }
+
             for (MeshList::const_iterator it = mMeshes.begin(); it != mMeshes.end(); ++it)
             {
+                if (mAbort)
+                    break;
+
                 try
                 {
                     std::string mesh  = *it;
@@ -118,17 +145,6 @@ namespace MWWorld
                     // error will be shown when visiting the cell
                 }
             }
-
-            if (mIsExterior)
-            {
-                try
-                {
-                    mPreloadedObjects.push_back(mTerrain->cacheCell(mX, mY));
-                }
-                catch(std::exception& e)
-                {
-                }
-            }
         }
 
     private:
@@ -141,8 +157,13 @@ namespace MWWorld
         Resource::BulletShapeManager* mBulletShapeManager;
         Resource::KeyframeManager* mKeyframeManager;
         Terrain::World* mTerrain;
+        MWRender::LandManager* mLandManager;
         bool mPreloadInstances;
 
+        volatile bool mAbort;
+
+        osg::ref_ptr<Terrain::View> mTerrainView;
+
         // keep a ref to the loaded objects to make sure it stays loaded as long as this cell is in the preloaded state
         std::vector<osg::ref_ptr<const osg::Object> > mPreloadedObjects;
     };
@@ -151,41 +172,56 @@ namespace MWWorld
     class UpdateCacheItem : public SceneUtil::WorkItem
     {
     public:
-        UpdateCacheItem(Resource::ResourceSystem* resourceSystem, Terrain::World* terrain, double referenceTime)
+        UpdateCacheItem(Resource::ResourceSystem* resourceSystem, double referenceTime)
             : mReferenceTime(referenceTime)
             , mResourceSystem(resourceSystem)
-            , mTerrain(terrain)
         {
         }
 
         virtual void doWork()
         {
             mResourceSystem->updateCache(mReferenceTime);
-
-            mTerrain->updateCache();
         }
 
     private:
         double mReferenceTime;
         Resource::ResourceSystem* mResourceSystem;
-        Terrain::World* mTerrain;
     };
 
-    CellPreloader::CellPreloader(Resource::ResourceSystem* resourceSystem, Resource::BulletShapeManager* bulletShapeManager, Terrain::World* terrain)
+    CellPreloader::CellPreloader(Resource::ResourceSystem* resourceSystem, Resource::BulletShapeManager* bulletShapeManager, Terrain::World* terrain, MWRender::LandManager* landManager)
         : mResourceSystem(resourceSystem)
         , mBulletShapeManager(bulletShapeManager)
         , mTerrain(terrain)
+        , mLandManager(landManager)
         , mExpiryDelay(0.0)
         , mMinCacheSize(0)
         , mMaxCacheSize(0)
         , mPreloadInstances(true)
+        , mLastResourceCacheUpdate(0.0)
     {
     }
 
     CellPreloader::~CellPreloader()
     {
+        if (mTerrainPreloadItem)
+        {
+            mTerrainPreloadItem->abort();
+            mTerrainPreloadItem->waitTillDone();
+            mTerrainPreloadItem = NULL;
+        }
+
+        if (mUpdateCacheItem)
+        {
+            mUpdateCacheItem->waitTillDone();
+            mUpdateCacheItem = NULL;
+        }
+
+        for (PreloadMap::iterator it = mPreloadCells.begin(); it != mPreloadCells.end();++it)
+            it->second.mWorkItem->abort();
+
         for (PreloadMap::iterator it = mPreloadCells.begin(); it != mPreloadCells.end();++it)
             it->second.mWorkItem->waitTillDone();
+
         mPreloadCells.clear();
     }
 
@@ -193,12 +229,12 @@ namespace MWWorld
     {
         if (!mWorkQueue)
         {
-            std::cerr << "can't preload, no work queue set " << std::endl;
+            std::cerr << "Error: can't preload, no work queue set " << std::endl;
             return;
         }
         if (cell->getState() == CellStore::State_Unloaded)
         {
-            std::cerr << "can't preload objects for unloaded cell" << std::endl;
+            std::cerr << "Error: can't preload objects for unloaded cell" << std::endl;
             return;
         }
 
@@ -226,12 +262,15 @@ namespace MWWorld
             }
 
             if (oldestTimestamp + threshold < timestamp)
+            {
+                oldestCell->second.mWorkItem->abort();
                 mPreloadCells.erase(oldestCell);
+            }
             else
                 return;
         }
 
-        osg::ref_ptr<PreloadItem> item (new PreloadItem(cell, mResourceSystem->getSceneManager(), mBulletShapeManager, mResourceSystem->getKeyframeManager(), mTerrain, mPreloadInstances));
+        osg::ref_ptr<PreloadItem> item (new PreloadItem(cell, mResourceSystem->getSceneManager(), mBulletShapeManager, mResourceSystem->getKeyframeManager(), mTerrain, mLandManager, mPreloadInstances));
         mWorkQueue->addWorkItem(item);
 
         mPreloadCells[cell] = PreloadEntry(timestamp, item);
@@ -239,7 +278,32 @@ namespace MWWorld
 
     void CellPreloader::notifyLoaded(CellStore *cell)
     {
-        mPreloadCells.erase(cell);
+        PreloadMap::iterator found = mPreloadCells.find(cell);
+        if (found != mPreloadCells.end())
+        {
+            // do the deletion in the background thread
+            if (found->second.mWorkItem)
+            {
+                found->second.mWorkItem->abort();
+                mUnrefQueue->push(mPreloadCells[cell].mWorkItem);
+            }
+
+            mPreloadCells.erase(found);
+        }
+    }
+
+    void CellPreloader::clear()
+    {
+        for (PreloadMap::iterator it = mPreloadCells.begin(); it != mPreloadCells.end();)
+        {
+            if (it->second.mWorkItem)
+            {
+                it->second.mWorkItem->abort();
+                mUnrefQueue->push(it->second.mWorkItem);
+            }
+
+            mPreloadCells.erase(it++);
+        }
     }
 
     void CellPreloader::updateCache(double timestamp)
@@ -247,13 +311,25 @@ namespace MWWorld
         for (PreloadMap::iterator it = mPreloadCells.begin(); it != mPreloadCells.end();)
         {
             if (mPreloadCells.size() >= mMinCacheSize && it->second.mTimeStamp < timestamp - mExpiryDelay)
+            {
+                if (it->second.mWorkItem)
+                {
+                    it->second.mWorkItem->abort();
+                    mUnrefQueue->push(it->second.mWorkItem);
+                }
                 mPreloadCells.erase(it++);
+            }
             else
                 ++it;
         }
 
-        // the resource cache is cleared from the worker thread so that we're not holding up the main thread with delete operations
-        mWorkQueue->addWorkItem(new UpdateCacheItem(mResourceSystem, mTerrain, timestamp), true);
+        if (timestamp - mLastResourceCacheUpdate > 1.0 && (!mUpdateCacheItem || mUpdateCacheItem->isDone()))
+        {
+            // the resource cache is cleared from the worker thread so that we're not holding up the main thread with delete operations
+            mUpdateCacheItem = new UpdateCacheItem(mResourceSystem, timestamp);
+            mWorkQueue->addWorkItem(mUpdateCacheItem, true);
+            mLastResourceCacheUpdate = timestamp;
+        }
     }
 
     void CellPreloader::setExpiryDelay(double expiryDelay)
@@ -286,4 +362,69 @@ namespace MWWorld
         mWorkQueue = workQueue;
     }
 
+    void CellPreloader::setUnrefQueue(SceneUtil::UnrefQueue* unrefQueue)
+    {
+        mUnrefQueue = unrefQueue;
+    }
+
+    class TerrainPreloadItem : public SceneUtil::WorkItem
+    {
+    public:
+        TerrainPreloadItem(const std::vector<osg::ref_ptr<Terrain::View> >& views, Terrain::World* world, const std::vector<osg::Vec3f>& preloadPositions)
+            : mAbort(false)
+            , mTerrainViews(views)
+            , mWorld(world)
+            , mPreloadPositions(preloadPositions)
+        {
+        }
+
+        virtual void doWork()
+        {
+            for (unsigned int i=0; i<mTerrainViews.size() && i<mPreloadPositions.size() && !mAbort; ++i)
+            {
+                mWorld->preload(mTerrainViews[i], mPreloadPositions[i]);
+                mTerrainViews[i]->reset(0);
+            }
+        }
+
+        virtual void abort()
+        {
+            mAbort = true;
+        }
+
+    private:
+        volatile bool mAbort;
+        std::vector<osg::ref_ptr<Terrain::View> > mTerrainViews;
+        Terrain::World* mWorld;
+        std::vector<osg::Vec3f> mPreloadPositions;
+    };
+
+    void CellPreloader::setTerrainPreloadPositions(const std::vector<osg::Vec3f> &positions)
+    {
+        if (mTerrainPreloadItem && !mTerrainPreloadItem->isDone())
+            return;
+        else if (positions == mTerrainPreloadPositions)
+            return;
+        else
+        {
+            if (mTerrainViews.size() > positions.size())
+            {
+                for (unsigned int i=positions.size(); i<mTerrainViews.size(); ++i)
+                    mUnrefQueue->push(mTerrainViews[i]);
+                mTerrainViews.resize(positions.size());
+            }
+            else if (mTerrainViews.size() < positions.size())
+            {
+                for (unsigned int i=mTerrainViews.size(); i<positions.size(); ++i)
+                    mTerrainViews.push_back(mTerrain->createView());
+            }
+
+            // TODO: provide some way of giving the preloaded view to the main thread when we enter the cell
+            // right now, we just use it to make sure the resources are preloaded
+            mTerrainPreloadPositions = positions;
+            mTerrainPreloadItem = new TerrainPreloadItem(mTerrainViews, mTerrain, positions);
+            mWorkQueue->addWorkItem(mTerrainPreloadItem);
+        }
+    }
+
 }
