@@ -137,6 +137,9 @@ namespace
                     iter->load (state);
                     return;
                 }
+
+            std::cerr << "Warning: Dropping reference to " << state.mRef.mRefID << " (invalid content file link)" << std::endl;
+            return;
         }
 
         // new reference
@@ -194,7 +197,7 @@ namespace MWWorld
         else
         {
             std::cerr
-                << "Error: could not resolve cell reference '" << ref.mRefID << "'"
+                << "Warning: could not resolve cell reference '" << ref.mRefID << "'"
                 << " (dropping reference)" << std::endl;
         }
     }
@@ -664,7 +667,7 @@ namespace MWWorld
 
             default:
                 std::cerr
-                    << "WARNING: Ignoring reference '" << ref.mRefID << "' of unhandled type\n";
+                    << "Error: Ignoring reference '" << ref.mRefID << "' of unhandled type\n";
                 return;
         }
 
@@ -895,7 +898,7 @@ namespace MWWorld
 
             if (!visitor.mFound)
             {
-                std::cerr << "Dropping moved ref tag for " << refnum.mIndex << " (moved object no longer exists)" << std::endl;
+                std::cerr << "Warning: Dropping moved ref tag for " << refnum.mIndex << " (moved object no longer exists)" << std::endl;
                 continue;
             }
 
@@ -905,7 +908,7 @@ namespace MWWorld
 
             if (otherCell == NULL)
             {
-                std::cerr << "Dropping moved ref tag for " << movedRef->mRef.getRefId()
+                std::cerr << "Warning: Dropping moved ref tag for " << movedRef->mRef.getRefId()
                           << " (target cell " << movedTo.mWorldspace << " no longer exists). Reference moved back to its original location." << std::endl;
                 // Note by dropping tag the object will automatically re-appear in its original cell, though potentially at inapproriate coordinates.
                 // Restore original coordinates:
