@@ -116,6 +116,7 @@ namespace MWWorld
             virtual void readEquipmentState (const MWWorld::ContainerStoreIterator& iter, int index, const ESM::InventoryState& inventory);
 
             bool canActorAutoEquip(const MWWorld::Ptr& actor, const MWWorld::Ptr& item);
+            ContainerStoreIterator findSlot (int slot) const;
 
         public:
 
@@ -155,6 +156,7 @@ namespace MWWorld
             /// \note if no item selected, return end() iterator
 
             ContainerStoreIterator getSlot (int slot);
+            ConstContainerStoreIterator getSlot(int slot) const;
 
             void unequipAll(const MWWorld::Ptr& actor);
             ///< Unequip all currently equipped items.
@@ -162,6 +164,9 @@ namespace MWWorld
             void autoEquip (const MWWorld::Ptr& actor);
             ///< Auto equip items according to stats and item value.
 
+            void autoEquipShield(const MWWorld::Ptr& actor);
+            ///< Auto-equip the shield with most health.
+
             const MWMechanics::MagicEffects& getMagicEffects() const;
             ///< Return magic effects from worn items.
 
@@ -169,10 +174,14 @@ namespace MWWorld
             ///< \attention This function is internal to the world model and should not be called from
             /// outside.
 
-            virtual bool stacks (const ConstPtr& ptr1, const ConstPtr& ptr2);
+            virtual bool stacks (const ConstPtr& ptr1, const ConstPtr& ptr2) const;
             ///< @return true if the two specified objects can stack with each other
 
+            virtual int remove(const std::string& itemId, int count, const Ptr& actor);
+            virtual int remove(const std::string& itemId, int count, const Ptr& actor, bool equipReplacement);
+
             virtual int remove(const Ptr& item, int count, const Ptr& actor);
+            virtual int remove(const Ptr& item, int count, const Ptr& actor, bool equipReplacement);
             ///< Remove \a count item(s) designated by \a item from this inventory.
             ///
             /// @return the number of items actually removed
