@@ -17,25 +17,18 @@ namespace MWWorld
     {
             std::list<std::pair<std::string, Ptr> > mScripts;
             std::list<std::pair<std::string, Ptr> >::iterator mIter;
-            MWWorld::ConstPtr mIgnore;
             const MWWorld::ESMStore& mStore;
 
         public:
 
             LocalScripts (const MWWorld::ESMStore& store);
 
-            void setIgnore (const ConstPtr& ptr);
-            ///< Mark a single reference for ignoring during iteration over local scripts (will revoke
-            /// previous ignores).
-
             void startIteration();
             ///< Set the iterator to the begin of the script list.
 
-            bool isFinished() const;
-            ///< Is iteration finished?
-
-            std::pair<std::string, Ptr> getNext();
-            ///< Get next local script (must not be called if isFinished())
+            bool getNext(std::pair<std::string, Ptr>& script);
+            ///< Get next local script
+            /// @return Did we get a script?
 
             void add (const std::string& scriptName, const Ptr& ptr);
             ///< Add script to collection of active local scripts.
