@@ -531,7 +531,7 @@ WeatherManager::WeatherManager(MWRender::RenderingManager& rendering, const Fall
     , mQueuedWeather(0)
     , mRegions()
     , mResult()
-    , mAmbientSound()
+    , mAmbientSound(nullptr)
     , mPlayingSoundID()
 {
     mTimeSettings.mNightStart = mSunsetTime + mSunsetDuration;
@@ -695,9 +695,9 @@ void WeatherManager::update(float duration, bool paused)
 
         double theta;
         if ( !is_night ) {
-            theta = M_PI * (adjustedHour - mSunriseTime) / dayDuration;
+            theta = static_cast<float>(osg::PI) * (adjustedHour - mSunriseTime) / dayDuration;
         } else {
-            theta = M_PI * (1.f - (adjustedHour - adjustedNightStart) / nightDuration);
+            theta = static_cast<float>(osg::PI) * (1.f - (adjustedHour - adjustedNightStart) / nightDuration);
         }
 
         osg::Vec3f final(
@@ -731,19 +731,21 @@ void WeatherManager::update(float duration, bool paused)
     {
         stopSounds();
         if (!mResult.mAmbientLoopSoundID.empty())
-            mAmbientSound = MWBase::Environment::get().getSoundManager()->playSound(mResult.mAmbientLoopSoundID, 1.0, 1.0, MWBase::SoundManager::Play_TypeSfx, MWBase::SoundManager::Play_Loop);
-
+            mAmbientSound = MWBase::Environment::get().getSoundManager()->playSound(
+                mResult.mAmbientLoopSoundID, mResult.mAmbientSoundVolume, 1.0,
+                MWSound::Type::Sfx, MWSound::PlayMode::Loop
+            );
         mPlayingSoundID = mResult.mAmbientLoopSoundID;
     }
-    if (mAmbientSound.get())
+    else if (mAmbientSound)
         mAmbientSound->setVolume(mResult.mAmbientSoundVolume);
 }
 
 void WeatherManager::stopSounds()
 {
-    if (mAmbientSound.get())
+    if (mAmbientSound)
         MWBase::Environment::get().getSoundManager()->stopSound(mAmbientSound);
-    mAmbientSound.reset();
+    mAmbientSound = nullptr;
     mPlayingSoundID.clear();
 }
 
@@ -1069,7 +1071,7 @@ inline void WeatherManager::calculateResult(const int weatherID, const float gam
         mResult.mSunDiscColor = lerp(osg::Vec4f(1,1,1,1), current.mSunDiscSunsetColor, factor);
         // The SunDiscSunsetColor in the INI isn't exactly the resulting color on screen, most likely because
         // MW applied the color to the ambient term as well. After the ambient and emissive terms are added together, the fixed pipeline
-        // would then clamp the total lighting to (1,1,1). A noticable change in color tone can be observed when only one of the color components gets clamped.
+        // would then clamp the total lighting to (1,1,1). A noticeable change in color tone can be observed when only one of the color components gets clamped.
         // Unfortunately that means we can't use the INI color as is, have to replicate the above nonsense.
         mResult.mSunDiscColor = mResult.mSunDiscColor + osg::componentMultiply(mResult.mSunDiscColor, mResult.mAmbientColor);
         for (int i=0; i<3; ++i)
