@@ -84,7 +84,7 @@ namespace ESM
         mData.mWeight = 0;
         mData.mValue = 0;
         mData.mIsScroll = 0;
-        mData.mSkillID = 0;
+        mData.mSkillId = 0;
         mData.mEnchant = 0;
         mName.clear();
         mModel.clear();
