@@ -6,6 +6,7 @@
 
 #include "cellref.hpp"
 #include "locals.hpp"
+#include "animationstate.hpp"
 
 namespace ESM
 {
@@ -31,6 +32,8 @@ namespace ESM
 
         unsigned int mVersion;
 
+        ESM::AnimationState mAnimationState;
+
         ObjectState() : mHasCustomState(true), mVersion(0)
         {}
 
