@@ -16,6 +16,45 @@
 namespace ESMTerrain
 {
 
+    class LandCache
+    {
+    public:
+        typedef std::map<std::pair<int, int>, osg::ref_ptr<const LandObject> > Map;
+        Map mMap;
+    };
+
+    LandObject::LandObject()
+    {
+    }
+
+    LandObject::LandObject(const ESM::Land *land, int loadFlags)
+        : mLand(land)
+        , mLoadFlags(loadFlags)
+    {
+        mLand->loadData(mLoadFlags, &mData);
+    }
+
+    LandObject::LandObject(const LandObject &copy, const osg::CopyOp &copyop)
+    {
+    }
+
+    LandObject::~LandObject()
+    {
+    }
+
+    const ESM::Land::LandData *LandObject::getData(int flags) const
+    {
+        if ((mData.mDataLoaded & flags) != flags)
+            return NULL;
+        return &mData;
+    }
+
+    int LandObject::getPlugin() const
+    {
+        return mLand->mPlugin;
+    }
+
+
     const float defaultHeight = ESM::Land::DEFAULT_HEIGHT;
 
     Storage::Storage(const VFS::Manager *vfs, const std::string& normalMapPattern, const std::string& normalHeightMapPattern, bool autoUseNormalMaps, const std::string& specularMapPattern, bool autoUseSpecularMaps)
@@ -28,20 +67,10 @@ namespace ESMTerrain
     {
     }
 
-    const ESM::Land::LandData *Storage::getLandData (int cellX, int cellY, int flags)
-    {
-        if (const ESM::Land *land = getLand (cellX, cellY))
-            return land->getLandData (flags);
-
-        return 0;
-    }
-
     bool Storage::getMinMaxHeights(float size, const osg::Vec2f &center, float &min, float &max)
     {
         assert (size <= 1 && "Storage::getMinMaxHeights, chunk size should be <= 1 cell");
 
-        /// \todo investigate if min/max heights should be stored at load time in ESM::Land instead
-
         osg::Vec2f origin = center - osg::Vec2f(size/2.f, size/2.f);
 
         int cellX = static_cast<int>(std::floor(origin.x()));
@@ -53,7 +82,9 @@ namespace ESMTerrain
         int endRow = startRow + size * (ESM::Land::LAND_SIZE-1) + 1;
         int endColumn = startColumn + size * (ESM::Land::LAND_SIZE-1) + 1;
 
-        if (const ESM::Land::LandData *data = getLandData (cellX, cellY, ESM::Land::DATA_VHGT))
+        osg::ref_ptr<const LandObject> land = getLand (cellX, cellY);
+        const ESM::Land::LandData* data = land ? land->getData(ESM::Land::DATA_VHGT) : 0;
+        if (data)
         {
             min = std::numeric_limits<float>::max();
             max = -std::numeric_limits<float>::max();
@@ -73,10 +104,10 @@ namespace ESMTerrain
 
         min = defaultHeight;
         max = defaultHeight;
-        return true;
+        return false;
     }
 
-    void Storage::fixNormal (osg::Vec3f& normal, int cellX, int cellY, int col, int row)
+    void Storage::fixNormal (osg::Vec3f& normal, int cellX, int cellY, int col, int row, LandCache& cache)
     {
         while (col >= ESM::Land::LAND_SIZE-1)
         {
@@ -99,7 +130,9 @@ namespace ESMTerrain
             row += ESM::Land::LAND_SIZE-1;
         }
 
-        if (const ESM::Land::LandData *data = getLandData (cellX, cellY, ESM::Land::DATA_VNML))
+        const LandObject* land = getLand(cellX, cellY, cache);
+        const ESM::Land::LandData* data = land ? land->getData(ESM::Land::DATA_VNML) : 0;
+        if (data)
         {
             normal.x() = data->mNormals[col*ESM::Land::LAND_SIZE*3+row*3];
             normal.y() = data->mNormals[col*ESM::Land::LAND_SIZE*3+row*3+1];
@@ -110,18 +143,18 @@ namespace ESMTerrain
             normal = osg::Vec3f(0,0,1);
     }
 
-    void Storage::averageNormal(osg::Vec3f &normal, int cellX, int cellY, int col, int row)
+    void Storage::averageNormal(osg::Vec3f &normal, int cellX, int cellY, int col, int row, LandCache& cache)
     {
         osg::Vec3f n1,n2,n3,n4;
-        fixNormal(n1, cellX, cellY, col+1, row);
-        fixNormal(n2, cellX, cellY, col-1, row);
-        fixNormal(n3, cellX, cellY, col, row+1);
-        fixNormal(n4, cellX, cellY, col, row-1);
+        fixNormal(n1, cellX, cellY, col+1, row, cache);
+        fixNormal(n2, cellX, cellY, col-1, row, cache);
+        fixNormal(n3, cellX, cellY, col, row+1, cache);
+        fixNormal(n4, cellX, cellY, col, row-1, cache);
         normal = (n1+n2+n3+n4);
         normal.normalize();
     }
 
-    void Storage::fixColour (osg::Vec4f& color, int cellX, int cellY, int col, int row)
+    void Storage::fixColour (osg::Vec4f& color, int cellX, int cellY, int col, int row, LandCache& cache)
     {
         if (col == ESM::Land::LAND_SIZE-1)
         {
@@ -134,7 +167,9 @@ namespace ESMTerrain
             row = 0;
         }
 
-        if (const ESM::Land::LandData *data = getLandData (cellX, cellY, ESM::Land::DATA_VCLR))
+        const LandObject* land = getLand(cellX, cellY, cache);
+        const ESM::Land::LandData* data = land ? land->getData(ESM::Land::DATA_VCLR) : 0;
+        if (data)
         {
             color.r() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3] / 255.f;
             color.g() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3+1] / 255.f;
@@ -146,7 +181,6 @@ namespace ESMTerrain
             color.g() = 1;
             color.b() = 1;
         }
-
     }
 
     void Storage::fillVertexBuffers (int lodLevel, float size, const osg::Vec2f& center,
@@ -155,7 +189,7 @@ namespace ESMTerrain
                                             osg::ref_ptr<osg::Vec4Array> colours)
     {
         // LOD level n means every 2^n-th vertex is kept
-        size_t increment = 1 << lodLevel;
+        size_t increment = static_cast<size_t>(1) << lodLevel;
 
         osg::Vec2f origin = center - osg::Vec2f(size/2.f, size/2.f);
 
@@ -174,15 +208,24 @@ namespace ESMTerrain
         float vertY = 0;
         float vertX = 0;
 
+        LandCache cache;
+
         float vertY_ = 0; // of current cell corner
         for (int cellY = startCellY; cellY < startCellY + std::ceil(size); ++cellY)
         {
             float vertX_ = 0; // of current cell corner
             for (int cellX = startCellX; cellX < startCellX + std::ceil(size); ++cellX)
             {
-                const ESM::Land::LandData *heightData = getLandData (cellX, cellY, ESM::Land::DATA_VHGT);
-                const ESM::Land::LandData *normalData = getLandData (cellX, cellY, ESM::Land::DATA_VNML);
-                const ESM::Land::LandData *colourData = getLandData (cellX, cellY, ESM::Land::DATA_VCLR);
+                const LandObject* land = getLand(cellX, cellY, cache);
+                const ESM::Land::LandData *heightData = 0;
+                const ESM::Land::LandData *normalData = 0;
+                const ESM::Land::LandData *colourData = 0;
+                if (land)
+                {
+                    heightData = land->getData(ESM::Land::DATA_VHGT);
+                    normalData = land->getData(ESM::Land::DATA_VNML);
+                    colourData = land->getData(ESM::Land::DATA_VCLR);
+                }
 
                 int rowStart = 0;
                 int colStart = 0;
@@ -197,8 +240,8 @@ namespace ESMTerrain
                 // Only relevant for chunks smaller than (contained in) one cell
                 rowStart += (origin.x() - startCellX) * ESM::Land::LAND_SIZE;
                 colStart += (origin.y() - startCellY) * ESM::Land::LAND_SIZE;
-                int rowEnd = rowStart + std::min(1.f, size) * (ESM::Land::LAND_SIZE-1) + 1;
-                int colEnd = colStart + std::min(1.f, size) * (ESM::Land::LAND_SIZE-1) + 1;
+                int rowEnd = std::min(static_cast<int>(rowStart + std::min(1.f, size) * (ESM::Land::LAND_SIZE-1) + 1), static_cast<int>(ESM::Land::LAND_SIZE));
+                int colEnd = std::min(static_cast<int>(colStart + std::min(1.f, size) * (ESM::Land::LAND_SIZE-1) + 1), static_cast<int>(ESM::Land::LAND_SIZE));
 
                 vertY = vertY_;
                 for (int col=colStart; col<colEnd; col += increment)
@@ -235,11 +278,11 @@ namespace ESMTerrain
 
                         // Normals apparently don't connect seamlessly between cells
                         if (col == ESM::Land::LAND_SIZE-1 || row == ESM::Land::LAND_SIZE-1)
-                            fixNormal(normal, cellX, cellY, col, row);
+                            fixNormal(normal, cellX, cellY, col, row, cache);
 
                         // some corner normals appear to be complete garbage (z < 0)
                         if ((row == 0 || row == ESM::Land::LAND_SIZE-1) && (col == 0 || col == ESM::Land::LAND_SIZE-1))
-                            averageNormal(normal, cellX, cellY, col, row);
+                            averageNormal(normal, cellX, cellY, col, row, cache);
 
                         assert(normal.z() > 0);
 
@@ -259,7 +302,7 @@ namespace ESMTerrain
 
                         // Unlike normals, colors mostly connect seamlessly between cells, but not always...
                         if (col == ESM::Land::LAND_SIZE-1 || row == ESM::Land::LAND_SIZE-1)
-                            fixColour(color, cellX, cellY, col, row);
+                            fixColour(color, cellX, cellY, col, row, cache);
 
                         color.a() = 1;
 
@@ -279,7 +322,7 @@ namespace ESMTerrain
     }
 
     Storage::UniqueTextureId Storage::getVtexIndexAt(int cellX, int cellY,
-                                           int x, int y)
+                                           int x, int y, LandCache& cache)
     {
         // For the first/last row/column, we need to get the texture from the neighbour cell
         // to get consistent blending at the borders
@@ -289,7 +332,12 @@ namespace ESMTerrain
             --cellX;
             x += ESM::Land::LAND_TEXTURE_SIZE;
         }
-        if (y >= ESM::Land::LAND_TEXTURE_SIZE) // Y appears to be wrapped from the other side because why the hell not?
+        while (x >= ESM::Land::LAND_TEXTURE_SIZE)
+        {
+            ++cellX;
+            x -= ESM::Land::LAND_TEXTURE_SIZE;
+        }
+        while (y >= ESM::Land::LAND_TEXTURE_SIZE) // Y appears to be wrapped from the other side because why the hell not?
         {
             ++cellY;
             y -= ESM::Land::LAND_TEXTURE_SIZE;
@@ -298,15 +346,17 @@ namespace ESMTerrain
         assert(x<ESM::Land::LAND_TEXTURE_SIZE);
         assert(y<ESM::Land::LAND_TEXTURE_SIZE);
 
-        if (const ESM::Land::LandData *data = getLandData (cellX, cellY, ESM::Land::DATA_VTEX))
+        const LandObject* land = getLand(cellX, cellY, cache);
+
+        const ESM::Land::LandData *data = land ? land->getData(ESM::Land::DATA_VTEX) : 0;
+        if (data)
         {
             int tex = data->mTextures[y * ESM::Land::LAND_TEXTURE_SIZE + x];
             if (tex == 0)
                 return std::make_pair(0,0); // vtex 0 is always the base texture, regardless of plugin
-            return std::make_pair(tex, getLand (cellX, cellY)->mPlugin);
+            return std::make_pair(tex, land->getPlugin());
         }
-        else
-            return std::make_pair(0,0);
+        return std::make_pair(0,0);
     }
 
     std::string Storage::getTextureName(UniqueTextureId id)
@@ -319,7 +369,7 @@ namespace ESMTerrain
         const ESM::LandTexture* ltex = getLandTexture(id.first-1, id.second);
         if (!ltex)
         {
-            std::cerr << "Unable to find land texture index " << id.first-1 << " in plugin " << id.second << ", using default texture instead" << std::endl;
+            std::cerr << "Warning: Unable to find land texture index " << id.first-1 << " in plugin " << id.second << ", using default texture instead" << std::endl;
             return defaultTexture;
         }
 
@@ -332,10 +382,6 @@ namespace ESMTerrain
     void Storage::getBlendmaps(float chunkSize, const osg::Vec2f &chunkCenter,
         bool pack, ImageVector &blendmaps, std::vector<Terrain::LayerInfo> &layerList)
     {
-        // TODO - blending isn't completely right yet; the blending radius appears to be
-        // different at a cell transition (2 vertices, not 4), so we may need to create a larger blendmap
-        // and interpolate the rest of the cell by hand? :/
-
         osg::Vec2f origin = chunkCenter - osg::Vec2f(chunkSize/2.f, chunkSize/2.f);
         int cellX = static_cast<int>(std::floor(origin.x()));
         int cellY = static_cast<int>(std::floor(origin.y()));
@@ -347,10 +393,6 @@ namespace ESMTerrain
         int rowEnd = rowStart + chunkSize * (realTextureSize-1) + 1;
         int colEnd = colStart + chunkSize * (realTextureSize-1) + 1;
 
-        assert (rowStart >= 0 && colStart >= 0);
-        assert (rowEnd <= realTextureSize);
-        assert (colEnd <= realTextureSize);
-
         // Save the used texture indices so we know the total number of textures
         // and number of required blend maps
         std::set<UniqueTextureId> textureIndices;
@@ -360,10 +402,12 @@ namespace ESMTerrain
         // So we're always adding _land_default.dds as the base layer here, even if it's not referenced in this cell.
         textureIndices.insert(std::make_pair(0,0));
 
+        LandCache cache;
+
         for (int y=colStart; y<colEnd; ++y)
             for (int x=rowStart; x<rowEnd; ++x)
             {
-                UniqueTextureId id = getVtexIndexAt(cellX, cellY, x, y);
+                UniqueTextureId id = getVtexIndexAt(cellX, cellY, x, y, cache);
                 textureIndices.insert(id);
             }
 
@@ -399,7 +443,7 @@ namespace ESMTerrain
             {
                 for (int x=0; x<blendmapSize; ++x)
                 {
-                    UniqueTextureId id = getVtexIndexAt(cellX, cellY, x+rowStart, y+colStart);
+                    UniqueTextureId id = getVtexIndexAt(cellX, cellY, x+rowStart, y+colStart, cache);
                     assert(textureIndicesMap.find(id) != textureIndicesMap.end());
                     int layerIndex = textureIndicesMap.find(id)->second;
                     int blendIndex = (pack ? static_cast<int>(std::floor((layerIndex - 1) / 4.f)) : layerIndex - 1);
@@ -421,8 +465,12 @@ namespace ESMTerrain
         int cellX = static_cast<int>(std::floor(worldPos.x() / 8192.f));
         int cellY = static_cast<int>(std::floor(worldPos.y() / 8192.f));
 
-        const ESM::Land* land = getLand(cellX, cellY);
-        if (!land || !(land->mDataTypes&ESM::Land::DATA_VHGT))
+        osg::ref_ptr<const LandObject> land = getLand(cellX, cellY);
+        if (!land)
+            return defaultHeight;
+
+        const ESM::Land::LandData* data = land->getData(ESM::Land::DATA_VHGT);
+        if (!data)
             return defaultHeight;
 
         // Mostly lifted from Ogre::Terrain::getHeightAtTerrainPosition
@@ -461,10 +509,10 @@ namespace ESMTerrain
         */
 
         // Build all 4 positions in normalized cell space, using point-sampled height
-        osg::Vec3f v0 (startXTS, startYTS, getVertexHeight(land, startX, startY) / 8192.f);
-        osg::Vec3f v1 (endXTS, startYTS, getVertexHeight(land, endX, startY) / 8192.f);
-        osg::Vec3f v2 (endXTS, endYTS, getVertexHeight(land, endX, endY) / 8192.f);
-        osg::Vec3f v3 (startXTS, endYTS, getVertexHeight(land, startX, endY) / 8192.f);
+        osg::Vec3f v0 (startXTS, startYTS, getVertexHeight(data, startX, startY) / 8192.f);
+        osg::Vec3f v1 (endXTS, startYTS, getVertexHeight(data, endX, startY) / 8192.f);
+        osg::Vec3f v2 (endXTS, endYTS, getVertexHeight(data, endX, endY) / 8192.f);
+        osg::Vec3f v3 (startXTS, endYTS, getVertexHeight(data, startX, endY) / 8192.f);
         // define this plane in terrain space
         osg::Plane plane;
         // FIXME: deal with differing triangle alignment
@@ -496,11 +544,23 @@ namespace ESMTerrain
 
     }
 
-    float Storage::getVertexHeight(const ESM::Land *land, int x, int y)
+    float Storage::getVertexHeight(const ESM::Land::LandData* data, int x, int y)
     {
         assert(x < ESM::Land::LAND_SIZE);
         assert(y < ESM::Land::LAND_SIZE);
-        return land->getLandData()->mHeights[y * ESM::Land::LAND_SIZE + x];
+        return data->mHeights[y * ESM::Land::LAND_SIZE + x];
+    }
+
+    const LandObject* Storage::getLand(int cellX, int cellY, LandCache& cache)
+    {
+        LandCache::Map::iterator found = cache.mMap.find(std::make_pair(cellX, cellY));
+        if (found != cache.mMap.end())
+            return found->second;
+        else
+        {
+            found = cache.mMap.insert(std::make_pair(std::make_pair(cellX, cellY), getLand(cellX, cellY))).first;
+            return found->second;
+        }
     }
 
     Terrain::LayerInfo Storage::getLayerInfo(const std::string& texture)
@@ -570,4 +630,9 @@ namespace ESMTerrain
         return ESM::Land::LAND_SIZE;
     }
 
+    int Storage::getBlendmapScale(float chunkSize)
+    {
+        return ESM::Land::LAND_TEXTURE_SIZE*chunkSize;
+    }
+
 }
