@@ -1,18 +1,12 @@
 #include "configurationmanager.hpp"
 
-#include <string>
 #include <iostream>
-#include <algorithm>
-#include <ctype.h>
 
 #include <components/files/escape.hpp>
 
-#include <boost/bind.hpp>
 #include <boost/algorithm/string/erase.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/filesystem/fstream.hpp>
-#include <boost/iostreams/filtering_stream.hpp>
-
 /**
  * \namespace Files
  */
@@ -81,8 +75,6 @@ void ConfigurationManager::processPaths(Files::PathContainer& dataDirs, bool cre
     for (Files::PathContainer::iterator it = dataDirs.begin(); it != dataDirs.end(); ++it)
     {
         path = it->string();
-        boost::erase_all(path, "\"");
-        *it = boost::filesystem::path(path);
 
         // Check if path contains a token
         if (!path.empty() && *path.begin() == '?')
@@ -130,7 +122,7 @@ void ConfigurationManager::processPaths(Files::PathContainer& dataDirs, bool cre
     }
 
     dataDirs.erase(std::remove_if(dataDirs.begin(), dataDirs.end(),
-        boost::bind(&boost::filesystem::path::empty, _1)), dataDirs.end());
+        std::bind(&boost::filesystem::path::empty, std::placeholders::_1)), dataDirs.end());
 }
 
 bool ConfigurationManager::loadConfig(const boost::filesystem::path& path,
