@@ -2,8 +2,7 @@
 #define OPENMW_CONSTRAINEDFILESTREAM_H
 
 #include <istream>
-
-#include <boost/shared_ptr.hpp>
+#include <memory>
 
 namespace Files
 {
@@ -17,7 +16,7 @@ public:
     virtual ~ConstrainedFileStream();
 };
 
-typedef boost::shared_ptr<std::istream> IStreamPtr;
+typedef std::shared_ptr<std::istream> IStreamPtr;
 
 IStreamPtr openConstrainedFileStream(const char *filename, size_t start=0, size_t length=0xFFFFFFFF);
 
