@@ -8,6 +8,12 @@ namespace VFS
     class Manager;
 }
 
+namespace osg
+{
+    class Stats;
+    class State;
+}
+
 namespace Resource
 {
     class ObjectCache;
@@ -23,11 +29,18 @@ namespace Resource
         /// Clear cache entries that have not been referenced for longer than expiryDelay.
         virtual void updateCache(double referenceTime);
 
+        /// Clear all cache entries.
+        virtual void clearCache();
+
         /// How long to keep objects in cache after no longer being referenced.
         void setExpiryDelay (double expiryDelay);
 
         const VFS::Manager* getVFS() const;
 
+        virtual void reportStats(unsigned int frameNumber, osg::Stats* stats) const {}
+
+        virtual void releaseGLObjects(osg::State* state);
+
     protected:
         const VFS::Manager* mVFS;
         osg::ref_ptr<Resource::ObjectCache> mCache;
