@@ -5,6 +5,7 @@
 
 #include <osg/NodeVisitor>
 #include <osg/Group>
+#include <osg/Geometry>
 #include <osg/FrontFace>
 #include <osg/PositionAttitudeTransform>
 #include <osg/MatrixTransform>
@@ -29,14 +30,31 @@ namespace SceneUtil
             mFilter2 = "tri " + mFilter;
         }
 
+        virtual void apply(osg::MatrixTransform& node)
+        {
+            traverse(node);
+        }
         virtual void apply(osg::Node& node)
         {
-            std::string lowerName = Misc::StringUtils::lowerCase(node.getName());
+            traverse(node);
+        }
+        virtual void apply(osg::Group& node)
+        {
+            traverse(node);
+        }
+
+        virtual void apply(osg::Drawable& drawable)
+        {
+            std::string lowerName = Misc::StringUtils::lowerCase(drawable.getName());
             if ((lowerName.size() >= mFilter.size() && lowerName.compare(0, mFilter.size(), mFilter) == 0)
                     || (lowerName.size() >= mFilter2.size() && lowerName.compare(0, mFilter2.size(), mFilter2) == 0))
-                mToCopy.push_back(&node);
-            else
-                traverse(node);
+            {
+                osg::Node* node = &drawable;
+                while (node && node->getNumParents() && !node->getStateSet())
+                    node = node->getParent(0);
+                if (node)
+                    mToCopy.push_back(node);
+            }
         }
 
         void doCopy()
@@ -45,7 +63,7 @@ namespace SceneUtil
             {
                 osg::ref_ptr<osg::Node> node = *it;
                 if (node->getNumParents() > 1)
-                    std::cerr << "CopyRigVisitor warning: node has multiple parents" << std::endl;
+                    std::cerr << "Error CopyRigVisitor: node has multiple parents" << std::endl;
                 while (node->getNumParents())
                     node->getParent(0)->removeChild(node);
 
@@ -63,7 +81,18 @@ namespace SceneUtil
         std::string mFilter2;
     };
 
-    osg::ref_ptr<osg::Node> attach(osg::ref_ptr<osg::Node> toAttach, osg::Node *master, const std::string &filter, const std::string &attachNode)
+    void mergeUserData(osg::UserDataContainer* source, osg::Object* target)
+    {
+        if (!target->getUserDataContainer())
+            target->setUserDataContainer(source);
+        else
+        {
+            for (unsigned int i=0; i<source->getNumUserObjects(); ++i)
+                target->getUserDataContainer()->addUserObject(source->getUserObject(i));
+        }
+    }
+
+    osg::ref_ptr<osg::Node> attach(osg::ref_ptr<osg::Node> toAttach, osg::Node *master, const std::string &filter, osg::Group* attachNode)
     {
         if (dynamic_cast<SceneUtil::Skeleton*>(toAttach.get()))
         {
@@ -78,21 +107,18 @@ namespace SceneUtil
                 osg::ref_ptr<osg::Node> newHandle = handle->getChild(0);
                 handle->removeChild(newHandle);
                 master->asGroup()->addChild(newHandle);
+                mergeUserData(toAttach->getUserDataContainer(), newHandle);
                 return newHandle;
             }
             else
             {
                 master->asGroup()->addChild(handle);
+                handle->setUserDataContainer(toAttach->getUserDataContainer());
                 return handle;
             }
         }
         else
         {
-            FindByNameVisitor find(attachNode);
-            master->accept(find);
-            if (!find.mFoundNode)
-                throw std::runtime_error(std::string("Can't find attachment node ") + attachNode);
-
             FindByNameVisitor findBoneOffset("BoneOffset");
             toAttach->accept(findBoneOffset);
 
@@ -108,9 +134,13 @@ namespace SceneUtil
                 trans->setPosition(boneOffset->getMatrix().getTrans());
                 // The BoneOffset rotation seems to be incorrect
                 trans->setAttitude(osg::Quat(osg::DegreesToRadians(-90.f), osg::Vec3f(1,0,0)));
+
+                // Now that we used it, get rid of the redundant node.
+                if (boneOffset->getNumChildren() == 0 && boneOffset->getNumParents() == 1)
+                    boneOffset->getParent(0)->removeChild(boneOffset);
             }
 
-            if (attachNode.find("Left") != std::string::npos)
+            if (attachNode->getName().find("Left") != std::string::npos)
             {
                 if (!trans)
                     trans = new osg::PositionAttitudeTransform;
@@ -132,13 +162,13 @@ namespace SceneUtil
 
             if (trans)
             {
-                find.mFoundNode->addChild(trans);
+                attachNode->addChild(trans);
                 trans->addChild(toAttach);
                 return trans;
             }
             else
             {
-                find.mFoundNode->addChild(toAttach);
+                attachNode->addChild(toAttach);
                 return toAttach;
             }
         }
