@@ -13,10 +13,9 @@ namespace SceneUtil
     /// @brief Mesh skinning implementation.
     /// @note A RigGeometry may be attached directly to a Skeleton, or somewhere below a Skeleton.
     /// Note though that the RigGeometry ignores any transforms below the Skeleton, so the attachment point is not that important.
-    /// @note To avoid race conditions, the rig geometry needs to be double buffered. This can be done
-    /// using a FrameSwitch node that has two RigGeometry children. In the future we may want to consider implementing
-    /// the double buffering inside RigGeometry.
-    class RigGeometry : public osg::Geometry
+    /// @note The internal Geometry used for rendering is double buffered, this allows updates to be done in a thread safe way while
+    /// not compromising rendering performance. This is crucial when using osg's default threading model of DrawThreadPerContext.
+    class RigGeometry : public osg::Drawable
     {
     public:
         RigGeometry();
@@ -24,6 +23,9 @@ namespace SceneUtil
 
         META_Object(SceneUtil, RigGeometry)
 
+        // At this point compileGLObjects() remains unimplemented, hard to avoid race conditions
+        // and there is limited value in compiling anyway since the data will change again for the next frame
+
         struct BoneInfluence
         {
             osg::Matrixf mInvBindMatrix;
@@ -45,19 +47,22 @@ namespace SceneUtil
 
         osg::ref_ptr<osg::Geometry> getSourceGeometry();
 
-        // Called automatically by our CullCallback
-        void update(osg::NodeVisitor* nv);
-
-        // Called automatically by our UpdateCallback
-        void updateBounds(osg::NodeVisitor* nv);
+        virtual void accept(osg::NodeVisitor &nv);
+        virtual bool supports(const osg::PrimitiveFunctor&) const { return true; }
+        virtual void accept(osg::PrimitiveFunctor&) const;
 
     private:
+        void cull(osg::NodeVisitor* nv);
+        void updateBounds(osg::NodeVisitor* nv);
+
+        osg::ref_ptr<osg::Geometry> mGeometry[2];
+        osg::Geometry* getGeometry(unsigned int frame) const;
+
         osg::ref_ptr<osg::Geometry> mSourceGeometry;
-        osg::ref_ptr<osg::Vec4Array> mSourceTangents;
+        osg::ref_ptr<const osg::Vec4Array> mSourceTangents;
         Skeleton* mSkeleton;
 
-        osg::NodePath mSkelToGeomPath;
-        osg::Matrixf mGeomToSkelMatrix;
+        osg::ref_ptr<osg::RefMatrix> mGeomToSkelMatrix;
 
         osg::ref_ptr<InfluenceMap> mInfluenceMap;
 
