@@ -4,11 +4,6 @@
 #include <osg/ref_ptr>
 #include <osg/Referenced>
 
-namespace osg
-{
-    class Object;
-}
-
 namespace SceneUtil
 {
     class WorkQueue;
@@ -22,12 +17,14 @@ namespace SceneUtil
         UnrefQueue();
 
         /// Adds an object to the list of objects to be unreferenced. Call from the main thread.
-        void push(const osg::Object* obj);
+        void push(const osg::Referenced* obj);
 
         /// Adds a WorkItem to the given WorkQueue that will clear the list of objects in a worker thread, thus unreferencing them.
         /// Call from the main thread.
         void flush(SceneUtil::WorkQueue* workQueue);
 
+        unsigned int getNumItems() const;
+
     private:
         osg::ref_ptr<UnrefWorkItem> mWorkItem;
     };
