@@ -16,19 +16,30 @@ namespace Gui
     public:
         MyGUI::IntSize getRequestedSize();
 
+        ImageButton();
+
         /// Set mImageNormal, mImageHighlighted and mImagePushed based on file convention (image_idle.ext, image_over.ext and image_pressed.ext)
         void setImage(const std::string& image);
 
+    private:
+        void updateImage();
+
     protected:
         virtual void setPropertyOverride(const std::string& _key, const std::string& _value);
         virtual void onMouseLostFocus(MyGUI::Widget* _new);
         virtual void onMouseSetFocus(MyGUI::Widget* _old);
         virtual void onMouseButtonPressed(int _left, int _top, MyGUI::MouseButton _id);
         virtual void onMouseButtonReleased(int _left, int _top, MyGUI::MouseButton _id);
+        virtual void onKeySetFocus(MyGUI::Widget* _old);
+        virtual void onKeyLostFocus(MyGUI::Widget* _new);
 
         std::string mImageHighlighted;
         std::string mImageNormal;
         std::string mImagePushed;
+
+        bool mMouseFocus;
+        bool mMousePress;
+        bool mKeyFocus;
     };
 
 }
